Package: ggplotplus
Title: Universal Design-Oriented Enhancements for 'ggplot2'
Version: 0.5.5
Authors@R: 
    person("Alex", "Bajcz", email = "bajcz003@umn.edu", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-5909-4676"))
BugReports: https://github.com/MAISRC/ggplotplus/issues
Description: A collection of enhancements to 'ggplot2', with a focus on creating Universally Designed, accessible graphs easily and quickly.
Depends: R (>= 4.1.0)
License: MIT + file LICENSE
Encoding: UTF-8
URL: https://github.com/MAISRC/ggplotplus
RoxygenNote: 7.3.3
Language: en-US
Imports: ggplot2 (>= 4.0.1), rlang, S7, grid, gtable, scales,
        viridisLite, polyclip, ggrepel (>= 0.9.0), dplyr, stats
Suggests: testthat (>= 3.0.0), ragg (>= 1.5.2), patchwork, cowplot
Config/testthat/edition: 3
LazyData: true
NeedsCompilation: no
Packaged: 2026-06-09 19:53:30 UTC; Bajcz
Author: Alex Bajcz [aut, cre] (ORCID: <https://orcid.org/0000-0002-5909-4676>)
Maintainer: Alex Bajcz <bajcz003@umn.edu>
Repository: CRAN
Date/Publication: 2026-06-17 13:40:08 UTC

# Generated by roxygen2: do not edit by hand

export(add_shape_plus)
export(direct_labels_plus)
export(geom_jitter_plus)
export(geom_point_plus)
export(geom_point_plus_shapes)
export(ggplotplus_to_cowplot)
export(ggplotplus_to_patchwork)
export(gridlines_plus)
export(scale_continuous_plus)
export(theme_plus)
export(yaxis_title_plus)
import(ggplot2)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,slice_max)
importFrom(dplyr,slice_min)
importFrom(dplyr,ungroup)
importFrom(ggplot2,class_ggplot)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,ggplot_gtable)
importFrom(ggplot2,update_ggplot)
importFrom(polyclip,polyclip)
importFrom(rlang,"%||%")
importFrom(rlang,.data)
importFrom(rlang,list2)
importFrom(scales,pal_viridis)
importFrom(scales,pretty_breaks)
importFrom(stats,median)
importFrom(utils,globalVariables)
importFrom(viridisLite,viridis)

# README
Dr. Alex Bajcz, Quantitative Ecologist, Minnesota Aquatic Invasive
Species Research Center (MAISRC)
2026-06-29

<img src="man/figures/ggplotplus_hex.png" alt="ggplotplus hex sticker" style="float: right; width: 180px;"/>

# *ggplotplus* – Universal Design additions for *ggplot2*

<!-- badges: start -->

[![CRAN
version](https://img.shields.io/cran/v/ggplotplus.svg)](https://CRAN.R-project.org/package=ggplotplus)
[![R-CMD-check](https://github.com/MAISRC/ggplotplus/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/MAISRC/ggplotplus/actions/workflows/R-CMD-check.yaml)
[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
<!-- badges: end -->

## What’s `ggplotplus` for?

`ggplot2` is an *extraordinarily* powerful graphics package, used by
perhaps millions to produce graphs for posters, presentations, papers,
and exploration. `ggplotplus` enhances `ggplot2` by:

1.  Providing tools that override many of `ggplot2`’s
    less-than-fully-accessible default design aspects so creating graphs
    that adhere to design best practices is easier and quicker.

2.  Enabling some design options–such as custom point shapes in
    scatterplots, direct group labeling, and relocation of the y-axis
    title–that are currently impossible via a single function call in
    `ggplot2`.

3.  Attempting to lower the learning curve and bandwidth required to
    make high-quality, Universal-Design-oriented graphs, ready for
    publication and sharing, using `ggplot2`.

In particular, `ggplotplus` is for anyone who wants or needs to use
`ggplot2` but has limited time, skill, energy, patience, experience,
etc. with which to devote to learning its many nuances. However, given
its evidence-based approach to graph design, the package is for everyone
who wants to build defensible, maximally consumable graphs quickly and
with a fraction of the fiddly customization often required to get a
typical ggplot up to the most exacting design standards. Plus,
`ggplotplus` has a few tricks to offer that, at present, aren’t
available via a single function call anywhere else–read on for details!

## Quick Start Guide

To begin using `ggplotplus`, first install the latest development
version from GitHub using the pak package:

``` r
# install.packages("devtools")  #IF NOT ALREADY INSTALLED
pak::pak("MAISRC/ggplotplus")
```

Or, alternatively, install the latest stable version from CRAN:

``` r
install.packages("ggplotplus")
```

Then, load it alongside `ggplot2`:

``` r
# install.packages("ggplot2")  #IF NOT ALREADY INSTALLED
library(ggplot2)
library(ggplotplus)
```

Once loaded, you can start layering its `_plus()` tools onto your
`ggplot2` calls to improve their design quickly and with minimal effort!

### Using the tools

Here’s a basic, default `ggplot2` as a point of comparison:

``` r
ggplot(iris, 
       mapping = aes(x = Petal.Length, 
                     y = Sepal.Length)) +
  geom_point(mapping = aes(color = Species)) 
```

![](README_files/figure-commonmark/point%20of%20comparison-1.png)

The biggest single change you can make to the design of your graphs via
`ggplotplus` is via its `theme_plus()` function, which overrides a suite
of `ggplot2`’s default design choices related to default color palettes,
common geometry layers, and layout parameters:

``` r
ggplot(iris, 
       mapping = aes(x = Petal.Length, 
                     y = Sepal.Length)) +
  geom_point(mapping = aes(color = Species)) + 
  theme_plus() #<--OVERHAULS THEME, GEOM, AND PALETTE RELATED DEFAULTS IN MANY WAYS. 
```

![](README_files/figure-commonmark/quick%20start%20use-1.png)

Do a quick comparison between this graph and the last one; `ggplotplus`
adjusts nearly every aspect’s of a ggplot’s design in an effort to make
them cleaner and more accessible.

### Taking it further

If you want to add thoughtful gridlines, polish the appearance of your
continuous scales, and/or move your y-axis title to somewhere more
thoughtful and easier to read, add `gridlines_plus()`,
`scale_continuous_plus()`, and `yaxis_title_plus()` to your calls,
respective:

``` r
ggplot(iris, 
       mapping = aes(x = Petal.Length, 
                     y = Sepal.Length)) +
  geom_point(mapping = aes(color = Species)) + 
  theme_plus() +
  scale_continuous_plus( #<--OVERHAULS AXIS BREAKS AND LIMITS (FOR CONTINUOUS SCALES ONLY!)
    scale = "x",
    name = "Petal length (cm)",
    thin.labels = TRUE) +  
  scale_continuous_plus(
    scale = "y", 
    name = "Sepal length (cm)") +
  yaxis_title_plus() + #<--RELOCATES AND RE-ORIENTS Y AXIS TITLE.
  gridlines_plus() + #<--ADDS THOUGHTFUL GRIDLINES, IF YOU *REALLY* WANT THEM.
  labs(color = expression(italic("Iris")*" species")) #<--THIS IS BASE GGPLOT2, BUT A NICE TOUCH!
```

![](README_files/figure-commonmark/even%20more%20tools-1.png)

Notice how, in this version, gridlines are subtle but still legible,
axes are more fully labelled, and the y-axis title is much easier to
spot and read.

## Thinking differently about graph design

Rather than using a legend at all, you can use `direct_labels_plus()` to
label the groups directly within the plotting area:

``` r
ggplot(iris, 
       mapping = aes(x = Petal.Length, 
                     y = Sepal.Length)) +
  geom_point(mapping = aes(color = Species)) + 
  theme_plus() +
  scale_continuous_plus(
    scale = "x",
    name = "Petal length (cm)",
    thin.labels = TRUE) +  
  scale_continuous_plus(
    scale = "y", 
    name = "Sepal length (cm)") +
  yaxis_title_plus() + 
  gridlines_plus() + 
  labs(color = expression(italic("Iris")*" species")) +
  direct_labels_plus(data = iris, #<--HELP THE FUNCTION UNDERSTAND HOW YOUR DATA LOOK...
                     x = Petal.Length,
                     y = Sepal.Length,
                     group = Species) + #<--...AND WHAT VARIABLE DETERMINES THE GROUPS.
  guides(color = "none") #<--OPTIONALLY, TURN THE LEGEND OFF.
```

![](README_files/figure-commonmark/direct%20labelling-1.png)

With this approach, the legend becomes optional, saving space,
increasing data density, and concentrating vital information close to
where the reader most needs and expects it.

Another option: Call the reader’s attention to one or some of the groups
over others. `scale_focus_plus()` makes doing this easy:

``` r
ggplot(iris, 
       mapping = aes(x = Petal.Length, 
                     y = Sepal.Length)) +
  geom_point(mapping = aes(color = Species)) + 
  theme_plus() +
  scale_continuous_plus(
    scale = "x",
    name = "Petal length (cm)",
    thin.labels = TRUE) +  
  scale_continuous_plus(
    scale = "y", 
    name = "Sepal length (cm)") +
  yaxis_title_plus() + 
  gridlines_plus() + 
  labs(color = expression(italic("Iris")*" species")) +
  scale_focus_plus(aes = "color", 
                   group_var = iris$Species, #<--HELP THE FUNCTION UNDERSTAND YOUR DATA.
                   focal_groups = "versicolor",
                   diff_nonfocal = TRUE #<--KEEP NON-FOCAL GROUPS DISTINCT.
                  )
```

![](README_files/figure-commonmark/focus%20plus-1.png)

In this version, our eyes can’t help but be drawn to the brightly
colored group first, helping us to understand the message of your graph
quicker.

### Try out new shapes

`ggplotplus` also introduces a new palette of shapes designed to be more
distinctive than base R’s point shapes. Access these via
`geom_point_plus()`:

``` r
ggplot(iris, 
       mapping = aes(x = Petal.Length, 
                     y = Sepal.Length)) +
  geom_point_plus( #<--SWITCH TO THIS FUNCTION TO ACCESS THE NEW SHAPES
    mapping = aes(shape = Species, #<--YOU MUST MAP SHAPE HERE TO A VARIABLE OR CONSTANT
                  fill = Species),
    chosen_shapes = c("squircle", #<--CHOOSE YOUR CUSTOM SHAPES, IF DESIRED.
                      "waffle",
                      "oval"),
    alpha = 0.7
    ) + 
  theme_plus() +
  scale_continuous_plus(
    scale = "x",
    name = "Petal length (cm)",
    thin.labels = TRUE) +  
  scale_continuous_plus(
    scale = "y", 
    name = "Sepal length (cm)") +
  yaxis_title_plus() +
  gridlines_plus() +
  labs(fill = expression(italic("Iris")*" species"), #<--WE NEED BOTH FILL AND SHAPE SCALES TO MATCH IN NAME EXACTLY FOR THEM TO COLLAPSE INTO 1.
       shape = expression(italic("Iris")*" species"))
```

![](README_files/figure-commonmark/new%20shapes-1.png)

The distinctiveness of these shapes makes them quick to distinguish,
even without the aid of differing colors.

The above graphs demonstrates how `ggplotplus`’s tools rethink the
default design features of `ggplot2`. The intention is to yield a more
universal product more quickly so you can spend less time fiddling with
your graphs and more time sharing them with a wider audience.

However, there’s a *lot* more to know! If you want to dive deeper,
[check out the full package
cookbook](https://maisrc.github.io/ggplotplus/).

## Development

`ggplotplus` is under active development. All current core functions are
expected to remain relatively stable, but some parameters, features, and
default settings may evolve, and addressing bugs, inconsistencies, and
gaps remains a high priority. User discretion and feedback is advised!
Please don’t hesitate to share bug reports or questionable performance
issues via the Issues feature on the package’s Github page.

## Acknowledgments

Thanks to Allan Cameron (@Allan Cameron) for answers to Stack Overflow
questions 79654995 and 79682150, which provided key coding elements
enabling the functionality of `yaxis_title_plus()` and
`geom_point_plus()`. Also, many thanks to Dr. Grant Vagle, who
frequently provided testing and feedback on the development of this
package.

#' Internal S7 class for gridline intent
#'
#' Stores user-specified gridline styling from `gridlines_plus()` so it can be
#' applied after ggplot2 has trained the plot scales.
#'
#' @keywords internal
#' @noRd
GridlinesPlus = S7::new_class(
  "GridlinesPlus",
  properties = list(
    color = S7::class_character,
    linewidth = S7::class_numeric,
    linetype = S7::class_character,
    notx = S7::class_logical,
    noty = S7::class_logical,
    override_legend_alphasize = S7::class_logical,
    enable_coaching = S7::class_logical
  )
)

#' Internal S7 class for theme_plus() additions
#'
#' Stores the completed ggplot2 theme object produced by `theme_plus()` and
#' whether geom-specific default adjustments should be applied during plot
#' building.
#'
#' @keywords internal
#' @noRd
ThemePlus = S7::new_class(
  "ThemePlus",
  properties = list(
    applyGeomDefaults = S7::class_logical,
    theme2add = S7::class_any,
    override_legend_alphasize = S7::class_logical,
    enable_coaching = S7::class_logical
  )
)

#' Internal S7 class for y-axis title intent
#'
#' Stores user intent from `yaxis_title_plus()` so the y-axis title can be moved
#' during gtable construction, after ggplot2 has assembled the plot layout.
#'
#' @keywords internal
#' @noRd
YAxisTitlePlus = S7::new_class(
  "YAxisTitlePlus",
  properties = list(
    location = S7::class_character,
    nudgeTopLegendDown = S7::class_logical,
    nudgeHowMuch = S7::class_numeric,
    override_legend_alphasize = S7::class_logical,
    enable_coaching = S7::class_logical
    )
)

#' Internal S7 state container for ggplotplus plots
#'
#' Stores deferred ggplotplus intents attached to a plot. This object is carried
#' on `GGPlotPlusPlot` objects and read during build and gtable stages.
#'
#' @keywords internal
#' @noRd
GGPlotPlusState = S7::new_class(
  "GGPlotPlusState",
  properties = list(
    grid = S7::new_property(class = S7::class_any, default = NULL),
    y_axis_title = S7::new_property(class = S7::class_any, default = NULL),
    theme = S7::new_property(class = S7::class_any, default = NULL),
    guides = S7::new_property(class = S7::class_any, default = NULL),
    warnings = S7::new_property(class = S7::class_any, default = NULL),
    general_intents = S7::new_property(class = S7::class_any, default = NULL)
  )
)

#' Internal S7 ggplot subclass for ggplotplus dispatch
#'
#' Subclasses ggplot2 plot objects so ggplotplus can dispatch custom build-stage
#' behavior without changing ggplot2 behavior globally.
#'
#' @keywords internal
#' @noRd
GGPlotPlusPlot = S7::new_class(
  "GGPlotPlusPlot", #IT'S SIGNATURE, OR NAME, IS GGPlotPlusPlot
  parent = ggplot2::class_ggplot, #THIS IS A SUBCLASS OF CLASS "ggplot"
  properties = list(
    ggplotplus = S7::class_any
  )
)

#' Internal S7 built-plot subclass for ggplotplus gtable dispatch
#'
#' Subclasses ggplot2 built plot objects so ggplotplus can pass deferred
#' gtable-stage intents, such as `yaxis_title_plus()`, from plot building into
#' gtable construction.
#'
#' @keywords internal
#' @noRd
GGPlotPlusBuilt = S7::new_class(
  "GGPlotPlusBuilt",
  parent = ggplot2::class_ggplot_built,
  properties = list(
    y_axis_title = S7::class_any
  )
)

# Backend functions -------------------------------------------------------

#' A backend version of `geom_point()` that can access distinctive shapes.
#'
#' This function behaves much like `ggplot2`'s `geom_point()` function except that it allows access to a palette of nine new shapes that vary in their openness, spikiness, and intersectionality, making them more easily distinguished. This function is not meant to be called directly--instead, `geom_point_plus()` calls and modifies this function and is the intended function for users.
#'
#' @param mapping Set of aesthetic mappings created by aes(), as in `ggplot2::geom_point()`.
#' @param data The data to be displayed in this layer, as in `ggplot2::geom_point()`.
#' @param stat The statistical transformation to use on the data for this layer, as in `ggplot2::geom_point()`.
#' @param position A position adjustment to use on the data for this layer, as in `ggplot2::geom_point()`.
#' @param shapes A named list of custom shapes to be drawn in place of `ggplot2`'s standard palette of shapes.
#' @param ... Other arguments passed on to layer()'s params argument, as in `ggplot2::geom_point()`.
#' @param na.rm Logical controlling whether missing values should be removed from the data with a warning or silently, as in `ggplot2::geom_point()`.
#' @param show.legend Logical controlling whether this layer should be included in the legend(s), as in `ggplot2::geom_point()`.
#' @param inherit.aes Logical controlling whether global aesthetics specified in `ggplot2::ggplot()` should be inherited locally by this layer or not, as in `ggplot2::geom_point()`.
#' @return A ggplot2 layer object.
#' @keywords internal
geom_point2 = function(mapping = NULL,
                       data = NULL,
                       stat = "identity",
                       position = "identity",
                       shapes = NULL, #THE KEY INPUT--THIS IS THE NEW PALETTE OF SHAPES AVAILABLE (PLUS NOW THOSE AVAILABLE FROM THE USER)
                       ...,
                       na.rm = FALSE,
                       show.legend = NA,
                       inherit.aes = TRUE){

  if(is.null(shapes)) {
    shapes = .pointplus_shapes()
  }

  if(length(shapes) == 0) {
    stop("No geom_point_plus() shapes are currently registered.", call. = FALSE)
  }

  ggplot2::layer(
    data = data,
    mapping = mapping,
    stat = stat,
    geom = GeomPointPlus, #IT CALLS THE NEW GEOMPOINTPLUS PROTO.
    position = position,
    show.legend = show.legend,
    inherit.aes = inherit.aes,
    params = rlang::list2(na.rm = na.rm, shapes = shapes, ...)
  )

}

#' An alternative version of `ggplot2`'s geomPoint proto that incorporates new, distinctive shapes.
#'
#' This ggplot proto object is called internally by `geom_point2()` and inherits most, but not all, of its methods and properties from `ggplot2`'s standard `geomPoint` proto. However, it has different default aesthetics, a different shapes palette, and can draw these new shapes in a legend. This subclass is not meant to be encountered by the user and is instead fodder for `geom_point_plus()`.
#'
#' @return A `ggplot2` ggproto subclass object.
#' @keywords internal
GeomPointPlus = ggplot2::ggproto(
  "PointPlus",
  ggplot2::GeomPoint, #THIS NEW GEOM PROTO INHERITS PROPERTIES AND METHODS FROM GEOMPOINT, BUT WE OVERRIDE TWO: DRAW_PANEL AND DRAW_KEY. IT BEHAVES EXACTLY LIKE GEOM_POINT UNLESS A CHARACTER-BASED SHAPE INPUT IS PROVIDED THAT MATCHES ONE IN THE CUSTOM SHAPES LIST.

  #DRAW PANEL IS THE METHOD THAT DICTATES WHAT EXACTLY GETS DRAWN IN THE PLOT PANEL. WE OVERRIDE IT SO OUR CUSTOM SHAPES COME THROUGH.
  draw_panel = function(self,
                        data,
                        panel_params,
                        coord,
                        shapes,
                        na.rm = FALSE) {

    if(is.factor(coord$shape)) { coord$shape = as.character(coord$shape) } #COERCE FACTORS, WHICH MIGHT LOOK NUMERIC, TO CHARACTERS SO CUSTOM SHAPE STRINGS WORK.

    coords = coord$transform(data, panel_params) #TRANSFORM DATA INTO THE PANEL COORDINATE SYSTEM FIRST.

    stroke_size = coords$stroke #NORMALIZE STROKE SIZE.
    stroke_size[is.na(stroke_size)] = 0

    #TWO DIFFERENT WAYS OF INPUTTING SHAPES MEANS TWO DIFFERENT WAYS OF ADDRESSING THE INPUTS. IF THE SHAPE DATA ARE NUMERIC, THE USER PRESUMABLY WANTS PLAIN-OLD GGPLOT2 POINTS AND WE DELEGATE THAT TO THE STANDARD POINTS GROB.

    if(!is.character(coords$shape) &&
       !any(21:25 %in% coords$shape)) {
      return(grid::pointsGrob(
        x   = coords$x,
        y   = coords$y,
        pch = coords$shape,
        gp  = grid::gpar(
          col       = ggplot2::alpha(coords$colour, coords$alpha),
          fill      = ggplot2::fill_alpha(coords$fill, coords$alpha),
          fontsize  = coords$size * ggplot2::.pt + stroke_size * ggplot2::.stroke / 2,
          lwd       = stroke_size * ggplot2::.stroke / 2
        )
      ))
    }

    #OTHERWISE, THE METHOD ASSUMES YOU WANT TO USE THE CUSTOM SHAPES, WHICH IT'LL DRAW BY CONNECTING A SERIES OF COORDINATES USING A PATHGROB.
    #EVERY SHAPE INPUT NEEDS TO BE IN THE SHAPES LIST, AND IT NEEDS TO BE A NAMED LIST WITH $X $Y AND $PIECE ELEMENTS. THE X AND Y VALUES SHOULD BE CENTERED AROUND 0,0 AND BE SCALED TO ~0.4 RADIUS TO BE OF SIMILAR SIZES TO EACH OTHER.

    #FIRST, WE MAKE SURE WE HAVE ALL THE REQUIRED DATA AND OTHERWISE RETURN NULLGROB.
    ok = is.finite(coords$x) &
      is.finite(coords$y) &
      !is.na(coords$shape)
    if(!any(ok)) { return(grid::nullGrob()) }

    coords = coords[ok, , drop = FALSE] #GET RID OF ALL NAs SO WE NEVER ERROR OUT TRYING TO USE THEM AS REFERENTS.

    #IF USER SPECIFIED ANY OF THE BASE SHAPES BTW. 21 AND 25, CONVERT TO CORRESPONDING TEXT STRINGS
    coords$shape = .standardize_pointplus_shape_names(coords$shape) #<--SEE MIDDLEWARE FOR THIS HELPER.

    #IF A USER SPECIFIED AN UNKNOWN SHAPE, BAIL EARLY WITH USEFUL MESSAGE.
    unknown = setdiff(unique(coords$shape),
                      names(shapes))
    if(length(unknown)) {
      stop(sprintf(
        "Unknown shape name(s): %s. Valid names are: %s",
        paste(unknown, collapse = ", "),
        paste(names(shapes), collapse = ", ")
      ), call. = FALSE)
    }

    #PRECOMPUTE THE POINT SIZING IN PTS, REFERRING ONLY TO NON-NA POINTS.
    size_pts = coords$size * ggplot2::.pt + stroke_size * ggplot2::.stroke / 2
    stroke_pts = (stroke_size * ggplot2::.stroke / 2) * 0.8

    .pointplus_scale = 1.2  #TWEAK AS NEEDED TO GET POINTS TO BE ROUGHLY THE RIGHT SIZE.

    #BUILD ONE PATH GROB (CUSTOM SHAPE) PER POINT, USING THE EVEN-ODD RULE TO PUNCH HOLES OUT OF THE CENTER OF SOME SHAPES.
    g = Map(function(x, y, shape, size, color, fill, lwd) {
      dat  = shapes[[shape]]
      xval = ggplot2::unit(x, "npc") + ggplot2::unit(dat$x * size * .pointplus_scale, "points")
      yval = ggplot2::unit(y, "npc") + ggplot2::unit(dat$y * size * .pointplus_scale, "points")
      grid::pathGrob(
        x = xval, y = yval, id = dat$piece,
        rule = "evenodd",
        gp = grid::gpar(
          col = color,
          fill = fill,
          lwd = lwd
        )
      )
    },
    coords$x,
    coords$y,
    coords$shape,
    size_pts,
    ggplot2::alpha(coords$colour, coords$alpha),
    ggplot2::fill_alpha(coords$fill, coords$alpha),
    stroke_pts)

    #ADD POINTS LAYER TO THE GRAPH'S GROBTREE.
    grid::grobTree(grobs = do.call(grid::gList, g), name = "geom_point_plus")
  },

  #DRAW_KEY IS THE METHOD USED TO DRAW THE SYMBOLS IN THE LEGENDS. IT FOLLOWS THE SAME BEHAVIOR AS DRAW_PANEL FOR THE MOST PART EXCEPT IT ONLY DRAWS A SINGLE SYMBOL PER SHAPE NEEDED AT COORDINATES OF 0.5, 0.5 IN NPC UNITS.
  draw_key = function (data, params, size) {

    #SET DEFAULTS THAT MIRROR WHAT THE PACKAGE USES FOR GEOM_POINT. PROBABLY NOT NECESSARY BUT ALSO PROBALBY NOT HURTING ANYTHING.
    if(is.null(data$shape)) data$shape = 21
    stroke_size = (data$stroke %||% 1.2) * 0.8
    stroke_size[is.na(stroke_size)] = 0
    size = (data$size %||% 6 * 1.3)


    #NUMERIC SHAPE VALUES ONCE AGAIN DIVERT TO STANDARD POINTSGROB.
    if(!is.character(data$shape)) {
      return(grid::pointsGrob(
        0.5,
        0.5,
        pch = data$shape,
        gp = grid::gpar(
          col = ggplot2::alpha(data$colour, data$alpha),
          fill = ggplot2::fill_alpha(data$fill, data$alpha),
          fontsize = size * ggplot2::.pt + stroke_size * ggplot2::.stroke / 2,
          lwd = stroke_size * ggplot2::.stroke / 2
        )
      ))
    }


    #OTHERWISE WE DRAW THE KEYS USING OUR CUSTOM SHAPES.
    size_pts   = size * ggplot2::.pt + stroke_size * ggplot2::.stroke / 2
    lwd_pts    = stroke_size * ggplot2::.stroke / 2

    shapes_vec = as.character(data$shape)
    g = Map(function(shape, color, fill) {
      dat  = params$shapes[[shape]]
      xval = grid::unit(0.5, "npc") + grid::unit(dat$x * size_pts, "points")
      yval = grid::unit(0.5, "npc") + grid::unit(dat$y * size_pts, "points")
      grid::pathGrob(
        x = xval, y = yval, id = dat$piece,
        rule = "evenodd",
        gp = grid::gpar(col = color, fill = fill, lwd = lwd_pts)
      )
    },
    shapes_vec,
    ggplot2::alpha(data$colour, data$alpha),
    ggplot2::fill_alpha(data$fill, data$alpha))

    #AND ADD TO THE PLOT'S GROB TREE.
    grid::grobTree(grobs = do.call(grid::gList, g), name = "geom_point_plus_key")
  }
)

# METHODS DISPATCHING -----------------------------------------------------
#' Ensures our plot carries the ggplotplus intent storage and class info into subsequence S7 dispatch methods.
#' @param plot A `ggplot` object.
#'
#' @return The same `ggplot` object, with `plot@ggplotplus` ensured to exist.
#'
#' @details
#' This does **not** build, modify, or draw the plot. It only prepares the
#' object so the custom S7 methods of ggplotplus can
#' later read the recorded intents during dispatch.
#'
#' @keywords internal
#' @noRd
.ensure_ggplotplus_plot = function(plot) {

  if(!inherits(plot, "GGPlotPlusPlot")) {
    plot = S7::convert(plot, GGPlotPlusPlot)
  }

  if(is.null(plot@ggplotplus)) {
    plot@ggplotplus = GGPlotPlusState()
  }

  plot
}

# S7 UPDATE_GGPLOT METHODS ------------------------------------------------
# update_ggplot = ggplot2::update_ggplot #JUST TO GET AROUND THE PARSER FOR TESTING!****

#' Add gridlines_plus() intent to a ggplot object
#'
#' Internal S7 method for adding `GridlinesPlus` objects to ggplot2 plots.
#' Ensures the plot carries ggplotplus state, then stores the gridline intent
#' for resolution during plot building after scales have been trained.
#'
#' @keywords internal
#' @noRd
S7::method(update_ggplot, #REGISTER A NEW SPECIFIC VERSION OF THE GENERIC UPDATE_GGPLOT2 METHOD
           list(GridlinesPlus, ggplot2::class_ggplot)) <- function(object, plot, ...) { #NOTE THAT THE SYNTAX HERE WITH THE ARROW OPERATOR IS ESSENTIAL
    plot = .ensure_ggplotplus_plot(plot) #ALWAYS RUN FIRST TO MAKE SURE GGPlotPlus_State exists
    plot@ggplotplus@grid = object #THE OBJECT IS THE INCOMING BITS AND BOBS FROM gridlines_plus. SINCE ITS A CLEARLY DEFINED S7 CLASS OBJECT ALREADY, WE USE @ TO REFER TO IT AND ALSO HAVE NO NEED TO UNPACK IT HERE.

    if(is.null(plot@ggplotplus@general_intents$override_legend_alphasize) || plot@ggplotplus@general_intents$override_legend_alphasize) { #THIS ENSURES ANY FALSE PERPETUATES.
      plot@ggplotplus@general_intents$override_legend_alphasize = object@override_legend_alphasize
    }
    if(is.null(plot@ggplotplus@general_intents$enable_coaching) ||
       plot@ggplotplus@general_intents$enable_coaching) {
      plot@ggplotplus@general_intents$enable_coaching = object@enable_coaching
    }
    return(plot)
}


#' Add yaxis_title_plus() intent to a ggplot object
#'
#' Internal S7 method for adding `YAxisTitlePlus` objects to ggplot2 plots.
#' Ensures the plot carries ggplotplus state, applies any margin or legend
#' nudging needed before layout, and stores the y-axis title intent for
#' resolution during gtable construction.
#'
#' @keywords internal
#' @noRd
S7::method(update_ggplot,
           list(YAxisTitlePlus, ggplot2::class_ggplot)) <- function(object, plot, ...) {

  plot = .ensure_ggplotplus_plot(plot)

  #IF BUNDLING THE NUDGE FUNCTION, IMPLEMENT HERE.
  if(isTRUE(object@nudgeTopLegendDown)) {
    plot = plot + .nudge_top_legend_down(howMuch = object@nudgeHowMuch)
  }

  plot@ggplotplus@y_axis_title = object
  if(is.null(plot@ggplotplus@general_intents$override_legend_alphasize) ||
     plot@ggplotplus@general_intents$override_legend_alphasize) { #THIS ENSURES ANY FALSE PERPETUATES.
    plot@ggplotplus@general_intents$override_legend_alphasize = object@override_legend_alphasize
  }
  if(is.null(plot@ggplotplus@general_intents$enable_coaching) ||
     plot@ggplotplus@general_intents$enable_coaching) {
    plot@ggplotplus@general_intents$enable_coaching = object@enable_coaching
  }
  return(plot)
}


#' Add theme_plus() to a ggplot object
#'
#' Internal S7 method for adding `ThemePlus` objects to ggplot2 plots. Ensures
#' the plot carries ggplotplus state, applies the completed ggplot2 theme, and
#' stores theme-related intent for later geom-default adjustments during plot-
#' building.
#'
#' @keywords internal
#' @noRd
S7::method(update_ggplot,
           list(ThemePlus, ggplot2::class_ggplot)) <- function(object, plot, ...) {

             plot = .ensure_ggplotplus_plot(plot)

             plot = plot + object@theme2add #<-APPLY THE DEFAULT THEME.

             plot@ggplotplus@theme = object

             if(is.null(plot@ggplotplus@general_intents$override_legend_alphasize) || plot@ggplotplus@general_intents$override_legend_alphasize) { #THIS ENSURES ANY FALSE PERPETUATES.
               plot@ggplotplus@general_intents$override_legend_alphasize = object@override_legend_alphasize
             }
             if(is.null(plot@ggplotplus@general_intents$enable_coaching) ||
                plot@ggplotplus@general_intents$enable_coaching) {
               plot@ggplotplus@general_intents$enable_coaching = object@enable_coaching
             }

             return(plot)
}

# S7 BUILD_GGPLOT METHOD --------------------------------------------------
# ggplot_build = ggplot2::ggplot_build #SAME****

#' Build a ggplotplus plot
#'
#' Internal S7 method for building `GGPlotPlusPlot` objects. Applies deferred
#' ggplotplus build-stage behavior before delegating to ggplot2's ordinary plot
#' build machinery.
#'
#' This method currently patches theme-plus geom defaults into layers before
#' building, applies scale-aware gridline theme adjustments after scales are
#' trained, and forwards any gtable-stage y-axis title intent by converting the
#' built object to `GGPlotPlusBuilt`.
#'
#' @keywords internal
#' @noRd
S7::method(ggplot_build, GGPlotPlusPlot) <- function(plot, ...) {


  ###ALPHA/SIZE LEGEND OVERRIDES OPERATIONS
  #INTENT: WHEN A USER HAS A LEGEND FOR SHAPE, FILL, AND/OR COLOR, AND THEY SET SIZE AND/OR ALPHA TO SMALL VALUES, THESE SMALL VALUES ALSO APPLY *UNNECESSARILY* TO THE KEYS IN THE LEGEND, MAKING THEM HARDER TO READ.
  #HERE, WE DETERMINE IF SUCH ANY OF THE FORMER SCALES HAVE BEEN MAPPED AND OVERRIDE THE DEFAULT AES FOR THE LEGEND KEYS FOR THOSE SCALES FOR ALL AESTHETICS WITHIN C("ALPHA", "SIZE") THAT HAVEN'T ALSO BEEN MAPPED, UNLESS THE USER HAS REQUESTED WE DON'T.

  should_we_override = plot@ggplotplus@general_intents$override_legend_alphasize
  if(should_we_override &&
     !isTRUE(plot@ggplotplus@general_intents$legend_alphasize_override_applied)) { #<--FAIL EARLY

    #CHECK FOR MAPPED SCALES
  has_fill_mapped = .plot_has_mapped_aes(plot, c("fill")) #<--GO SEE MIDDLEWARE.R FOR THIS HELPER.
  has_colour_mapped = .plot_has_mapped_aes(plot, c("colour"))
  has_shape_mapped = .plot_has_mapped_aes(plot, c("shape"))
  has_alpha_mapped = .plot_has_mapped_aes(plot, c("alpha"))
  has_size_mapped = .plot_has_mapped_aes(plot, c("size"))

  if(any(has_fill_mapped, has_colour_mapped, has_shape_mapped) && #YES, WE HAVE THESE.
     any(!has_alpha_mapped, !has_size_mapped) #AND WE LACK AT LEAST ONE OF THESE.
  ) {

    override_list = list()
    if(!has_alpha_mapped) {
      override_list$alpha = 1 #ADD SIZE AND/OR ALPHA AS APPROPRIATE
    }
    if(!has_size_mapped) {
      override_list$size = 5
    }

    #THEN, SELECTIVELY OVERRIDE RELEVANT LEGENDS.
    if(length(override_list) > 0) {

      target_aes = character(0)

      #FIGURE OUT WHICH OUT OF FILL, COLOUR, AND SHAPE ARE BEING MAPPED...
      if(has_fill_mapped && !.guide_is_none_for_aes(plot, "fill") &&
         !.aes_mapped_var_is_continuous(plot, "fill")) { #<--DON'T DO THIS WHEN MAPPED TO A CONTINUOUS VARIABLE!
        target_aes = c(target_aes, "fill")
      }

      if(has_colour_mapped && !.guide_is_none_for_aes(plot, "colour") &&
         !.aes_mapped_var_is_continuous(plot, "colour")) {
        target_aes = c(target_aes, "colour")
      }

      if(has_shape_mapped && !.guide_is_none_for_aes(plot, "shape")) {
        target_aes = c(target_aes, "shape")
      }

      if(length(target_aes) > 0) {

        all_target_aes = target_aes

        #JUNCTION ALL TARGET LABELS WITH THE LABELS OF THE TARGET AES SCALES.
        target_labels = vapply(target_aes, function(aes_name) {
          .get_prebuild_aes_label(plot, aes_name)
        }, character(1))


        #IF THE LABELS ARE DUPLICATED FOR ANY TWO OR MORE OF THESE SCALES, NO NEED TO OVERRIDE.AES FOR BOTH, AS THOSE SCALES ARE LIKELY TO COLLAPSE.
        keep_aes = target_aes[!duplicated(target_labels)]
        drop_aes = setdiff(all_target_aes, keep_aes) #NOTE WHICH THESE ARE.

        #THEN, CHECK THE ONES WE'RE GOING TO NOT MANIPULATE FOR ANY override.aes() CONDITIONS AND DROP THOSE SO THERE IS ONLY THE ONE. THIS SHOULD BE OK BECAUSE THE LEGENDS SHOULD BE PLANNING TO COLLAPSE.
        for(aes_name in drop_aes) {
          guide = plot@guides$guides[[aes_name]]

          if(!is.null(guide) && inherits(guide, "GuideLegend")) {
            guide$params$override.aes = list()
            plot = plot + ggplot2::guides(!!aes_name := guide)
          }
        }

        #THEN APPLY THE OVERRIDE.AES OVERRIDES.
        if("fill" %in% keep_aes) {
          plot = plot + ggplot2::guides(
            fill = .merge_legend_override(plot, "fill", override_list)
          )
        }

        if("colour" %in% keep_aes) {
          plot = plot + ggplot2::guides(
            colour = .merge_legend_override(plot, "colour", override_list)
          )
        }

        if("shape" %in% keep_aes) {
          plot = plot + ggplot2::guides(
            shape = .merge_legend_override(plot, "shape", override_list)
          )
        }
      }
    }

  }
  plot@ggplotplus@general_intents$legend_alphasize_override_applied = TRUE #MAKES THIS IDEMPOTENT TO ONLY RUN EVER THE ONCE, NO MATTER HOW MANY BUILDS OCCUR.

  }#/END OVERRIDING LEGEND DEFAULTS FOR SIZE/ALPHA


   ###THEME PLUS GEOM DEFAULTS OPERATIONS
  if(.s7_prop_is_true(plot@ggplotplus@theme, "applyGeomDefaults")) {

    plotLayers = plot$layers #ID ALL LAYERS (GEOM) IN THIS PLOT...
    if(length(plotLayers) > 0) { #POSSIBLE THEY PUT IN NO GEOMS, IN WHICH CASE THERE'D BE NO LAYERS.
    for(layer in 1:length(plotLayers)) { #GO LAYER BY LAYER

    thisLayer = plotLayers[[layer]] #ISOLATE THIS LAYER
    geom2match = class(thisLayer$geom)[1] #GET THE CLASS TYPE OF THIS GEOM
    geom2match = tolower(sub("^Geom", "", geom2match)) #YANK OUT THE GEOM BIT AND COERCE TO LOWER.

    defaults2use = geom_plus_defaults[[geom2match]] #GRAB THE DEFAULTS FOR IT FROM OUR GLOBAL OBJ.
    if(is.null(defaults2use)) { next } #SKIP IF NONE.

    if(length(defaults2use$aes) > 0) { #IF THIS GEOM HAS AES DEFAULTS...
    for(aes in 1:length(defaults2use$aes)) { #GO ONE BY ONE THRU THEM

      currAes2Check = names(defaults2use$aes[aes]) #GET NAME OF CURR AES

      if(any( #IF THE USER HAS ALREADY SPECIFIED SOMETHING FOR THIS AES, SKIP.
        currAes2Check %in% names(thisLayer$mapping),
        (currAes2Check %in% names(plot$mapping) && isTRUE(thisLayer$inherit.aes)),
        currAes2Check %in% names(thisLayer$aes_params)
      )) { next }

      #OTHERWISE, INSERT DEFAULT INTO OBJ.
      plot$layers[[layer]]$aes_params[[currAes2Check]] = defaults2use$aes[[aes]]

      }#END AES DEFAULTS LOOP

    } #END AES DEFAULTS REGION

    if(length(defaults2use$params$geom_params) > 0) {
      for(Gparam in 1:length(defaults2use$params$geom_params)) {

        currParam2Check = names(defaults2use$params$geom_params[Gparam])

        if(.param_is_already_set(thisLayer, currParam2Check)) {
          next
        }

        #OTHERWISE, INSERT DEFAULT INTO OBJ.
        plot$layers[[layer]]$geom_params[[currParam2Check]] = defaults2use$params$geom_params[[Gparam]]
      }
    }
    if(length(defaults2use$params$stat_params) > 0) {
      for(Sparam in 1:length(defaults2use$params$stat_params)) {

        currParam2Check = names(defaults2use$params$stat_params[Sparam])

        if(any( #FEWER PLACES TO LOOK FOR LAYER PARAMETERS...
          currParam2Check %in% names(thisLayer$stat_params)
        )) { next }

        #OTHERWISE, INSERT DEFAULT INTO OBJ.
        plot$layers[[layer]]$stat_params[[currParam2Check]] = defaults2use$params$stat_params[[Sparam]]

        }

      }#END PARAM DEFAULTS LOOP

    } #END LAYER BY LAYER GEOM DEFAULTS CHECK
   } #NO LAYERS/NO LAYERS CHECK

  } #END THEME PLUS GEOM DEFAULTS REGION


    plain_plot = S7::convert(plot, ggplot2::class_ggplot) #CONVERT TO REGULAR OLD class_ggplot TO BUILD GENERICALLY.
    built = ggplot2::ggplot_build(plain_plot, ...)

    ###GRIDLINES PLUS OPERATIONS
    grid_intents = plot@ggplotplus@grid # CONVENIENCE OBJ
    if(!is.null(grid_intents)) {

      grid_theme = .apply_gridlines_plus(built, grid_intents) #<--GO SEE MIDDLEWARE.R FOR THIS HELPER.

      built$plot = built$plot + grid_theme

    } #END GRIDLINES_PLUS OPERATIONS


    should_we_coach = plot@ggplotplus@general_intents$enable_coaching &&
      .ggplotplus_coaching_enabled()

    if(should_we_coach) {

    ###GUIDING MESSAGES CONCERNING OVER-RELIANCE ON DISCRETE SHAPES/COLORS
    #***I THINK THIS WORKS, BUT IT LOOKS LIKE USING ggplot2::get_guide_data() COULD MAYBE HAVE BEEN EASIER.
    plot_scales = built@plot@scales$scales
    discrete_plus_FCS = Filter(function(x) { inherits(x, "ScaleDiscrete") &&
                                         any(c("fill", "colour", "shape") %in% x$aesthetics) }, plot_scales)

    warned_colour_fill = FALSE

    if(length(discrete_plus_FCS)) {
      for(i in 1:length(discrete_plus_FCS)) {
        uniqVals = length(unique(discrete_plus_FCS[[i]]$range$range))
        if(any(c("fill", "colour") %in% discrete_plus_FCS[[i]]$aesthetics) &&
           uniqVals > 7 &&
           warned_colour_fill == FALSE) {
          message("\nNote: You've mapped color and/or fill to a discrete variable with > 7 levels. This is not recommended. Even when using a color palette designed for maximum contrast and discernability (such as viridis), most humans are not able to readily distinguish all colors from one another in any palette beyond about 7 colors. Consider using a different visual channel, filtering or consolidating to a smaller number of levels, or layering on a second visual channel (such as shape or line type). Alternatively, consider shuffling the color values to make dissimilar colors appear nearer to one another to facilitate comparisons. Set enable_coaching to FALSE to disable these messages.")
          warned_colour_fill = TRUE
        }
        if("shape" %in% discrete_plus_FCS[[i]]$aesthetics &&
           uniqVals > 9) {
          message("\nNote: You've mapped shape to a discrete variable with > 9 levels. This is not recommended. Even when using a shape palette designed for maximum contrast and discernability (such as that available via geom_point_plus()), most humans are not able to readily distinguish all shapes from one another in any palette beyond about 9 shapes. Consider using a different visual channel, filtering or consolidating to a smaller number of levels, or layering on a second visual channel (such as color or angle of orientation). Alternatively, consider shuffling the shape values to make dissimilar shapes appear nearer to one another to facilitate comparisons. Set enable_coaching to FALSE to disable these messages.")
        }
      }
    }

    ###GUIDING MESSAGES AROUND RENAMING SCALES TO BE MORE INFORMATIVE
    plot_labs = ggplot2::get_labs(built)
    plot_labs = plot_labs[vapply(plot_labs, is.character, logical(1))]
    plot_labs = unlist(plot_labs, use.names = TRUE)

    scale_labs = intersect(
      names(plot_labs),
      c("x", "y", "colour", "color", "fill", "shape", "size", "alpha",
        "linetype", "linewidth")
    )

    data_names = .plot_data_names(built@plot)

    unchanged_labs = scale_labs[plot_labs[scale_labs] %in% data_names]

    length_these = length(unchanged_labs)

    if(length_these) {
      combined = ifelse(length_these == 1, unchanged_labs, paste0(unchanged_labs, collapse = " and "))

      message(sprintf(
        "\nNote: For your %s scale(s), you didn't apparently set a title different than the name of the column mapped to that scale. This is not generally recommended. Column names tend to be machine- rather than human-readable, lack typical spacing, capitalization, and punctuation usage, and they tend to lack units. Consider using ggplot2::labs() to provide these scales with new, human-readable and informative titles. Set enable_coaching to FALSE to disable these messages.",
        combined
      ))
    }

    } #/END COACHING SECTION


    #REGISTERING INTENT TO MOVE ALONG TO THE GTABLE METHOD FOR YAXIS TITLE PLUS AS NEEDED.
    if(!is.null(plot@ggplotplus@y_axis_title)) {
      built = S7::convert(built, GGPlotPlusBuilt)
      built@y_axis_title = YAxisTitlePlus(
          location = plot@ggplotplus@y_axis_title@location
        )
    }

    return(built)

}



# S7 GTABLE METHOD --------------------------------------------------------

# ggplot_gtable = ggplot2::ggplot_gtable #SAME****


#' Convert a ggplotplus built plot to a gtable
#'
#' Internal S7 method for converting `GGPlotPlusBuilt` objects to gtables.
#' Delegates first to ggplot2's ordinary gtable construction, then applies any
#' deferred ggplotplus layout edits that require access to the completed gtable.
#'
#' This method currently supports `yaxis_title_plus()`, which moves the y-axis
#' title into a custom row above or below the panel region.
#'
#' @keywords internal
#' @noRd
S7::method(ggplot_gtable, GGPlotPlusBuilt) <- function(data, ...) {

  plain_data = S7::convert(data, ggplot2::class_ggplot_built) #CONVERT TO REGULAR OLD class_ggplot TO GTABLE GENERICALLY.
  gtable = ggplot2::ggplot_gtable(plain_data)

  yaxis_intents = data@y_axis_title # CONVENIENCE OBJ

  ###Y-AXIS TITLE PLUS OPERATIONS
  if(is.null(yaxis_intents)) {
  return(gtable)
  }

  gtable = .apply_yaxis_title_plus(data, gtable, yaxis_intents)

  return(gtable)

}

#' Subtle, Minimal Gridlines For When and Where They Help
#'
#' Adds light and easily ignored major gridlines along only axes mapped to
#' continuous variables. Minor gridlines are
#' blanked. This enables the benefits of gridlines (in instances where there are some) but minimizes visual clutter and cognitive load.
#'
#' `gridlines_plus()` ignores any theme instructions to blank major panel scales in either the x or y direction via, e.g., `theme_plus(panel.grid.major.y = ggplot2::element_blank())`. Instead, users can set, e.g., noty == TRUE to prevent gridlines from being drawn in a specific direction, even if that scale is continuous. Similarly, gridline color, linewidth, and linetypes should be set directly in `gridlines_plus()` instead of via `theme()`. Trying to do the latter will fail without error or warning!
#'
#' @param color Gridline color. Single character string. Default: `"gray90"`.
#' @param linewidth Gridline width (theme line units). Single numeric. Default: `1.2`.
#' @param linetype Gridline type. Single string (e.g., `"solid"`, `"dashed"`).
#' @param notx,noty Logicals indicating whether gridlines should not be drawn in a specific direction (i.e., the user wants those to be blank even when the axis is continuous). Default to FALSE (gridlines are added).
#' @param override_legend_alphasize Logical indicating whether, for any fill, color, and/or shape legends, to override the size and/or alpha values to the ggplotplus default values (1 and 5, respectively). Defaults to TRUE.
#' @param enable_coaching Logical indicating whether ggplotplus should perform certain automated checks for possible non-Universal graph design and provide coaching messages when these are detected. Defaults to TRUE.
#'
#' @return An ggplot class object for adding to a plot with `+`.
#'
#' @details
#' Under the hood, `gridlines_plus()` checks layer and/or global mappings to
#' see if `x` and/or `y` are continuous. It does this by inspecting the trained panel scales.
#' It then turns on **major** gridlines for continuous directions and explicitly
#' blanks gridlines for other axes (as well as **all minor** gridlines).
#'
#' @examples
#' library(ggplot2)
#'
#' ggplot2::ggplot(iris, ggplot2::aes(Sepal.Length, Petal.Length)) +
#'   ggplot2::geom_point() +
#'   theme_plus() + #WE DON'T RECOMMEND USING gridlines_plus() WITHOUT ALSO USING theme_plus()
#'   gridlines_plus()
#'
#' # Only y is continuous here (x is discrete) → y-only major gridlines
#' ggplot2::ggplot(mtcars, ggplot2::aes(factor(cyl), mpg)) +
#'   ggplot2::geom_boxplot() +
#'   gridlines_plus(color = "grey85", linewidth = 1, linetype = "dashed")
#'
#' # Works with derived continuous axes (histogram)
#' ggplot2::ggplot(mtcars, aes(mpg)) +
#'   ggplot2::geom_histogram() +
#'   gridlines_plus()
#'
#' @export
gridlines_plus = function(color = "gray90",
                          linewidth = 1.2,
                          linetype = "solid",
                          notx = FALSE,
                          noty = FALSE,
                          override_legend_alphasize = TRUE,
                          enable_coaching = TRUE) {
  GridlinesPlus(
    color = color,
    linewidth = linewidth,
    linetype = linetype,
    notx = notx,
    noty = noty,
    override_legend_alphasize = override_legend_alphasize,
    enable_coaching = enable_coaching
  )
}


#' Relocate a Y Axis Title to Above the Y Axis on a Ggplot and Turn it Horizontal.
#'
#' This function relocates the y axis title of a ggplot to the top, above the y axis line and left-justified to the left edge of the y axis labels, sort of like a plot subtitle. It also orients the text horizontally for space-efficiency and easy reading. This is otherwise difficult to do using `ggplot2`'s default styling tools.
#'
#' @param location A length-1 character string matching either "top" or "bottom" for the placement of the new y axis title. Defaults to `"top"`. `"bottom"` should generally only be used when the x axis labels have been moved to the top of the graph (uncommon).
#' @param nudgeTopLegendDown A length-1 logical indicating whether a top legend (box) (if any) should be moved down to align with the relocated y axis title (where they could clip into each other). Defaults to FALSE.
#'
#' @param nudgeHowMuch A length-1 positive integer indicating how much to nudge the top legend (box) (if any) down, if `nudgeTopLegendDown` == `TRUE`. Defaults to `20` points as a general guess and may need adjusting.
#' @param override_legend_alphasize Logical indicating whether, for any fill, color, and/or shape legends, to override the size and/or alpha values to the ggplotplus default values (1 and 5, respectively). Defaults to TRUE.
#' @param enable_coaching Logical indicating whether ggplotplus should perform certain automated checks for possible non-Universal graph design and provide coaching messages when these are detected. Defaults to TRUE.
#'
#' @return An ggplot class object for adding to a plot with `+`.
#' @examples
#' #WE DO NOT RECOMMEND USING yaxis_title_plus() WITHOUT theme_plus()
#' ggplot2::ggplot(iris, ggplot2::aes(x=Sepal.Length, y=Petal.Length)) +
#' ggplot2::geom_point() +
#' theme_plus() +
#' yaxis_title_plus()
#' @export
yaxis_title_plus = function(location = "top",
                            nudgeTopLegendDown = FALSE,
                            nudgeHowMuch = 20,
                            override_legend_alphasize = TRUE,
                            enable_coaching = TRUE) {
  YAxisTitlePlus(
    location = location,
    nudgeTopLegendDown = nudgeTopLegendDown,
    nudgeHowMuch = nudgeHowMuch,
    override_legend_alphasize = override_legend_alphasize,
    enable_coaching = enable_coaching
  )
}



#' Continuous Scales With Endpoint-Aware Breaks
#'
#' `scale_continuous_plus()` is an opinionated wrapper around ggplot2's
#' continuous x, y, colour, and fill scales. It chooses "pretty" breaks while
#' gently expanding the scale limits so breaks generally will appear near both
#' ends of the data range.
#'
#' This is useful because ggplot2's default continuous scales frequently will leave the ends of an
#' axis or colorbar visually unlabeled, making it look as if an endpoint break is
#' missing.
#'
#' @param scale Character string specifying which scale to modify. Options are
#'   `"x"`, `"y"`, `"colour"`/`"color"`, and `"fill"`.
#' @param ... Additional arguments passed to the corresponding ggplot2
#'   continuous scale function. Arguments such as `name`, `labels`, `guide`,
#'   `position`, and `expand` may be supplied. User-supplied `breaks`, `limits`,
#'   `n.breaks`, `trans`, and `transform` are ignored with a warning because
#'   this function controls those components directly.
#' @param thin.labels Logical. If `TRUE`, every other break label is blanked
#'   to reduce crowding. Defaults to `FALSE`.
#' @param pad.labels Character string, either `"start"` or `"end"`. Used when a
#'   user-supplied label vector is shorter than the internally computed break
#'   vector by one label (which must be assessed via trial and error currently). `"start"` pads a blank label at the beginning;
#'   `"end"` pads one at the end.
#' @param target.breaks Integer target number of major breaks. This is a target and
#'   not a guarantee because breaks are chosen using a "pretty" break algorithm.
#'   Default is `5`.
#' @param buffer_frac Numeric fraction of the data span used to decide whether a
#'   break is close enough to each endpoint. Default is `0.05`.
#' @param split_name Logical. If `TRUE`, spaces in a named `name` argument are
#'   replaced with line breaks. This can help long axis or legend titles fit
#'   better. Default is `FALSE`.
#'
#' @details
#' `scale_continuous_plus()` routes to one of
#' [ggplot2::scale_x_continuous()], [ggplot2::scale_y_continuous()],
#' [ggplot2::scale_colour_continuous()], or
#' [ggplot2::scale_fill_continuous()] based on `scale`.
#'
#' Unlike the ggplot2 defaults, this function intentionally controls `breaks`
#' and `limits`. If either is supplied through `...`, it's ignored with a
#' warning. Transformed scales are also not currently supported; pre-transform
#' the data or use ggplot2's scale functions directly when a transformed scale is
#' needed.
#'
#' User-supplied label vectors are supported, but endpoint-aware breaks can
#' sometimes create hidden outer breaks. When possible, this function pads label
#' vectors with blank labels to align them with the computed break vector in length. If
#' alignment is ambiguous by one label, use `pad.labels` to choose which side of the input labels vector to
#' pad.
#'
#' @return A ggplot2 continuous scale object.
#'
#' @examples
#' library(ggplot2)
#'
#' ggplot(iris, aes(Sepal.Width, Sepal.Length)) +
#'   geom_point() +
#'   scale_continuous_plus(scale = "x") +
#'   scale_continuous_plus(scale = "y")
#'
#' ggplot(iris, aes(Sepal.Width, Sepal.Length, fill = Petal.Length)) +
#'   geom_point(shape = 21) +
#'   scale_continuous_plus(scale = "x") +
#'   scale_continuous_plus(scale = "y") +
#'   scale_continuous_plus(scale = "fill")
#'
#' ggplot(iris, aes(Sepal.Width, Sepal.Length)) +
#'   geom_point() +
#'   scale_continuous_plus(
#'     scale = "y",
#'     name = "Sepal length",
#'     labels = LETTERS[1:5],
#'     pad.labels = "start"
#'   )
#'
#' @export
scale_continuous_plus =
  function(scale = NA,
           ...,
           thin.labels = FALSE,
           pad.labels = "start",
           target.breaks = 5,
           buffer_frac = 0.05,
           split_name = FALSE) {

  lookup = data.frame(
    incoming = c("fill", "color", "colour", "x", "y"),
    outgoing = c("scale_fill_continuous",
                 "scale_colour_continuous",
                 "scale_colour_continuous",
                 "scale_x_continuous",
                 "scale_y_continuous"
                )
  )

  if(is.na(scale)) { stop("A scale is required. Please provide one. The options are \"x\", \"y\", \"colour\", and \"fill\"") }

  user.args = list(...)

  any_user_breaks = .partial_match_user_arg(user.args, "breaks")
  any_user_limits = .partial_match_user_arg(user.args, "limits")
  bad = !is.null(any_user_breaks) | !is.null(any_user_limits)

  if(bad) {
    warning("This function sets opinionated breaks and limits, so yours were ignored. If you want to set these yourself, use ggplot2::scale_*_continuous().",
         call. = FALSE)
   user.args = .remove_partial_match_user_arg(user.args, "breaks")
   user.args = .remove_partial_match_user_arg(user.args, "limits")
  }

  any_nbreaks = .partial_match_user_arg(user.args, "n.breaks")

  if(!is.null(any_nbreaks)) {
    warning("Use the target.breaks argument instead of the n.breaks argument to control the target number of breaks.")
    user.args = .remove_partial_match_user_arg(user.args, "n.breaks")
  }

  any_trans = .partial_match_user_arg(user.args, "trans")

  if(!is.null(any_trans)) {
    warning(
      "A `trans` argument was supplied, but that argument is deprecated, so it was ignored.",
      call. = FALSE
    )
    user.args = .remove_partial_match_user_arg(user.args, "trans")
  }

  any_transform = .partial_match_user_arg(user.args, "transform")

  if(!is.null(any_transform)) {
  warning(
    "A `transform` argument was supplied, but `scale_continuous_plus()` does not currently support transformed scales, so your transform input was ignored. Pre-transform the data or use `ggplot2::scale_*_continuous(transform = ...)` instead.",
    call. = FALSE
  )
    user.args = .remove_partial_match_user_arg(user.args, "transform")
  }

  split_requested = isTRUE(split_name)
  any_name = .partial_match_user_arg(user.args, "name")
  if(split_requested && is.null(any_name)) {
    warning("`split_name` was set to TRUE but no named `name` argument was provided. Please provide a named `name` argument to use `split_name.")
    split_name = FALSE
  }

  if(split_requested &&
     !is.null(any_name)) {
    user.args = .remove_partial_match_user_arg(user.args, "name")
    user.args$name = gsub(" ", "\n", any_name)
  }

  args = user.args

  if(is.null(.partial_match_user_arg(args, "expand"))) {
    args$expand = c(0,0)
  }

  args$breaks = function(x) {
   .endpoint_breaks(x, n = target.breaks, buffer_frac = buffer_frac, Return = "breaks")
  }

  args$limits = function(x) {
    .endpoint_breaks(x, n = target.breaks, buffer_frac = buffer_frac, Return = "limits")
   }

  #MAKE USER LABEL DECISIONS IF ANY USER LABELS EXIST
  any_user_labels = .partial_match_user_arg(user.args, "labels")

  if (!is.null(any_user_labels) || thin.labels) {

  #DETERMINE WHAT KIND OF USER LABELS WERE GIVEN AND BRANCH
  user_label_mode =
    if (is.null(any_user_labels)) {
      "none"
    } else if (is.function(any_user_labels)) {
      "function"
    } else if (is.atomic(any_user_labels) || is.expression(any_user_labels)) {
      "vector"
    } else {
      stop("`labels` must be NULL, a function, or a vector/expression.", call. = FALSE)
    }

  args$labels = function(x) {

    labs = x #DEFAULT TO X, OVERRIDE AS NEEDED IF USER PROVIDED LABELS.

    if(user_label_mode != "none") {
    if(user_label_mode == "function") {
        labs = any_user_labels(x) #RUN USER LABELING FUNCTION
      } else {

        #OTHERWISE, USER PROVIDED VECTOR, SO LET'S ADDRESS THE 4 POSSIBILITIES:
        delta = length(x) - length(any_user_labels)

      if (delta == 0) { #1: THEY MATCH PERFECTLY, SO ROLL WITH THEM.
        labs = any_user_labels

      } else if (delta == 1) { #2: THEY MISS BY ONE, UNFORTUNATELY, SO EITHER PAD LEFT OR PAD RIGHT AS USER REQUESTS.
        if (pad.labels == "start") {
          labs = c("", any_user_labels)
        } else {
          labs = c(any_user_labels, "")
        }
      } else if (delta == 2) { ##3: THEY MISS BY TWO, SO PAD EITHER SIDE.
        labs = c("", any_user_labels, "")
      } else {
        stop( #4: GIVE UP AND CRY THAT THE LABELS DON'T MATCH.
          paste0( #THIS IS A NICE ERROR CHATGPT--TELLS YOU HOW MANY YOU SHOULD HAVE BEEN GOING FOR.
            "Custom labels could not be aligned to the computed breaks. ",
            "Expected ", length(x), " labels but received ", length(any_user_labels), "."
          ),
          call. = FALSE
        )
      }

      }
    }

    #IF THE USER WANTS EVERY OTHER REAL LABEL BLANKED...
    if(thin.labels) {

      #A LITTLE HARD TO GUESS WHAT TO DO IF THEY DIDN'T SPECIFY LABELS SINCE WE COULD BE OFF, BUT WE'LL JUST GUESS AND THEY CAN USE PAD.LABELS TO NUDGE IT.
      if(user_label_mode == "none") {
        if(pad.labels == "start") {
          labs[seq(from = 1, to = length(labs), by = 2)] = ""
        } else {
          labs[seq(from = 2, to = length(labs), by = 2)] = ""
        }
      } else {

      labs[is.na(labs)] = "" #GUARDS AGAINST GGPLOT TRAINING NONSENSE.
      first_real = which(labs != "")[1] #FIND THE FIRST "REAL" LABEL
      if(!is.na(first_real)) {
        blank_these = seq(from = first_real + 1,
                  to = length(labs),
                  by = 2)
        labs[blank_these] = ""
      }
     }
    }

    return(labs)
   }
  } #END LABEL SETTING SKIP

  do.call(lookup$outgoing[lookup$incoming == scale], args)
}


#' A Universal Design-Oriented Base Ggplot2 Theme With Scalable and Overridable Defaults
#'
#' `theme_plus()` returns a ggplot2 theme designed to make
#' publication-quality, accessible graphs easier to produce. It keeps all of
#' ggplot2’s normal behaviors (last theme wins; user overrides take precedence),
#' but bakes in opinionated defaults with Universal Design in mind. A few knobs
#' let you scale typography/lines, flip the legend layout, and switch the
#' background color if desired.
#'
#' Internally, text sizes are expressed with `rel()`, so they scale with
#' `base_font_size`. Line/rect line thicknesses start from `base_linewidth` and
#' `base_rectlinewidth` and scale similarly with `rel()`.
#'
#' The function builds
#' a  base theme, then *adds* any user overrides via
#' `theme(...)`, so the user's preferences always take precedence.
#'
#' @param ... Optional additional theme settings passed to [ggplot2::theme()]. These are applied *after* the base theme, so the theme's defaults only "win" when no matching settings are provided by the user
#'   (same as in ggplot2).
#' @param legend_pos Where to put the legend. `"top"` (default) creates a
#'   horizontal stripe at the top for the legend (box) when one is present;
#'   `"right"` uses a vertical legend at the right (ggplot2’s usual position)
#'   but with design modifications. `"bottom"` is also an option (largely same
#'   as `"top"`).
#' @param base_font_size Base text size (in points) for most text elements. These
#'   will scale via `rel()`. Default is `16`.
#' @param base_linewidth Baseline thickness for most **line** theme elements
#'   (e.g., axis lines and tick marks). Defaults to `1.2`. Specific elements
#'    may use `rel()` multipliers on top of this.
#' @param base_rectlinewidth Baseline line thickness for most **rect** theme elements
#'   (e.g., legend frames). Defaults to `1.2`.
#' @param line_color Default color for most line elements (axis lines, frames, etc.).
#'   Defaults to `"black"`.
#' @param text_color Default color for most text elements. Defaults to `"black"`.
#' @param background_color Background fill applied to the panel, plot, legend,
#'   and strip backgrounds. Defaults to a slightly warm white, `"#FFFEFD"`, to reduce eyestrain.
#' @param palette_discrete,palette_continuous Default viridis-family color palette codes ("A" through "H") to use for discrete and continuous scales, respectively.
#' @param begin_discrete,end_discrete,begin_continuous,end_continuous Numeric values ranging between 0 and 1 for where to begin drawing colors from a viridis palette for a discrete and continuous color scale, respectively.
#' @param export_width,export_height Length-1 numeric values indicating your intended export (most likely via ggplot2::ggsave()) width and height, respectively. This rescales font and line sizes internally to stay relatively appropriately for your intended export size.
#' @param override_legend_alphasize Logical indicating whether, for any fill, color, and/or shape legends, to override the size and/or alpha values to the ggplotplus default values (1 and 5, respectively). Defaults to TRUE.
#' @param enable_coaching Logical indicating whether ggplotplus should perform certain automated checks for possible non-Universal graph design and provide coaching messages when these are detected. Defaults to TRUE.
#'
#' @return A ggplot2 theme object to add with `+`.
#'
#' @examples
#' # Basic use
#' library(ggplot2)
#' ggplot(iris, aes(Sepal.Length, Petal.Length, colour = Species)) +
#'   geom_point() +
#'   theme_plus()
#'
#' # Prefer the right-side legend and pure white background
#' ggplot(mtcars, aes(wt, mpg)) +
#'   geom_point() +
#'   theme_plus(legend_pos = "right", background_color = "white")
#'
#' # Scale text up and make lines a bit lighter
#' ggplot(iris, aes(Sepal.Length, Petal.Length)) +
#'   geom_point() +
#'   theme_plus(base_font_size = 18, base_linewidth = 1.0)
#'
#' # You can still override any element normally via theme()
#' ggplot(iris, aes(Sepal.Length, Petal.Length)) +
#'   geom_point() +
#'   theme_plus() +
#'   theme(axis.line = element_line(linewidth = 0.8))
#'
#' # But you could just as easily do so via theme_plus()
#' ggplot(iris, aes(Sepal.Length, Petal.Length)) +
#'   geom_point() +
#'   theme_plus(axis.line = element_line(linewidth = 0.8))
#'
#' @seealso [ggplot2::theme()], [ggplot2::theme_gray()], [ggplot2::theme_get()]
#' @export
theme_plus = function(...,
                      legend_pos = "top",
                      base_font_size = 16,
                      base_linewidth = 1.2,
                      base_rectlinewidth = 1.2,
                      line_color = "black",
                      text_color = "black",
                      background_color = "#FFFEFD",
                      palette_discrete = "D",
                      palette_continuous = "E",
                      begin_discrete = 0,
                      end_discrete = 0.72,
                      begin_continuous = 0,
                      end_continuous = 1,
                      export_width = 7.25,
                      export_height = 5.95,
                      override_legend_alphasize = TRUE,
                      enable_coaching = TRUE) {

  #ONE ODD INTERACTION IS WITH strip.text(), WHICH CAN ONLY BE BLANKED BY HITTING X AND Y SEPARATELY. SOMETHING TO PONDER WHEN ADJUSTING SUBSCALES SEPARATELY...
  user_theme = ggplot2::theme(...)
  dots = rlang::list2(...)
  if(inherits(dots$strip.text, "element_blank")) {
    user_theme = user_theme +
      ggplot2::theme(
        strip.text.x = ggplot2::element_blank(),
        strip.text.y = ggplot2::element_blank()
      )
  }


  gg_palette_theme = .theme_plus_palettes(palette_discrete,
                                          palette_continuous,
                                          begin_discrete,
                                          end_discrete,
                                          begin_continuous,
                                          end_continuous)

  ref_width = 6.25
  ref_height = 7.79

  scale_factor = sqrt((export_width * export_height) / (ref_width * ref_height))

  base_font_size = base_font_size * scale_factor
  base_linewidth = base_linewidth * scale_factor
  base_rectlinewidth = base_rectlinewidth * scale_factor

  #WE NOW USE THE REL() FUNCTION TO KEEP THINGS SCALED RELATIVE TO THE BASE_FONT_SIZE FOR CONVENIENCE

  default_theme = ggplot2::theme_gray(base_size = base_font_size,
                                      base_line_size = base_linewidth,
                                      base_rect_size = base_rectlinewidth
  ) +
    ggplot2::theme_sub_axis(
      line = ggplot2::element_line(color = line_color, linewidth = ggplot2::rel(1), lineend = "square"), #ADD THICK BLACK X AND Y AXIS LINES WITH SQUARE ENDS TO ENSURE THAT THEY APPEAR TO VISUALLY MEET.
      title = ggplot2::element_text(color = text_color, size = ggplot2::rel(1.125), face = "bold", family = "sans"),
      text = ggplot2::element_text(size = ggplot2::rel(1), color = text_color), #ENSURE AXIS LABELS ARE BLACK AND SIZE 16
      ticks.length = ggplot2::unit(0.2, "cm"), #INCREASE SIZE OF AXIS TICK MARKS TO BE MORE NOTICEABLE.
      ticks = ggplot2::element_line(color = line_color, linewidth = ggplot2::rel(0.75)),
    ) +
    ggplot2::theme_sub_axis_x(title = ggplot2::element_text(margin = ggplot2::margin(t = 10))) +
    ggplot2::theme_sub_axis_y(title = ggplot2::element_text(vjust = 0.5, margin = ggplot2::margin(r = 15), angle = 90)) +
    ggplot2::theme_sub_legend(
      box = "vertical", #<--MAKES MULTIPLE LEGENDS GO VERTICAL
      title = ggplot2::element_text(color = text_color, face = "bold", family = "sans", size = ggplot2::rel(1.12)),
      text = ggplot2::element_text(size = ggplot2::rel(1), color = text_color),
      key = ggplot2::element_rect(fill = "transparent", color = "white"),
      background = ggplot2::element_rect(color = NA, fill = background_color),
      ticks.length = ggplot2::unit(0.2, "cm"),
      ticks = ggplot2::element_line(color = "white", linewidth = ggplot2::rel(0.75), linetype = "solid"), #MAKE THE TICKS WHITE
      frame = ggplot2::element_rect(color = line_color, linewidth = ggplot2::rel(1)), #MAKE SOLID BLACK LINES FOR THE LEGEND BORDER FOR CONTINUOUS SCALES.
    ) +
    ggplot2::theme_sub_panel(
      border = ggplot2::element_blank(),
      grid = ggplot2::element_blank(), #ELIMINATE MAJOR AND MINOR GRIDLINES
      background = ggplot2::element_rect(fill = background_color, color = NA), #SWITCH FROM GRAY TO WHITE BACKGROUND
      spacing = ggplot2::unit(1, "cm"),
    ) +
    ggplot2::theme_sub_plot(
      background = ggplot2::element_rect(fill = background_color, color = NA),
      title = ggplot2::element_blank(),
      subtitle = ggplot2::element_blank(),
    ) +
    ggplot2::theme_sub_strip(
      background = ggplot2::element_rect(color = NA, fill = background_color),
      text = ggplot2::element_text(color = text_color, size = ggplot2::rel(1), face = "bold"),
      text.y = ggplot2::element_text(margin = ggplot2::margin(l=5), angle = 0),
      text.x = ggplot2::element_text(margin = ggplot2::margin(b=5)),
      placement = "outside", #THIS ALWAYS ENSURES THAT AXIS LABELS GO CLOSER TO THE AXIS THAN THE STRIP LABELS WOULD.
    ) +
    ggplot2::theme(
      geom = ggplot2::element_geom( #IN HERE IS WHERE WE CAN NOW ADD GENERAL GEOM_*-RELATED STYLE OPINIONS.
        pointsize = 5,
        pointshape = 21,
        borderwidth = 1.2,
        colour = "black",
        linetype = "solid",
        linewidth = 1.35
      ),
    )

  #THIS NEXT LINE ENSURES THAT WE JUST RETURN A THEME OPTION RIGHT AWAY SO GGPLOT2 CAN HANDLE ALL THE COLLISIONS AND ADDING AS IT NORMALLY WOULD. NO NEED FOR A GGPLOT.ADD DISPATCH!
  #WE COLLIDE WITH A BASE GGPLOT2 THEME LAST IN CASE THE USER PROVIDES ANY CUSTOM GGPLOT2 THEMING OF THEIR OWN HERE, AS A CONVENIENCE.
  theme_plus2add = default_theme + .determine_legend_theme(legend_pos) + gg_palette_theme + user_theme

  ThemePlus(
    applyGeomDefaults = TRUE,
    theme2add = theme_plus2add,
    override_legend_alphasize = override_legend_alphasize,
    enable_coaching = enable_coaching
  )

}



#' Create and add a scatterplot layer to your `ggplot2` graph with new, distinctive shapes.
#'
#' This function behaves similarly to `ggplot2::geom_point()` except that it takes several new inputs: `shapes`, `n_shapes`, `shape_values`, `legend_title`, `key_size`, and `show_shape_scale`. These are explained below.
#'
#' Collectively, these inputs allow `geom_point_plus()` to access and draw several new and distinctive shapes that are designed to be more readily distinguishable from one another when shape communicates difference.
#'
#' To see the special shapes available via this function run `geom_point_plus_shapes()`.
#'
#' Note: As of Version 0.5.2, shapes 21-25 in R's default shapes palette are now also available via `geom_point_plus_shapes()`; these are called "circle", "square", "diamond", "triangle_up", and "triangle_down", respectively, though they can also be referred to by number.
#'
#' @param mapping Set of aesthetic mappings created by aes(), as in `ggplot2::geom_point()`.
#' @param data The data to be displayed in this layer, as in `ggplot2::geom_point()`.
#' @param stat The statistical transformation to use on the data for this layer, as in `ggplot2::geom_point()`.
#' @param position A position adjustment to use on the data for this layer, as in `ggplot2::geom_point()`.
#' @param avail_shapes A named list of custom shapes to be drawn in place of `ggplot2`'s standard palette of shapes. Defaults to `NULL` and is replaced internally with the palette of shapes designed specifically for use in `geom_point_plus()`. This should probably not be changed unless users have created new shapes they would like to use instead.
#' @param n_shapes A length-1 integer corresponding to the number of distinct shapes the function is allowed to pull from the shapes palette specified to `avail_shapes`. Defaults to the length of `avail_shapes` and should probably not be changed.
#' @param chosen_shapes A character string referring by name to elements in the current shapes registry that the function should use to allocate shapes to values, e.g. `c("flower", "octagon", "squircle)`. These are provided internally to a `scale_shape_manual()` call and are meant to circumvent the need for such a call to specify a specific subset of shapes to be used from the new shapes palette. Defaults to `NULL`, i.e., shapes are pulled from `shapes.list` in order. Numerical values will use `ggplot2`'s default shapes instead.
#' @param legend_title A length-1 character string corresponding to the name to be used for the shape legend title (if any). This is passed internally to `scale_shape_manual()` and is meant to help circumvent the need for the user to specify any such call directly.
#' @param legend_labels A character vector corresponding to the names to be used for the shape legend labels (if any). This is passed internally to `scale_shape_manual()` and is meant to help circumvent the need for the user to specify any such call directly.
#' @param include_shape_legend Logical indicating whether a shape legend will be shown (one is always shown unless this is set to FALSE, even when shape is being mapped to a constant and thus a legend may not be appropriate).
#' @param ... Other arguments passed on to this layer()'s params argument, as in `ggplot2::geom_point()`.
#' @param na.rm Logical value controlling whether missing values should be removed from the data with a warning or silently, as in `ggplot2::geom_point()`.
#' @param show.legend Logical value controlling whether this layer should be included in the legend(s), as in `ggplot2::geom_point()`.
#' @param inherit.aes Logical controlling whether global aesthetics specified in `ggplot2::ggplot()` should be inherited locally by this layer or not, as in `ggplot2::geom_point()`.
#' @param show_shape_scale Logical controlling whether a call to `ggplot2::shape_scale_manual()` should be included as part of the function's operations. Generally, this should be set to `TRUE` unless shape is being mapped to a constant, in which case leaving this `TRUE` would trigger a legend that is redundant.
#' @return A ggplot2 layer object.
#' @examples
#' ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg, fill = drat)) +
#' geom_point_plus(ggplot2::aes(shape = factor(gear)), size = 5)
#' ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg, fill = factor(cyl))) +
#' geom_point_plus(ggplot2::aes(shape = factor(carb)),
#' shape_values = c("squircle", "lotus", "sunburst", "octagon", "cross", "oval"),
#' size = 5, stroke = 0.4)
#' ggplot2::ggplot(iris, ggplot2::aes(Petal.Width, Petal.Length, fill = Species)) +
#' geom_point_plus(ggplot2::aes(shape = Species), size = 5, alpha = 0.7)
#'
#' @export
geom_point_plus = function(mapping = NULL,
                           data = NULL,
                           stat = "identity",
                           position = "identity",
                           avail_shapes = NULL, #A NAMED LIST OF SHAPES. DEFAULTS TO THOSE PROVIDED BY ggplotplus PLUS THOSE ADDED BY THE USER VIA ADD_SHAPE_PLUS. BUT NOT EVALUATED HERE--EVALUATED INSIDE FUNCTION ENVIRON.
                           n_shapes = length(avail_shapes), #HOW MANY DISTINCT SHAPES SHOULD BE PULLED FROM THE AVAILABLE PALETTE? DEFAULTS TO ALL OF THEM.
                           chosen_shapes = NULL, #WE PROVIDE DIRECT ACCESS TO THE VALUES ARGUMENT OF SCALE_SHAPE_MANUAL VIA THIS PARAMETER. THIS WAY, A USER NEEDN'T TACK ON AN ADDITIONAL CALL TO SCALE_SHAPE_MANUAL() TO CUSTOMIZE THE SHAPES USED.
                           legend_title = NULL, #WE ALSO PROVIDE DIRECT ACCESS TO THE TITLE ARGUMENT OF THE LEGEND, AS CHANGING THIS MANUALLY WOULD OTHERWISE REQUIRE ANOTHER CALL TO SCALE_SHAPE_DISCRETE AND THAT WOULD TRIGGER A WARNING AND RESET TO THE SHAPES PALETTE GGPLOT2 GENERALLY USES.
                           legend_labels = NULL, #SAME AS ABOVE.
                           include_shape_legend = TRUE, #WE PROVIDE DIRECT ACCESS TO WHETHER OR NOT A SHAPE LEGEND GETS SHOWN, FOR USE IN SINGLE-SHAPE SCATTERPLOTS WHERE THE CUSTOM SHAPES ARE USED INSTEAD OF GGPLOT2 DEFAULTS.
                           ...,
                           na.rm = FALSE,
                           show.legend = NA,
                           inherit.aes = TRUE,
                           show_shape_scale = TRUE) {

  #EVALUATES THE SHAPES REGISTRY HERE, UNLESS A DIFFERENT ONE IS PROVIDED.
  if(is.null(avail_shapes)) {
    avail_shapes = .pointplus_shapes()
  }

  dot_args = rlang::list2(...)

  #IF SHAPE IS NOT BEING MAPPED, JUST RETURN THE LAYER WITHOUT DRAWING ANY SCALE
  #THIS ONLY CHECKS MAPPING, WHICH IS LOCAL. IF SHAPE IS MAPPED GLOBALLY INSTEAD, A SCALE_SHAPE_MANUAL CALL IS PRETTY HARMLESS.
  shape_is_mapped = .has_mapped_aes(mapping, dot_args, "shape")

  chosen_shapes = .standardize_pointplus_shape_names(chosen_shapes) #SEE MIDDLEWARE FOR THIS HELPER.

  #IF CHOSEN_SHAPES IS A CONSTANT, PASS IT ALONG AS A CONSTANT AND NOT AN AES.
  if(!shape_is_mapped &&
     !is.null(chosen_shapes) &&
     length(chosen_shapes) == 1 &&
     is.character(chosen_shapes) &&
     chosen_shapes %in% names(avail_shapes)) {
    dot_args$shape = chosen_shapes
    chosen_shapes = NULL
  }

  #THIS FUNCTION IS MOSTLY JUST A WRAPPER TO GEOM_POINT2 INTERNALLY.
  geom_call = do.call(
    geom_point2,
    c(
      list(
        mapping = mapping,
        data = data,
        stat = stat,
        position = position,
        shapes = avail_shapes,
        na.rm = na.rm,
        show.legend = if(isTRUE(include_shape_legend)) {show.legend}  else {c(shape = FALSE)},
        inherit.aes = inherit.aes
      ),
      dot_args
    )
  )

  #HERE, SELECT THE EXACT SHAPES TO DRAW FROM THE SHAPES PALETTE, DEFAULTING TO THE SHAPES IN THE REGISTRY IN ORDER IF NO SPECIFIC SHAPES WERE CHOSEN.
  if(is.null(chosen_shapes) || length(chosen_shapes) == 0) {
    values = rep(names(avail_shapes), length.out = n_shapes)
  } else {
    values = rep(chosen_shapes, length.out = n_shapes)
  }

  #BUILD THE LEGEND IF WE'RE GOING TO, BUT ONLY PUT IN TITLE IF THE USER PROVIDED ONE.
  guide_obj = if(isTRUE(include_shape_legend)) {
    ggplot2::guide_legend()
  } else {
    "none"
  }

  scale_args = list(values = values, guide = guide_obj)

  #ADD THE SHAPE SCALE
  if(!is.null(legend_title)) {
  scale_args$name = legend_title
  }
  if(!is.null(legend_labels)) {
  scale_args$labels = legend_labels
  }

  scale_call = do.call(ggplot2::scale_shape_manual, scale_args)

  #AND ADD A SIZE SCALE TO MAKE THE DEFAULT SIZES A BIT HEFTIER.
  if(is.null(.partial_match_user_arg(dot_args, "size")) &&
     isTRUE(any(c("size") %in% names(mapping)))) {
    size_scale_call = ggplot2::scale_size(range = c(3, 5.5))
  } else {
    size_scale_call = NULL
  }

  #IF THE USER HAS MAPPED SHAPE GLOBALLY + DOES NOT WANT A SCALE_SHAPE_MANUAL CALL (BECAUSE THEY'VE MAYBE SET SHAPE TO A CONSTANT THERE), THIS WILL PREVENT THE SCALE FROM SHOWING BY SUPPRESSING THE SCALE CALL IF THEY'VE TOGGLED THIS.
  if(show_shape_scale == FALSE) {
    return(geom_call)
  }

  #BE MORE EXPLICIT ABOUT THE OUTPUT STRUCTURE.
  Filter(Negate(is.null), list(geom_call, scale_call, size_scale_call))

}


#' Jittered points with ggplotplus point shapes
#'
#' \code{geom_jitter_plus()} is a convenience wrapper around
#' \code{geom_point_plus()} that applies jittering to reduce overplotting.
#' It supports the same custom shape palette and fillable point rendering
#' as \code{geom_point_plus()} while exposing the familiar \code{width},
#' \code{height}, and \code{seed} arguments used by
#' \code{ggplot2::position_jitter()}.
#'
#' @inheritParams ggplot2::geom_point
#' @param width,height Amount of horizontal and vertical jitter. Passed to
#'   \code{ggplot2::position_jitter()} when \code{position = "jitter"}.
#' @param seed Random seed used by \code{ggplot2::position_jitter()} to make
#'   jittering reproducible. Defaults to \code{NA}, matching `ggplot2`.
#' @param ... Additional arguments passed to \code{geom_point_plus()}, including
#'   ggplotplus-specific arguments such as \code{chosen_shapes} and
#'   \code{legend_title}. See `?geom_point_plus` for details.
#'
#' @return A ggplot2 layer.
#'
#' @examples
#' ggplot2::ggplot(iris, ggplot2::aes(Species, Sepal.Length)) +
#'   geom_jitter_plus(
#'     ggplot2::aes(shape = Species, fill = Petal.Length),
#'     width = 0.15,
#'     seed = 1,
#'     colour = "black"
#'   )
#'
#' ggplot2::ggplot(iris, ggplot2::aes(Species, Sepal.Length)) +
#'   geom_jitter_plus(
#'     ggplot2::aes(shape = Species),
#'     chosen_shapes = c("plus", "flower", "lotus"),
#'     seed = 123
#'   )
#'
#' @export
geom_jitter_plus = function(mapping = NULL,
                            data = NULL,
                            stat = "identity",
                            position = "jitter",
                            ...,
                            width = NULL,
                            height = NULL,
                            seed = NA,
                            na.rm = FALSE,
                            show.legend = NA,
                            inherit.aes = TRUE) {

  #SANITIZING ANY PARTIAL MATCHING INPUTS.
  dot.args = list(...)
  anywidth = .partial_match_user_arg(args = dot.args, target = "width")
  if(length(anywidth) > 0) {
    width = anywidth
    dot.args = .remove_partial_match_user_arg(dot.args, "width")
  }
  anyheight = .partial_match_user_arg(args = dot.args, target = "height")
  if(length(anyheight) > 0) {
    width = anyheight
    dot.args = .remove_partial_match_user_arg(dot.args, "height")
  }
  anyseed = .partial_match_user_arg(args = dot.args, target = "seed")
  if(length(anyseed) > 0) {
    width = anyseed
    dot.args = .remove_partial_match_user_arg(dot.args, "seed")
  }

  #IF WIDTH OR HEIGHT IS NOT NULL, AND/OR SEED IS NOT NULL, AND WE'RE ON POSITION JITTER...
  if(!is.null(width) || !is.null(height) || !is.na(seed)) {
    if(!identical(position, "jitter")) {
      warning(
        "`width`, `height`, and `seed` are ignored when `position` is not \"jitter\".",
        call. = FALSE
      )
    } else { #...PASS TO GGPLOT'S POSITION_JITTER.
      position = ggplot2::position_jitter(
        width = width,
        height = height,
        seed = seed
      )
    }
  }

  #FROM THERE ON OUT, IT'S ALL GEOM_POINT_PLUS'S JOB.
  geom_point_plus(
    mapping = mapping,
    data = data,
    stat = stat,
    position = position,
    ...,
    na.rm = na.rm,
    show.legend = show.legend,
    inherit.aes = inherit.aes
  )
}



#' Add direct labels to grouped point or line plots
#'
#' [direct_labels_plus()] adds procedurally placed text labels to grouped
#' point or line plots as an alternative to using a legend, which might be
#' space-inefficient, off to one side away from where readers will see it,
#' or force readers to jump their focus long distances to align groups with
#' labels. The function is intended as a friendlier alternative to
#' manually placing group labels via [ggplot2::annotate()]. Labels are drawn
#' using [ggrepel::geom_label_repel()], so they'll repel one another as well
#' as the plotted data they're labeling (within reason).
#'
#' This function is experimental. It currently works best for ordinary
#' scatterplots and grouped line/path plots where each group has a visually
#' meaningful position in two-dimensional (x/y) space. It will not
#' work for every ggplot2 geometry, statistic, coordinate system, or faceting
#' arrangement.
#'
#' @param data A data frame containing the variables to be both plotted and
#' labelled. Most often, this will be the same data frame as supplied to
#' [ggplot2::ggplot()] but needn't be.
#' @param x,y Unquoted column names giving the x- and y-coordinates of the data
#' to be plotted and thus also labelled.
#' @param group Unquoted column name giving the grouping variable with which to
#' label the underlying data. Must be a single unquoted column name and not an
#' expression.
#' @param placement Where labels should be placed relative to each group.
#' One of "top", "right", "bottom", or "left". For geometry = "point", this
#' controls the target location used to choose a representative point from each
#' group. For geometry = "line", this chooses the endpoint or extreme point to
#' label. Experimenting with different placements to find the one that works
#' best for a particular graph is advised!
#' @param geometry The kind of geometry being labelled. Currently supports
#' "point" and "line".
#' @param adj_fact A single numeric value controlling how far label targets are
#' adjusted toward or away from the selected edge of each group. Values are
#' interpreted as a proportion of the group-specific x- or y-range. Positive
#' values move targets outward (towards the plot edge); negative values move
#' them inward. For geometry = "point", this changes the target spot used to
#' choose a particular point from each group to label, but does not move the
#' final label anchor away from the selected point. In practice, this is likely
#' not particularly useful for point geometries most of the time as a result.
#' @param key_labels Optional replacement contents for the labels. May be one
#' of:
#' \itemize{
#' \item NULL, in which case group values are used as labels (the default
#' behavior);
#' \item A labelling function;
#' \item a named character vector of the form
#' c("old_group_name" = "New label"); or
#' \item an unnamed character vector with one label per group. Unnamed
#' labels will be assigned to groups in alphanumeric order.
#' }
#' @param facet_vars Optional character vector of max length 2 (or else NULL)
#' giving one or two faceting variables you're using to facet your plot. When
#' supplied, label locations are calculated separately within each
#' group-by-facet combination. This is useful when adding direct labels to
#' faceted plots.
#' @param ... Additional arguments passed along to
#' [ggrepel::geom_label_repel()].
#'
#' @return A ggplot2 layer produced by [ggrepel::geom_label_repel()].
#'
#' @details
#' For point geometries, direct_labels_plus() calculates one label location
#' per group by finding the observed point closest to a group-specific target
#' positioned towards one of the "edges" of a group's cluster of points.
#' For "top" and "bottom" placement, the target is horizontally centered on
#' the group's median x-value and vertically placed near the group's maximum or
#' minimum y-value. For "left" and "right" placement, the same logic is
#' applied with x and y reversed.
#'
#' For line geometries, labels are placed at the group-specific endpoint or
#' extreme value implied by placement: the largest x-value for "right", the
#' smallest x-value for "left", the largest y-value for "top", and the
#' smallest y-value for "bottom". The final label anchor may then be adjusted
#' by adj_fact.
#'
#' To help labels repel from plotted points or lines, the function silently adds
#' empty-label rows at the original data coordinates. ggrepel does not draw
#' these empty labels, but still uses their positions when placing visible
#' labels. This helps to ensure, in general, that labels neither cover each other
#' nor the underlying data they're labelling.
#'
#' Currently, the function sets defaults for the following parameters of
#' [ggrepel::geom_label_repel()]: size (5), box.padding (0.5), max.overlaps
#' (Inf), segment.size (1), and min.segment.length (0). However, these are
#' overridable, if the user provides named arguments that at least partially
#' match.
#'
#' Known limitations:
#' \itemize{
#' \item Label locations are calculated in the raw data space supplied to the
#' function. Transformed scales, reversed axes, and non-Cartesian coordinate
#' systems may give unexpected results. Pre-transform data rather than entering
#' raw data and transforming later.
#' \item coord_flip(), coord_polar(), map projections, and other coordinate
#' transformations are not currently supported.
#' \item The function places labels according to the data values supplied in
#' `data`, not those subsequently generated by ggplot2 stat functions. For
#' example, to label fitted smooth lines like those from
#' [ggplot2::geom_smooth()], feed this function fitted values outputted from
#' such a function instead of the raw data.
#' \item Very dense plots or plots with many groups will likely still have
#' overlapping or poorly placed labels. ggrepel helps, but it cannot make a
#' crowded plot uncrowded. Alas.
#' \item Polygon, ribbon, area, segment, curve, and spatial geometries are not
#' currently supported but might be in the future.
#' }
#'
#' @examples
#' ggplot2::ggplot(iris, ggplot2::aes(Sepal.Length, Sepal.Width, color = Species)) +
#' ggplot2::geom_point() +
#' direct_labels_plus(
#' data = iris,
#' x = Sepal.Length,
#' y = Sepal.Width,
#' group = Species,
#' placement = "right",
#' geometry = "point"
#' ) +
#' ggplot2::guides(color = "none")
#'
#' line_data = ChickWeight |>
#' dplyr::group_by(Diet, Time) |>
#' dplyr::summarize(weight = mean(weight), .groups = "drop")
#'
#' ggplot2::ggplot(line_data, ggplot2::aes(Time, weight, color = Diet)) +
#' ggplot2::geom_line(ggplot2::aes(group = Diet)) +
#' direct_labels_plus(
#' data = line_data,
#' x = Time,
#' y = weight,
#' group = Diet,
#' placement = "right",
#' geometry = "line",
#' adj_fact = 0.05
#' ) +
#' ggplot2::guides(color = "none")
#'
#' @export
direct_labels_plus = function(data,
                              x,
                              y,
                              group, #WHAT TO LABEL BY
                              placement = "top", #WHERE TO PLACE THE LABELS RELATIVE TO THE EDGES OF THE PLOT
                              geometry = "point", #LINES VS. POINTS
                              adj_fact = 0, #HOW MUCH TO MOVE THE LABELS TOWARDS THE EDGE (POSITIVE) OR CENTER (NEGATIVE) OF THE PLOT
                              key_labels = NULL, #WHAT TO PLACE INSIDE THE LABELS.
                              facet_vars = NULL,
                              ...) {

  #CONFIRM VALID ARGS
  placement = match.arg(placement, c("top", "right", "bottom", "left"))
  geometry = match.arg(geometry, c("point", "line"))


  ##EARLY FAILURE CHECKS
  if(!is.data.frame(data)) {
    stop("`data` must be a data frame.", call. = FALSE)
  }

  if(!is.null(facet_vars)) {

    if(!is.character(facet_vars) || length(facet_vars) > 2) {
      stop("`facet_vars` must be NULL or a character vector <= 2 column names.", call. = FALSE)
    }

    missing_facet_vars = setdiff(facet_vars, names(data))

    if(length(missing_facet_vars) > 0) {
      stop(
        "`facet_vars` contains column name(s) not found in `data`: ",
        paste(missing_facet_vars, collapse = ", "),
        call. = FALSE
      )
    }

  }

  if(!is.numeric(adj_fact) || length(adj_fact) != 1 || is.na(adj_fact)) {
    stop("`adj_fact` must be a single numeric value (a proportion of the horizontal/vertical range of the data to adjust the label targets by).", call. = FALSE)
  }

  group_quo = rlang::enquo(group)

  if(!rlang::quo_is_symbol(group_quo)) {
    stop("`group` must be a bare column name, not an expression.", call. = FALSE)
  }

  group_name = rlang::as_name(group_quo)

  #PASS TO CORRECT HELPER ACCORDING TO GEOMETRY
  if(geometry == "point") {

    label_data = .directlabel_points(data = data,
                                    x = {{x}},
                                    y = {{y}},
                                    group = {{group}},
                                    placement = placement,
                                    adj_fact = adj_fact,
                                    facet_vars = facet_vars)

  } else if(geometry == "line") {

    label_data = .directlabel_lines(data = data,
                                   x = {{x}},
                                   y = {{y}},
                                   group = {{group}},
                                   placement = placement,
                                   adj_fact = adj_fact,
                                   facet_vars = facet_vars)

  }

  if(nrow(label_data) == 0) {
    stop("No complete x/y/group observations are available for direct labeling.", call. = FALSE)
  }

  #TO REPEL AWAY FROM THE LINES/POINTS AS WELL AS OTHER LABELS, WE SILENTLY ADD THE GEOMETRIES AS LABEL-LESS ENTITIES
  invis_points =  data %>%
    dplyr::transmute(
      x = {{x}},
      y = {{y}},
      .label_plus = "",
      dplyr::across(dplyr::any_of(c(facet_vars, group_name)))
    )

  #IF USERS SUPPLIED CUSTOM LABELS, WE CAN APPLY THEM. A NAMED VEC OF OLD = NEW IS ADVISED.

  label_data$.label_plus = .apply_key_labels_plus(label_data = label_data$.label_plus, key_labels = key_labels)

  #BIND THE FAKE AND REAL LABEL DATA TOGETHER.
  all_label_data = dplyr::bind_rows(label_data, invis_points)

  ##PARTIAL MATCHING AND ARGUMENT COLLISIONS--TO SET DEFAULTS THAT ARE OVERRIDABLE.
  dot.args = list(...)
    any_size = .partial_match_user_arg(dot.args, "size")
    any_boxpad = .partial_match_user_arg(dot.args, "box.padding")
    any_maxover = .partial_match_user_arg(dot.args, "max.overlaps")
    any_segsize = .partial_match_user_arg(dot.args, "segment.size")
    any_minseg = .partial_match_user_arg(dot.args, "min.segment.length")

    if(length(any_size) > 0) { use_size = any_size; dot.args = .remove_partial_match_user_arg(dot.args, "size") } else { use_size = 5 }
    if(length(any_boxpad) > 0) { use_boxpad = any_boxpad; dot.args = .remove_partial_match_user_arg(dot.args, "box.padding") } else { use_boxpad = 0.5 }
    if(length(any_maxover) > 0) { use_maxover = any_maxover; dot.args = .remove_partial_match_user_arg(dot.args, "max.overlaps") } else { use_maxover = Inf }
    if(length(any_segsize) > 0) { use_segsize = any_segsize; dot.args = .remove_partial_match_user_arg(dot.args, "segment.size") } else { use_segsize = 1 }
    if(length(any_minseg) > 0) { use_minseg = any_minseg; dot.args = .remove_partial_match_user_arg(dot.args, "min.segment.length") } else { use_minseg = 0 }

  #THEN FINALLY PASS THIS ALL ALONG, ALONG WITH ANY DOT ARGS, TO GEOM_LABEL_REPEL.
  do.call(
    ggrepel::geom_label_repel,
    c(
      list(
    data = all_label_data,
    ggplot2::aes(x = x, y = y, label = .label_plus),
    color = "black",
    inherit.aes = FALSE,
    show.legend = FALSE,
    size = use_size,
    box.padding = use_boxpad,
    max.overlaps = use_maxover,
    segment.size = use_segsize,
    min.segment.length = use_minseg),
    dot.args
   )
  )

}



#' Demo plot showing geom_point_plus() shapes
#'
#' A prebuilt ggplot object displaying the nine custom point shapes designed specifically for use in `geom_point_plus()`. As of Version 0.5.2, other shapes are also available, but those are not shown via this function. See the documentation for `geom_point_plus()` for details.
#'
#' @format A ggplot object.
#'
#' @return Returns a ggplot2 graph showing the ggplotplus shapes palette.
#'
#' @export
geom_point_plus_shapes = function() {

  ggplot2::ggplot(data = data.frame(x = rep(c(0.5,1.5,2.5), each = 3),
                                                           y = rep(c(1,2,3), times = 3),
                                                           shape = factor(1:9))) +
  geom_point_plus(ggplot2::aes(x = .data$x, y = .data$y, shape = .data$shape, fill = .data$shape), chosen_shapes = c("squircle", "octagon", "flower", "economy", "plus", "waffle", "oval", "sunburst", "lotus"),
                  size = 10, stroke = 1)+
  ggplot2::theme_minimal() +
  ggplot2::lims(y=c(0.5, 3.5), x = c(0.4, 3)) +
  ggplot2::annotate("text", x = 0.8, y = 1, label = "Closed\nRounded\nUncrossed", size = 4) +
  ggplot2::annotate("text", x = 0.8, y = 2, label = "Closed\nPointed\nUncrossed", size = 4) +
  ggplot2::annotate("text", x = 0.8, y = 3, label = "Closed\nRounded\nCrossed", size = 4) +
  ggplot2::annotate("text", x = 1.8, y = 1, label = "Open\nPointed\nUncrossed", size = 4) +
  ggplot2::annotate("text", x = 1.82, y = 2, label = "Intermediate", size = 4) +
  ggplot2::annotate("text", x = 1.8, y = 3, label = "Open\nPointed\nCrossed", size = 4) +
  ggplot2::annotate("text", x = 2.8, y = 1, label = "Open\nRounded\nUncrossed", size = 4) +
  ggplot2::annotate("text", x = 2.8, y = 2, label = "Closed\nPointed\nCrossed", size = 4) +
  ggplot2::annotate("text", x = 2.8, y = 3, label = "Open\nRounded\nCrossed", size = 4) +
  ggplot2::theme(legend.position = "none",
                 axis.text = ggplot2::element_blank(),
                 axis.title = ggplot2::element_blank(),
                 axis.line = ggplot2::element_blank(),
                 axis.ticks = ggplot2::element_blank(),
                 axis.title.x = ggplot2::element_blank(),
                 axis.title.y = ggplot2::element_blank(),
                 panel.grid = ggplot2::element_blank()) +
  ggplot2::annotate("text", x = 0.5, y = 0.7, label = "squircle", size = 5, fontface = 'bold') +
  ggplot2::annotate("text", x = 0.5, y = 1.7, label = "octagon", size = 5, fontface = 'bold') +
  ggplot2::annotate("text", x = 0.5, y = 2.7, label = "flower", size = 5, fontface = 'bold') +
  ggplot2::annotate("text", x = 1.5, y = 0.7, label = "economy", size = 5, fontface = 'bold') +
  ggplot2::annotate("text", x = 1.5, y = 1.7, label = "plus", size = 5, fontface = 'bold') +
  ggplot2::annotate("text", x = 1.5, y = 2.7, label = "waffle", size = 5, fontface = 'bold') +
  ggplot2::annotate("text", x = 2.5, y = 0.7, label = "oval", size = 5, fontface = 'bold') +
  ggplot2::annotate("text", x = 2.5, y = 1.7, label = "sunburst", size = 5, fontface = 'bold') +
  ggplot2::annotate("text", x = 2.5, y = 2.7, label = "lotus", size = 5, fontface = 'bold')
}


#' Add a custom shape for geom_point_plus()
#'
#' Registers a custom point shape for use with \code{geom_point_plus()}.
#' Custom shapes are defined by a data frame of connected polygon coordinates
#' and, once validated, are stored in a session-level shape registry.
#'
#' A shape must be supplied as a data frame with columns \code{x}, \code{y},
#' and \code{piece}. The \code{x} and \code{y} columns define the vertices of
#' the shape, centered around \code{0, 0}. Coordinates should generally be
#' scaled to about \code{+/-0.4} to match the scaling of built-in point shapes.
#' The \code{piece} column identifies separate polygon pieces within the same
#' shape to generate "holes," as appropriate.
#'
#' For inspiration and to model inputs, see `ggplotplus_shapes_list` for the
#' structure of the built-in point shapes.
#'
#' @param name A single character string giving the name of the new shape.
#' @param shape A data frame with columns \code{x}, \code{y}, and \code{piece}.
#' @param overwrite Logical. If \code{FALSE}, the default, an error is returned
#'   when \code{name} already exists in the shape registry. If \code{TRUE}, the
#'   existing shape is replaced with the newly provided shape. Useful if, e.g.,
#'   you'd like to scale an existing shape up or down in size.
#' @param ... Additional arguments. Partial argument matching is supported for
#' friendly UX.
#'
#' @return Invisibly returns the registered shape name.
#'
#' @examples
#' test_star = data.frame(
#'   x = c(0.000,  0.118,  0.380,  0.190,  0.235,
#'         0.000, -0.235, -0.190, -0.380, -0.118),
#'   y = c(0.400,  0.124,  0.124, -0.047, -0.324,
#'        -0.153, -0.324, -0.047,  0.124,  0.124),
#'   piece = 1
#' )
#'
#' add_shape_plus("test_star", test_star)
#'
#' @export
add_shape_plus = function(name = NULL,
                                    shape = NULL,
                                    overwrite = FALSE,
                          ...) {

  #TRY TO PARTIAL MATCH NAME FIRST.
  dot.args = list(...)
  if(length(dot.args) > 0) {
  any_name = .partial_match_user_arg(dot.args, "name")
  if(length(any_name) > 0) {
    name = any_name
    dot.args = .remove_partial_match_user_arg(dot.args, "name")
  }

  #IF UNNAMED FIRST ARG COULD REASONABLY BE NAME, PUT IN THAT FORMAL NAME.
  if(is.character(dot.args[[1]]) &&
     length(dot.args[[1]]) == 1) {
    name = dot.args[[1]]
    dot.args[[1]] = NULL
   }

  }

  #SAFETY CHECK--DO WE HAVE A SINGLE CHARACTER NAME THAT ISN'T BLANK?
  if(!is.character(name) || length(name) != 1 || is.na(name) || name == "") {
    stop("`name` must be a single non-missing character string. Give your new shape a name!", call. = FALSE)
  }


  name = trimws(name)
  name = .standardize_pointplus_shape_names(name) #PREVENTS A USER FROM ADDING A "21" OR SOMETHING.

  if(name == ".initialized") {
    stop("`.initialized` is reserved for internal ggplotplus use. Do not call shapes by that name.", call. = FALSE)
  }

  .initialize_pointplus_shape_registry()

  #HANDLES SITUATION WHERE A USER TRIES TO ADD A SHAPE WE'VE ALREADY GOT.
  if(exists(name, envir = .pointplus_shape_registry, inherits = FALSE) && !overwrite) {
    stop(
      "A pointplus shape named `", name, "` already exists. ",
      "Use `overwrite = TRUE` to replace it.",
      call. = FALSE
    )
  }


  #NOW, DO THE SAME CLEANING ON SHAPE
  if(length(dot.args) > 0) {
    any_shape = .partial_match_user_arg(dot.args, "shape")
    if(length(any_shape) > 0) {
      shape = any_shape
      dot.args = .remove_partial_match_user_arg(dot.args, "shape")
    }

    #IF UNNAMED FIRST REMAINING ARG COULD REASONABLY BE SHAPE, PUT IN THAT FORMAL NAME.
    if(is.data.frame(dot.args[[1]]) &&
       ncol(dot.args[[1]]) >= 3 &&
       all(c("x", "y", "piece") %in% names(dot.args[[1]]))) { #<--SHOULD BE THREE COLS W/ THESE NAMES AT LEAST.
      shape = dot.args[[1]]
      dot.args[[1]] = NULL
    }

  }

  shape = .validate_pointplus_shape(shape, name = name) #NOW ACTUALLY TRY TO USE THE SHAPE THEY'VE GIVEN.

  assign(name, shape, envir = .pointplus_shape_registry) #IF WE GET THIS FAR, ADD THE SHAPE TO THE REGISTRY.

  #MAKE SURE TO PRESERVE THE INTENDED ORDER AND APPEND USER-ADDED SHAPES TO THE END.
  if(!name %in% .pointplus_shape_registry$.order) {
    .pointplus_shape_registry$.order = c(.pointplus_shape_registry$.order, name)
  }

  if(length(dot.args) > 0) {
    dot.args = NULL
    warning("Additional, ambiguous arguments were provided but not used. ")
  }

  invisible(name) #AN INVISIBLE RETURN FOR SAFETY.
}


#' Convert a ggplotplus plot into a grob
#'
#' Convenience helper for converting ggplotplus plot objects into grobs using
#' \code{ggplot2::ggplotGrob()}.
#'
#' This is primarily intended for compatibility with packages such as
#' \pkg{cowplot} that may not yet fully recognize custom S7 plot subclasses
#' during internal dispatch.
#'
#' @param plot A ggplot or ggplotplus plot object.
#'
#' @return A grob object suitable for use with grid-based plotting utilities
#'   such as \code{cowplot::plot_grid()}.
#'
#' @examples
#' p = ggplot2::ggplot(iris, ggplot2::aes(Sepal.Length, Petal.Length, colour = Species)) +
#'   geom_point_plus()
#'
##' if(requireNamespace("cowplot", quietly = TRUE)) {
#'
#' cowplot::plot_grid(
#'   ggplotplus_to_cowplot(p),
#'   ggplotplus_to_cowplot(p)
#' )
#' }
#'
#' @export
ggplotplus_to_cowplot = function(plot) {
  ggplot2::ggplotGrob(plot)
}


#' Convert a ggplotplus plot into a patchwork-compatible element
#'
#' Convenience helper for converting ggplotplus plot objects into
#' patchwork-compatible wrapped elements.
#'
#' This helper is primarily intended for compatibility with \pkg{patchwork},
#' whose plot-composition operators may not yet fully recognize custom S7
#' plot subclasses under ggplot2 4.x.
#'
#' Internally, the plot is first converted into a grob using
#' \code{ggplotplus_to_cowplot()}, then wrapped using
#' \code{patchwork::wrap_elements()}.
#'
#' @param plot A ggplot or ggplotplus plot object.
#'
#' @return A patchwork-compatible wrapped plot element.
#'
#' @examples
#' if(requireNamespace("patchwork", quietly = TRUE)) {
#'
#' p1 = ggplot2::ggplot(iris, ggplot2::aes(Sepal.Length, Petal.Length, colour = Species)) +
#'   geom_point_plus()
#'
#' p2 = ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg, shape = factor(cyl))) +
#'   geom_point_plus()
#'
#' (ggplotplus_to_patchwork(p1) |
#'   ggplotplus_to_patchwork(p2)) +
#'   patchwork::plot_annotation(
#'     title = "ggplotplus + patchwork"
#'   )
#' }
#' @details
#' This helper requires the \pkg{patchwork} package, but ggplotplus does not
#' import patchwork. Users only need patchwork installed if they want to compose
#' ggplotplus plots with patchwork.
#' @export
ggplotplus_to_patchwork = function(plot) {

  if(!requireNamespace("patchwork", quietly = TRUE)) {
    stop(
      "`ggplotplus_to_patchwork()` requires the patchwork package. ",
      "Install it with `install.packages(\"patchwork\")`.",
      call. = FALSE
    )
  }

  patchwork::wrap_elements(full = ggplotplus_to_cowplot(plot))
}

test_that("scale_continuous_plus builds for x and y scales", {
  p = ggplot(iris, aes(Petal.Length, Sepal.Length)) +
    geom_point() +
    scale_continuous_plus(scale = "x", name = "Petal length (cm)") +
    scale_continuous_plus(scale = "y", name = "Sepal length (cm)") +
    theme_plus(enable_coaching = F)

  expect_s3_class(ggplot2::ggplot_build(p), "ggplot_built")
})

test_that("scale_continuous_plus supports thinned labels", {
  p = ggplot(iris, aes(Petal.Length, Sepal.Length)) +
    geom_point() +
    scale_continuous_plus(scale = "x", thin.labels = TRUE)

  expect_s3_class(ggplot2::ggplot_build(p), "ggplot_built")
})

test_that("coaching warns for too many discrete fill values", {
  dat = data.frame(
    x = seq_len(10),
    y = seq_len(10),
    g = factor(seq_len(10))
  )

  p = ggplot(dat, aes(x, y, fill = g)) +
    geom_point(shape = 21) +
    theme_plus(enable_coaching = TRUE)

  expect_message(
    ggplot2::ggplot_build(p),
    "discrete variable with > 7 levels"
  )
})

test_that("coaching can be disabled locally", {
  dat = data.frame(
    x = seq_len(10),
    y = seq_len(10),
    g = factor(seq_len(10))
  )

  p = ggplot(dat, aes(x, y, fill = g)) +
    geom_point(shape = 21) +
    theme_plus(enable_coaching = FALSE)

  expect_message(
    ggplot2::ggplot_build(p),
    regexp = NA
  )
})

test_that("coaching can be disabled globally", {
  old = options(ggplotplus.enable_coaching = FALSE)
  on.exit(options(old), add = TRUE)

  dat = data.frame(
    x = seq_len(10),
    y = seq_len(10),
    g = factor(seq_len(10))
  )

  p = ggplot(dat, aes(x, y, fill = g)) +
    geom_point(shape = 21) +
    theme_plus(enable_coaching = TRUE)

  expect_message(
    ggplot2::ggplot_build(p),
    regexp = NA
  )
})


test_that("bar/column coaching is quiet when ordinary baseline is visible", {

  plot_data = iris %>%
    dplyr::group_by(Species) %>%
    dplyr::summarize(mean_Sepal.Length = mean(Sepal.Length), .groups = "drop")

  p = ggplot2::ggplot(
    plot_data,
    ggplot2::aes(y = Species, x = mean_Sepal.Length, fill = Species)
  ) +
    ggplot2::geom_col() +
    theme_plus() +
    labs(fill = "A title", x = "Another title", y = "A third title")

  expect_message(
    print(p),
    NA
  )

})

test_that("bar/column coaching warns when coord_cartesian crops out ordinary baseline", {

  plot_data = iris %>%
    dplyr::group_by(Species) %>%
    dplyr::summarize(mean_Sepal.Length = mean(Sepal.Length), .groups = "drop")

  p = ggplot2::ggplot(
    plot_data,
    ggplot2::aes(y = Species, x = mean_Sepal.Length, fill = Species)
  ) +
    ggplot2::geom_col() +
    theme_plus() +
    ggplot2::coord_cartesian(xlim = c(4, 8))

  expect_message(
    print(p),
    "omitting 0 distorts"
  )

})

test_that("bar/column coaching is quiet when log10 baseline is visible", {

  plot_data = iris %>%
    dplyr::group_by(Species) %>%
    dplyr::summarize(mean_Sepal.Length = mean(Sepal.Length), .groups = "drop")

  p = ggplot2::ggplot(
    plot_data,
    ggplot2::aes(y = Species, x = mean_Sepal.Length, fill = Species)
  ) +
    ggplot2::geom_col() +
    theme_plus() +
    ggplot2::scale_x_log10() +
    labs(fill = "A title", x = "Another title", y = "A third title")

  expect_message(
    print(p),
    NA
  )

})

test_that("bar/column coaching warns when coord_cartesian crops out log10 baseline", {

  plot_data = iris %>%
    dplyr::group_by(Species) %>%
    dplyr::summarize(mean_Sepal.Length = mean(Sepal.Length), .groups = "drop")

  p = ggplot2::ggplot(
    plot_data,
    ggplot2::aes(y = Species, x = mean_Sepal.Length, fill = Species)
  ) +
    ggplot2::geom_col() +
    theme_plus() +
    ggplot2::scale_x_log10() +
    ggplot2::coord_cartesian(xlim = c(3, 8))

  expect_message(
    print(p),
    "omitting 0 distorts"
  )

})

test_that("bar/column coaching checks all visible panels", {

  plot_data = data.frame(
    group = rep(c("a", "b", "c"), 2),
    facet = rep(c("one", "two"), each = 3),
    value = c(2, 3, 4, 20, 30, 40)
  )

  p = ggplot2::ggplot(
    plot_data,
    ggplot2::aes(x = group, y = value)
  ) +
    theme_plus() +
    ggplot2::geom_col() +
    ggplot2::facet_wrap(ggplot2::vars(facet), scales = "free_y") +
    ggplot2::coord_cartesian(ylim = c(10, 45))

  expect_message(
    print(p),
    "omitting 0 distorts"
  )

})


test_that("bar/column coaching ignores non-bar geoms", {

  p = ggplot2::ggplot(
    iris,
    ggplot2::aes(x = Sepal.Length, y = Sepal.Width)
  ) +
    theme_plus() +
    ggplot2::geom_point() +
    labs(x = "A title", y = "Another title") +
    ggplot2::coord_cartesian(y = c(3, 5))

  expect_message(
    print(p),
    NA
  )

})

test_that("yaxis_title_plus builds on unfaceted plots", {
  p = ggplot(iris, aes(Petal.Length, Sepal.Length)) +
    geom_point() +
    theme_plus(enable_coaching = F) +
    yaxis_title_plus()

  expect_s3_class(ggplot2::ggplotGrob(p), "gtable")
})

test_that("yaxis_title_plus builds with facets and top strips", {
  p = ggplot(iris, aes(Petal.Length, Sepal.Length)) +
    geom_point() +
    facet_wrap(~Species) +
    theme_plus(enable_coaching = F) +
    yaxis_title_plus()

  expect_s3_class(ggplot2::ggplotGrob(p), "gtable")
})

test_that("gridlines_plus builds with continuous and discrete axes", {
  p = ggplot(iris, aes(Petal.Length, Species)) +
    geom_boxplot() +
    theme_plus(enable_coaching = F) +
    yaxis_title_plus() +
    gridlines_plus()

  expect_s3_class(ggplot2::ggplotGrob(p), "gtable")
})

test_that("ggplotplus plots convert to cowplot-compatible grobs", {
  p = ggplot(iris, aes(Petal.Length, Sepal.Length, fill = Species)) +
    geom_point_plus() +
    theme_plus(enable_coaching = F)

  expect_s3_class(ggplotplus_to_cowplot(p), "grob")
})

test_that("ggplotplus plots convert to patchwork elements when patchwork is installed", {
  skip_if_not_installed("patchwork")

  p = ggplot(iris, aes(Petal.Length, Sepal.Length, fill = Species)) +
    geom_point_plus() +
    theme_plus(enable_coaching = F)

  expect_s3_class(ggplotplus_to_patchwork(p), "wrapped_patch")
})

test_that("geom_point_plus builds with default shapes", {
  p = ggplot(iris, aes(Petal.Length, Sepal.Length, shape = Species)) +
    geom_point_plus()

  expect_s3_class(ggplot2::ggplot_build(p), "ggplot_built")
})

test_that("base fillable shape numbers are standardized", {
  expect_equal(
    ggplotplus:::.standardize_pointplus_shape_names(c("21", 22, "plus")),
    c("circle", "square", "plus")
  )
})

test_that("NULL shape choices remain NULL", {
  expect_null(ggplotplus:::.standardize_pointplus_shape_names(NULL))
})

test_that("custom shapes validate and register", {
  test_star = data.frame(
    x = c(0.000, 0.118, 0.380, 0.190, 0.235,
          0.000, -0.235, -0.190, -0.380, -0.118),
    y = c(0.400, 0.124, 0.124, -0.047, -0.324,
          -0.153, -0.324, -0.047, 0.124, 0.124),
    piece = 1
  )

  expect_silent(add_shape_plus("test_star", test_star, overwrite = TRUE))
  expect_true("test_star" %in% names(ggplotplus:::.pointplus_shapes()))
})

test_that("invalid custom shapes error helpfully", {
  bad_shape = data.frame(x = c(0, 1), y = c(0, 1))

  expect_error(
    add_shape_plus("bad_shape", bad_shape),
    "missing"
  )
})

test_that("constant alpha and size do not break legend building", {
  p = ggplot(iris, aes(Petal.Length, Sepal.Length)) +
    geom_point(aes(fill = Species), shape = 21, alpha = 0.3, size = 1) +
    theme_plus(enable_coaching = F)

  expect_s3_class(ggplot2::ggplot_build(p), "ggplot_built")
})

test_that("suppressed guides are not restored", {
  p = ggplot(iris, aes(Petal.Length, Sepal.Length)) +
    geom_point(aes(fill = Species), shape = 21, alpha = 0.3, size = 1) +
    scale_fill_discrete(guide = "none") +
    theme_plus(enable_coaching = F)

  built = ggplot2::ggplot_build(p)

  expect_true(ggplotplus:::.guide_is_none_for_aes(p, "fill"))
  expect_s3_class(built, "ggplot_built")
})

test_that("user guide overrides are preserved", {
  p = ggplot(iris, aes(Petal.Length, Sepal.Length)) +
    geom_point(aes(fill = Species), shape = 21, alpha = 0.3, size = 1) +
    guides(fill = guide_legend(override.aes = list(alpha = 0.8, size = 2))) +
    theme_plus(enable_coaching = F)

  built = ggplot2::ggplot_build(p)

  guide = built@plot@guides$guides[[1]]
  expect_equal(guide$params$override.aes$alpha, 0.8)
  expect_equal(guide$params$override.aes$size, 2)
})

test_that("merged fill and shape legends do not warn about duplicated override.aes", {
  p = ggplot(iris, aes(Sepal.Width, Sepal.Length, fill = Species, shape = Species)) +
    geom_point_plus(colour = "black") +
    theme_plus(legend.position = "top",
               enable_coaching = F)

  expect_warning(
    ggplot2::ggplot_build(p),
    regexp = NA
  )
})

# .onLoad() is a special function called automatically by R
# when the ggplotplus package is loaded via library().

#' geom_point_plus() shape registry
#'
#' Package-private environment used to store built-in and user-registered
#' point shapes for \code{geom_point_plus()}.
#'
#' @keywords internal
.onLoad = function(libname, pkgname) {
  S7::methods_register()
}

utils::globalVariables("ggplotplus_shapes_list")

.pointplus_shape_registry = new.env(parent = emptyenv()) #THIS CREATES A UNIFIED REGISTRY OF SHAPES, BOTH THOSE FROM THE PACKAGE AND THOSE ADDED BY THE USER, THAT GEOM_POINTS_PLUS() PULLS FROM.

#THIS INITIALIZES THE REGISTRY OF SHAPES UPON FIRST REQUEST INSTEAD ON ONLOAD.
.pointplus_shape_registry = new.env(parent = emptyenv())
.pointplus_shape_registry$.initialized = FALSE
.pointplus_shape_registry$.order = character(0)

.initialize_pointplus_shape_registry = function(force = FALSE) {

  current_shapes = setdiff(ls(.pointplus_shape_registry), c(".initialized", ".order"))

  if(!force &&
     isTRUE(.pointplus_shape_registry$.initialized) &&
     length(current_shapes) > 0) {
    return(invisible(NULL))
  }

  for(nm in names(ggplotplus_shapes_list)) {
    assign(nm, ggplotplus_shapes_list[[nm]], envir = .pointplus_shape_registry)
  }

  .pointplus_shape_registry$.order = names(ggplotplus_shapes_list)
  .pointplus_shape_registry$.initialized = TRUE

  invisible(NULL)
}

# IMPORT COMMANDS ---------------------------------------------------------

#' @importFrom rlang %||% list2 .data
#' @import ggplot2
#' @importFrom ggplot2 update_ggplot class_ggplot ggplot_build ggplot_gtable
#' @importFrom polyclip polyclip
#' @importFrom scales pretty_breaks pal_viridis
#' @importFrom viridisLite viridis
#' @importFrom dplyr %>% slice_max mutate group_by across all_of if_else slice_min select any_of ungroup
#' @importFrom stats median
#' @importFrom utils globalVariables
NULL

##THIS HELPS THE R CMD CHECKER KNOW THESE "VARIABLES" ARE MADE DURING DPLYR CHAINS AND ARE THUS BOUND BY DPLYR'S EVALUATION SCHEME AND SHOULDN'T BE VIEWED AS GLOBAL VARIABLES.
. = .label_plus = x_range = y_range = target_x = target_y = dist_to_target = NULL


# CUSTOM OBJECTS ----------------------------------------------------------

#' Custom shape palette for `geom_point_plus()`
#'
#' A named list of custom point shapes used by `geom_point_plus()`. Each element
#' contains `x` and `y` coordinates plus a `piece` identifier used when drawing
#' filled shapes and holes.
#'
#' @format A named list of data frames. Each data frame has columns:
#' \describe{
#'   \item{x}{X coordinates for the shape outline.}
#'   \item{y}{Y coordinates for the shape outline.}
#'   \item{piece}{Integer identifier for separate polygon pieces or holes.}
#' }
#'
#' @examples
#' names(ggplotplus_shapes_list)
"ggplotplus_shapes_list"

# geom_plus_defaults ------------------------------------------------------
#' Default settings for geometry layers created by `geom_plus()`
#'
#' A named list of default aesthetics used by `geom_plus()` to control styling of the resulting geometry layers.
#'
#' Most geom-specific default aesthetics are set in `theme_plus()`; what is set here is what can't be set there.
#'
#' @format A named list with elements like "point", "jitter", "boxplot", etc., corresponding to commonly used ggplot2 geometries. Use names(geom_plus_defaults) for a full list.
#' @keywords internal
#' @noRd
geom_plus_defaults = list(
  point = list(
    aes = list(fill = NA),
    params = list()
  ),
  point_plus = list(
    aes = list(fill = NA),
    params = list()
  ),
  jitter = list(
    aes = list(fill = NA),
    params = list()
  ),
  count = list(
    aes = list(fill = NA),
    params = list()
  ),
  boxplot = list(
    aes = list(),
    params = list(
      staplewidth = 0.25,
      outlier.colour = "black",
      outlier.stroke = 1.2,
      outlier.fill = "transparent",
      outlier.shape = 21
    )
  ),
  violin = list(
    aes = list(fill = "white"),
    params = list(
      geom_params = list(
        quantile_gp = list(
      linetype = "solid",
      linewidth = 0.5,
      colour = "black"
        )
    ),
    stat_params = list(
      quantiles = c(0.25, 0.5, 0.75)
    )
   )
  ),
  bar = list(
    aes = list(fill = "transparent"),
    params = list()
  ),
  col = list(
    aes = list(fill = "transparent"),
    params = list(width = 0.8)
  ),
  histogram = list(
    aes = list(fill = "transparent"),
    params = list()
  ),
  line = list(
    aes = list(alpha = 1),
    params = list()
  ),
  freqpoly = list(
    aes = list(alpha = 1),
    params = list()
  ),
  segment = list(
    aes = list(alpha = 1),
    params = list()
  ),
  abline = list(
    aes = list(alpha = 1),
    params = list()
  ),
  hline = list(
    aes = list(alpha = 1),
    params = list()
  ),
  vline = list(
    aes = list(alpha = 1),
    params = list()
  ),
  curve = list(
    aes = list(alpha = 1),
    params = list()
  ),
  smooth = list(
    aes = list(fill = "black", alpha = 0.3),
    params = list()
  ),
  area = list(
    aes = list(fill = "black", alpha = 0.3),
    params = list()
  ),
  ribbon = list(
    aes = list(fill = "black", alpha = 0.3, linewidth = NA),
    params = list()
  ),
  crossbar = list(
    aes = list(),
    params = list()
  ),
  errorbar = list(
    aes = list(),
    params = list()
  ),
  linerange = list(
    aes = list(),
    params = list()
  ),
  pointrange = list(
    aes = list(),
    params = list()
  ),
  density = list(
    aes = list(fill = "black", alpha = 0.3),
    params = list()
  ),
  dotplot = list(
    aes = list(fill = "transparent"),
    params = list()
  ),
  tile = list(
    aes = list(alpha = 1, colour = "black", linetype = "solid")
  )
)

# CONVENIENCE FUNCTIONS ---------------------------------------------------
#' Compute endpoint-aware continuous scale breaks or limits
#'
#' Internal helper used by `scale_continuous_plus()` to compute “pretty” breaks
#' while gently expanding the working range until breaks occur near both ends of
#' the original data limits.
#'
#' @param lims Numeric vector of length 2 of the incoming limits from ggplot.
#' @param n Target number of breaks passed to `scales::pretty_breaks()`.
#' @param buffer_frac Fraction of the data span used to decide whether an
#'   endpoint is close enough to a break.
#' @param Return Character string; either `"breaks"` to return computed breaks
#'   or `"limits"` to return expanded limits based on those breaks.
#'
#' @return A numeric vector of breaks or limits, as requested in `Return`.
#'
#' @keywords internal
#' @noRd
.endpoint_breaks = function(lims, n = 5, buffer_frac = 0.05, Return = c("breaks", "limits")) {

  Return = match.arg(Return)

  pretty_fn = scales::pretty_breaks(n)
  original_lo = lims[1]
  original_hi = lims[2]
  lo = original_lo
  hi = original_hi
  span = diff(lims)

  #SPAN COULD BE 0 IF LIMITS COME IN EQUAL FOR SOME REASON. THIS MAKES SURE BUFFER WOULD BE NON 0 SO WE ALWAYS GET BACK MORE THAN 1 VALUE FOR BREAKS.
  if(is.infinite(span) || span == 0) {
    eps = if(is.finite(original_lo) && original_lo != 0) {
      abs(original_lo) * 0.05
    } else { 1 }
    lo = original_lo - eps
    hi = original_hi + eps
    span = hi - lo
  }

  buffer = buffer_frac * span

  for (i in seq_len(50)) {
    brks = pretty_fn(c(lo, hi))
    got_low  = min(brks) <= (original_lo + buffer)
    got_high = max(brks) >= (original_hi - buffer)
    if (got_low && got_high) break
    if (!got_low) lo = lo - buffer
    if (!got_high) hi = hi + buffer
  }

  if(Return == "breaks") {
    return(brks)
  } else if(Return == "limits") {
    return(c(min(brks) - buffer, max(brks) + buffer))
  }
}


#' Build gridline theme adjustments from trained panel scales
#'
#' Internal helper used during the ggplotplus build stage. Inspects the trained
#' panel scales of a built plot and returns a ggplot2 theme object that keeps
#' major gridlines only for continuous position scales, using the styling stored
#' in a `GridlinesPlus` intent.
#'
#' Currently checks the first panel only. If the plot uses `coord_flip()`, the
#' x/y gridline decisions are swapped to match the rendered orientation.
#'
#' @param plot A built ggplot object.
#' @param intents A `GridlinesPlus` object containing gridline color, linewidth,
#'   and linetype settings.
#'
#' @return A ggplot2 theme object controlling major and minor panel gridlines styling instructions.
#'
#' @keywords internal
#' @noRd
.apply_gridlines_plus = function(plot, intents) {

  panel_scales = ggplot2::get_panel_scales(plot, i = 1, j = 1) #CHECK OUT THE PANEL SCALES IN THE BUILT OBJECT

  #DO THEY INHERIT A CONTINUOUS SCALE CLASS?
  x_is_cont = inherits(panel_scales$x, "ScaleContinuousPosition")
  y_is_cont = inherits(panel_scales$y, "ScaleContinuousPosition")

  #UNPACK STORED INTENTS AROUND KEEPING SOME AXES BLANK
  notx = intents@notx
  noty = intents@noty

  #NEED TO CHECK FOR AND RESPECT A COORD_FLIP BY SWAPPING THE ABOVE.
  if(inherits(plot@plot@coordinates, "CoordFlip")) {
    tmp = x_is_cont
    tmp2 = notx

    x_is_cont = y_is_cont
    notx = noty

    y_is_cont = tmp
    noty = tmp2
  }

  #CONDITIONALLY ADJUST GRIDLINES AS APPROPRIATE AND SUPPRESS ALL OTHERS.
  grid_theme = ggplot2::theme(
    panel.grid.major.x = if(x_is_cont && !notx) { #<--ENSURE INTENT OF USER IS ACKNOWLEDGED.
      ggplot2::element_line(
        colour = intents@color,
        linewidth = intents@linewidth,
        linetype = intents@linetype
      )
    } else {
      ggplot2::element_blank()
    },
    panel.grid.major.y = if(y_is_cont && !noty) {
      ggplot2::element_line(
        colour = intents@color,
        linewidth = intents@linewidth,
        linetype = intents@linetype
      )
    } else {
      ggplot2::element_blank()
    },
    panel.grid.minor.x = ggplot2::element_blank(),
    panel.grid.minor.y = ggplot2::element_blank()
  )

  return(grid_theme) #RETURN THE BUILT PLOT PLUS THE NEW GRIDLINE THEME RULES.
}


#' Apply yaxis_title_plus() gtable edits
#'
#' Internal helper used during gtable construction. Removes the original y-axis
#' title grob from the left-side title slot, then inserts a new horizontal title
#' grob in a custom row near the top or bottom of the plotting area.
#'
#' This helper uses `ggplot2::get_labs()` to recover the finalized axis label
#' and `ggplot2::calc_element()` to inherit the relevant axis-title theme
#' styling. If the plot uses `coord_flip()`, the displayed vertical-axis label is
#' taken from the x scale.
#'
#' @param data A built ggplot object, usually `GGPlotPlusBuilt`.
#' @param gt A gtable produced by `ggplot2::ggplot_gtable()`.
#' @param intents A `YAxisTitlePlus` object storing title-placement intent.
#'
#' @return A modified gtable.
#'
#' @keywords internal
#' @noRd
.apply_yaxis_title_plus = function(data, gt, intents) {

  ##STOP TO CONSIDER coord_flip TO DETERMINE WHICH AXIS WE'RE RETITLING.
  real_scale = if(inherits(data$plot$coordinates, "CoordFlip")) { "x" } else { "y" }
  title_element_name = if(real_scale == "y") { "axis.title.y" } else { "axis.title.x" }

  #UNPACK USER INTENTS
  location = intents@location %||% "top"

  labs = ggplot2::get_labs(data)

  lab = if(real_scale == "y") {
    labs$y
  } else {
    labs$x
  }

  if(is.null(lab) || identical(lab, "")) {
    lab = "Placeholder. Replace w/ labs(y = ...)."
  }

  #THEN, KILL THE EXISTING TITLE GROB IN THE GTABLE SO IT DOESN'T ALSO APPEAR.
  kill_names = c("ylab-l", "ylab-r")

  kill_idx = which(gt$layout$name %in% kill_names)
  if(length(kill_idx) > 0) { #OVERWRITE THEM WITH ZEROGROBS.
    gt$grobs[kill_idx] = replicate(length(kill_idx),
                                   ggplot2::zeroGrob(),
                                   simplify = FALSE)
  }
  #0 OUT THE WIDTHS OF THE COLS PREVIOUSLY HOLDING THOSE NAMES.
    cols2zero = unique(unlist(Map(seq.int, gt$layout$l[kill_idx], gt$layout$r[kill_idx])))
    gt$widths[cols2zero] = ggplot2::unit(0, "points")

  ###WE TRY TO LOCATE THE ROWS ABOVE WHEREVER THE AXIS-T OR BELOW THE AXIS-B ROWS ARE -- OR, ON A FACETED GRAPH, WHERE THE TOP OR BOTTOM STRIP LABELS ARE.
    axis_title_rows = which(grepl("^xlab-[tb]", gt$layout$name))
    strip_label_rows = which(grepl("^strip-[tb]", gt$layout$name))

    if(!length(axis_title_rows) & !length(strip_label_rows)) {
      return(gt) #IF SOMEHOW NONE, BREAK. ****WHY WOULD THIS OCCUR?
    }

  if(location == "top") {
    if(length(strip_label_rows) > 0 &&
       length(which(grepl("^strip-[t]", gt$layout$name)))) {

      target_axis_title_row = min(gt$layout$t[strip_label_rows]) #INSERT RIGHT WHERE THEY CURRENTLY ARE.
    } else {
      target_axis_title_row = min(gt$layout$t[axis_title_rows]) - 1 #GO ABOVE X AXIS TITLE ROW OTHERWISE.
    }
    new_title_row = target_axis_title_row + 1 #THE NEW ROW IS BELOW THE ONE I WAS TARGETING THANKS TO INCREMENTAL INDEXING.
  } else {
    if(length(strip_label_rows) > 0 & #REVERSE PATTERN ON BOTTOM.
       length(which(grepl("^strip-[b]", gt$layout$name)))) {
      target_axis_title_row = max(gt$layout$b[strip_label_rows])
    } else {
      target_axis_title_row = max(gt$layout$b[axis_title_rows]) + 1
    }
    new_title_row = target_axis_title_row - 1
  }

    #FIND THE COLUMN IN WHICH THE AXIS-L CONTENT WAS ORIGINALLY--THIS IS WHAT WE WILL JUSTIFY THE CONTENTS OF THE NEW TITLE TO ON THE LEFT-HAND SIDE.
  axis_l_cols = which(grepl("^axis-l", gt$layout$name))
  panel_cols = which(grepl("^panel", gt$layout$name))

  title_col = if(length(axis_l_cols) > 0) {
    min(gt$layout$l[axis_l_cols])  #GET THE MINIMUM ONE, IF THERE ARE MANY.
  } else {
    min(gt$layout$l[panel_cols]) #OTHERWISE, DEFAULT TO THE COL TO THE FAR LEFT OF THE PANEL...****IS THIS NECESSARY?
  }

  #THEN, INSERT ENTIRELY NEW ROW FOR THE TITLE, EITHER ON TOP OR ON BOTTOM.
  gt = gtable::gtable_add_rows(gt,
                               heights = ggplot2::unit(16, "pt"),
                               pos = target_axis_title_row)

  #THEN, BEGIN BUILDING THE TEXT GROB. SHOULD USE THE THEME STYLES FROM THE PREVIOUS TITLE.
  el = ggplot2::calc_element(title_element_name, data$plot$theme)
  gp = .ggplus_element_to_gpar(el) #JUST TRANSLATES THEME ARG NAMES TO GPAR ARG NAMES.

  if(location == "top") {
    vjust_val = 0
  } else {
    vjust_val = 1
  }

  #ACTUALLY ADD THE GROB
  gt = gtable::gtable_add_grob(
    gt,
    grob = grid::textGrob(
      lab,
      x = 0, y = 0.5,
      hjust = 0, vjust = vjust_val,
      rot = 0,
      gp = gp
    ),
    t = new_title_row,
    b = new_title_row,
    l = title_col,
    name = paste0("ggplotplus-", real_scale, "-title"),
    clip = "off"
  )

  return(gt)

}

#' Convert a ggplot2 theme text element to grid graphical parameters
#'
#' Internal helper for translating selected text-element settings from ggplot2's
#' theme system into a `grid::gpar()` object suitable for manually constructed
#' grobs.
#'
#' @param el A text theme element, usually returned by `ggplot2::calc_element()`.
#'
#' @return A `grid::gpar()` object containing color, font size, font face, font
#'   family, and line-height settings.
#'
#' @keywords internal
#' @noRd
.ggplus_element_to_gpar = function(el) {
  grid::gpar(
    col        = el$colour %||% el$color %||% "black",
    fontsize   = el$size %||% 11,
    fontface   = el$face %||% "plain",
    fontfamily = el$family %||% "",
    lineheight = el$lineheight %||% 0.9
  )
}


#' Extract a partially matched argument from a named list
#'
#' Internal helper that searches a list of named arguments for a single
#' partially matching name using `pmatch()`. Returns the matched value if found.
#'
#' @param args A named list of arguments.
#' @param target A character string giving the target argument name.
#'
#' @return The matched argument value, or `NULL` if no match is found.
#'
#' @keywords internal
#' @noRd
.partial_match_user_arg = function(args, target) {
  nms = names(args) #FIND ALL NAMED ARGUMENTS.

  if (is.null(nms)) { #IF NONE STOP--NO MATCHING POSSIBLE.
    return(NULL)
  }

  #INDEXES OF NAMED, PARTIALLY MATCHING ARGUMENTS.
  idx = which(!is.na(nms) & pmatch(nms, target, nomatch = 0L) > 0)

  if (length(idx) > 1) { #TOO MANY? STOP.
    stop(sprintf("Multiple arguments match '%s'.", target), call. = FALSE)
  }

  #IF EXACTLY 1, FIND THAT MATCHING ARGUMENT OR GIVE UP.
  if (length(idx) == 1) args[[idx]] else NULL
}


#' Remove a partially matched argument from a named list
#'
#' Internal helper that removes a single partially matching argument from a
#' named list using `pmatch()`. Intended for sanitizing user inputs before
#' forwarding arguments to ggplot2 functions.
#'
#' @param args A named list of arguments.
#' @param target A character string giving the target argument name.
#'
#' @return The input list with the matching argument removed, if present.
#'
#' @keywords internal
#' @noRd
.remove_partial_match_user_arg = function(args, target) {

  nms = names(args)

  # If no names, nothing to do
  if (is.null(nms)) {
    return(args)
  }

  # Identify matches (same logic as your fixed helper)
  idx = which(!is.na(nms) & pmatch(nms, target, nomatch = 0L) > 0)

  # If multiple matches, mirror your existing behavior
  if (length(idx) > 1) {
    stop(sprintf("Multiple arguments match '%s'.", target), call. = FALSE)
  }

  # If exactly one match, drop it
  if (length(idx) == 1) {
    args = args[-idx]
  }

  return(args)
}


#' Normalize viridis palette specification
#'
#' Internal helper that converts single-letter viridis palette codes (A–H)
#' to their full names.
#'
#' @param x A character vector giving a viridis palette name or code.
#'
#' @return A character string giving the normalized viridis palette name.
#'
#' @keywords internal
#' @noRd
.normalize_viridis = function(x) {
  map = c(A="magma", B="inferno", C="plasma", D="viridis",
          E="cividis", F="rocket", G="mako", H="turbo")
  if (length(x) == 1 && x %in% names(map)) map[[x]] else x
}


#' Create a discrete viridis palette function
#'
#' Internal helper that returns a function generating discrete color palettes
#' using `viridisLite::viridis()`, suitable for ggplot2 theme palette settings.
#'
#' @param option Viridis palette name.
#' @param begin,end Length one numeric values between 0 and 1 controlling palette endpoints.
#'
#' @return A function accepting `n` and returning a vector of colors.
#'
#' @keywords internal
#' @noRd
.make_discrete_palette = function(option, begin, end) {
  function(n) {
    viridisLite::viridis(
      n = n,
      option = option,
      begin = begin,
      end = end
    )
  }
}

#' Create a continuous viridis palette function
#'
#' Internal helper that returns a continuous palette function using
#' `scales::pal_viridis()`, suitable for ggplot2 theme palette settings.
#'
#' @param option Viridis palette name.
#' @param begin,end Length one numeric values between 0 and 1 controlling palette endpoints.
#'
#' @return A palette function mapping numeric values between 0 and 1 to colors.
#'
#' @keywords internal
#' @noRd
.make_continuous_palette = function(option, begin, end) {
  scales::pal_viridis(
    option = option,
    begin = begin,
    end = end
  )
}


#' Construct ggplotplus palette theme settings
#'
#' Internal helper that builds a `ggplot2::theme()` call configuring discrete
#' and continuous color/fill palettes using viridis-based palette functions.
#'
#' @param palette_discrete,palette_continuous Viridis palette names or codes.
#' @param begin_discrete,end_discrete Numeric endpoints for a discrete palette.
#' @param begin_continuous,end_continuous Numeric endpoints for a continuous palette.
#'
#' @return A ggplot2 theme object specifying palette settings.
#'
#' @keywords internal
#' @noRd
.theme_plus_palettes = function(palette_discrete,
                                palette_continuous,
                                begin_discrete,
                                end_discrete,
                                begin_continuous,
                                end_continuous) {

  palette_discrete = .normalize_viridis(palette_discrete)
  palette_continuous = .normalize_viridis(palette_continuous)

  disc = .make_discrete_palette(palette_discrete, begin_discrete, end_discrete)
  cont = .make_continuous_palette(palette_continuous, begin_continuous, end_continuous)

  return(ggplot2::theme(
    palette.colour.discrete = disc,
    palette.fill.discrete = disc,
    palette.colour.continuous = cont,
    palette.fill.continuous = cont
  ))
}

#' Legend-Position Theme Conditional Logic
#'
#' Builds a small theme fragment for customizing `"top"` vs `"right"` vs `"bottom"` legends.
#'
#' @inheritParams theme_plus
#' @return A ggplot2 theme object.
#' @keywords internal
#' @noRd
.determine_legend_theme = function(legend_pos = "top") {

  if(legend_pos == "top") {
    ggplot2::theme(
      legend.key.width = ggplot2::unit(1.5, "cm"),
      legend.key.height = ggplot2::unit(0.8, "cm"),
      legend.title = ggplot2::element_text(margin = ggplot2::margin(l = 15)),
      legend.margin = ggplot2::margin(t = 5, r = 5, b = 5, l = 5),
      plot.margin = ggplot2::margin(t = 5, r = 5, b = 5, l = 5),
      legend.box.just = "right",
      legend.justification = "right",
      legend.key.justification = "right",
      legend.title.position = "right",
      legend.key.spacing.x = ggplot2::unit(0.5, "cm"),
      legend.position = "top",
      legend.direction = "horizontal"
    )
  } else if(legend_pos == "right") {

    ggplot2::theme(
      legend.key.height = ggplot2::unit(1.5, "cm"),
      legend.key.width = ggplot2::unit(0.8, "cm"),
      legend.title = ggplot2::element_text(margin = ggplot2::margin(b = 15), hjust = 0.5),
      legend.box.just = "right",
      legend.margin = ggplot2::margin(t = 5, r = 10, b = 5, l = 5),
      plot.margin = ggplot2::margin(t = 5, r = 10, b = 5, l = 5),
      legend.justification = "right",
      legend.key.spacing.y = ggplot2::unit(0.5, "cm"),
      legend.position = "right",
      legend.direction = "vertical")
  } else if(legend_pos == "bottom") {
    ggplot2::theme(
      legend.key.width = ggplot2::unit(1.5, "cm"),
      legend.key.height = ggplot2::unit(0.8, "cm"),
      legend.title = ggplot2::element_text(margin = ggplot2::margin(l = 15)),
      legend.margin = ggplot2::margin(t = 5, r = 5, b = 5, l = 5),
      plot.margin = ggplot2::margin(t = 5, r = 5, b = 5, l = 5),
      legend.box.just = "right",
      legend.justification = "right",
      legend.key.justification = "right",
      legend.title.position = "right",
      legend.key.spacing.x = ggplot2::unit(0.5, "cm"),
      legend.position = "bottom",
      legend.direction = "horizontal"
    )
  }
}


#' Check whether a geom parameter is already set
#'
#' Internal helper used when applying `theme_plus()` geom defaults. Determines
#' whether a layer already has a geom parameter set so ggplotplus does not
#' overwrite explicit user intent.
#'
#' Includes a special case for violin `quantile_gp`, which exists by default in
#' an inactive state and should only count as set when quantile styling has
#' actually been supplied by the user.
#'
#' @param layer A ggplot2 layer object.
#' @param param Character string naming the geom parameter to check.
#'
#' @return Logical; `TRUE` if the parameter should be treated as already set.
#'
#' @keywords internal
#' @noRd
.param_is_already_set = function(layer, param) {

  if (param == "quantile_gp") {
    qgp = layer$geom_params$quantile_gp

    hasbeenset = !is.null(qgp) &&
      (
        (!is.null(qgp$linetype) && !is.na(qgp$linetype) && qgp$linetype != 0) ||
          !is.null(qgp$colour) ||
          !is.null(qgp$linewidth)
      )

    return(hasbeenset)
  }

  param %in% names(layer$geom_params)
}



#' Check whether an aesthetic is mapped locally
#'
#' Internal helper used by `geom_point_plus()` to detect whether a specific
#' aesthetic is mapped in a local `mapping` argument or in an unnamed `aes()`
#' object passed through `...`.
#'
#' This does not currently inspect global plot mappings.
#'
#' @param mapping A local ggplot2 aesthetic mapping, usually from `aes()`.
#' @param dots A list of additional arguments passed through `...`.
#' @param aes_name Character string naming the aesthetic to detect.
#'
#' @return Logical; `TRUE` if the aesthetic is mapped locally.
#'
#' @keywords internal
#' @noRd
.has_mapped_aes = function(mapping, dots, aes_name) {

  # collect all candidate aes objects
  aes_objs = list()

  # 1) explicit mapping arg
  if (!is.null(mapping) && inherits(mapping, "uneval")) {
    aes_objs = c(aes_objs, list(mapping))
  }

  # 2) unnamed aes(...) in ...
  for (obj in dots) {
    if (inherits(obj, "uneval")) {
      aes_objs = c(aes_objs, list(obj))
    }
  }

  if (length(aes_objs) == 0) return(FALSE)

  # check if any aes contains the target
  any(vapply(aes_objs, function(a) aes_name %in% names(a), logical(1)))
}


#' Nudge a top-positioned legend downward
#'
#' Internal helper that returns a `ggplot2::theme()` adjustment which shifts a
#' legend (box) positioned at the top of a plot slightly downward by applying a
#' negative bottom margin to the legend box.
#'
#' This is used by `yaxis_title_plus()` to prevent wasted space between the relocated
#' y-axis title and a top-positioned legend (box).
#'
#' @param howMuch Numeric value (in points) controlling how far the legend is
#'   nudged downward. Larger values move the legend further.
#'
#' @return A ggplot2 theme object adjusting `legend.box.margin`.
#'
#' @keywords internal
#' @noRd
.nudge_top_legend_down = function(howMuch = 20) {

  ggplot2::theme(legend.box.margin = ggplot2::margin(b = -howMuch, r = 5, t = 5, l = 5))

}


#' Standardize ggplotplus shape names
#'
#' Internal helper to normalize shape identifiers used by
#' \code{geom_point_plus()}. Converts numeric shape codes
#' corresponding to base R's fillable point shapes (21--25)
#' into their ggplotplus string equivalents.
#'
#' This ensures consistent downstream handling regardless of
#' whether users supply shapes as numbers (e.g., \code{21}) or
#' names (e.g., \code{"circle"}).
#'
#' @param shape A vector of shape identifiers. May be numeric,
#'   character, or factor.
#'
#' @return A character vector of standardized shape names.
#'
#' @keywords internal
.standardize_pointplus_shape_names = function(shape) {

  if(is.null(shape)) { #DEFENDS--IF NULL, THE DEFAULT, JUST RETURN NULL, AS THAT IS WHAT IS EXPECTED DOWNSTREAM.
    return(NULL)
  }

  validshapeslookup = c(
    "21" = "circle",
    "22" = "square",
    "23" = "diamond",
    "24" = "triangle_up",
    "25" = "triangle_down"
  )

  shape = as.character(shape)

  use_lookup = shape %in% names(validshapeslookup)

  shape[use_lookup] = validshapeslookup[shape[use_lookup]]

  shape
}


#' Validate a geom_point_plus() shape definition
#'
#' Internal helper used by \code{add_shape_plus()} to check that a custom
#' point shape has the structure needed by the ggplotplus shape-drawing
#' machinery.
#'
#' @param shape A proposed shape definition. Must be a data frame with columns
#'   \code{x}, \code{y}, and \code{piece}.
#' @param name Shape name, used by callers.
#'
#' @return A cleaned shape data frame containing only \code{x}, \code{y}, and
#'   \code{piece}. The \code{piece} column is converted to sequential integers if
#'   it wasn't already formatted like that.
#'
#' @keywords internal
.validate_pointplus_shape = function(shape, name = NULL) {

  #VARIOUS SAFETY CHECKS TO ENSURE WE GOT THE INPUTS WE NEEDED W/ HELPFUL ERROR MESSAGES IF NOT.
  required_cols = c("x", "y", "piece")

  if(!is.data.frame(shape)) {
    stop("`shape` must be a data frame containing three columns named 'x', 'y', and 'piece'. See ?add_shape_plus for details.", call. = FALSE)
  }

  missing_cols = setdiff(required_cols, names(shape))

  if(length(missing_cols) > 0) {
    stop(
      "`shape` is missing the following required column(s): ",
      paste(missing_cols, collapse = ", "),
      call. = FALSE
    )
  }

  if(!is.numeric(shape$x) || !is.numeric(shape$y)) {
    stop("`shape$x` and `shape$y` must both be numeric.", call. = FALSE)
  }

  if(any(!is.finite(shape$x)) || any(!is.finite(shape$y))) {
    stop("`shape$x` and `shape$y` must contain only finite numeric values.", call. = FALSE)
  }

  if(any(is.na(shape$piece))) {
    stop("`shape$piece` cannot contain missing values. These should be integers starting at 1.", call. = FALSE)
  }

  pieces = split(shape, shape$piece) #CHOPS INTO A LIST BY PIECE VAL.

  n_vertices = vapply(pieces, nrow, integer(1)) #APPLIES NROW TO EACH.

  if(any(n_vertices < 3)) {
    stop("Each `piece` of your shape must contain at least 3 points.", call. = FALSE)
  }

  #****THIS VALIDATES THE FIRST REQUIREMENT BUT NOT THE SECOND...THAT'S PROBABLY OK? MORE LIKELY YOU WILL FAIL BOTH IF YOU FAIL THE FIRST. CAN FINE-TUNE LATER.
  if(max(abs(c(shape$x, shape$y))) > 1) {
    warning(
      "`shape` coordinates extend beyond +/-1. ",
      "Point shapes are expected to be centered on 0 and scaled to roughly +/-0.4.",
      call. = FALSE
    )
  }

  shape = shape[, required_cols, drop = FALSE] #GETS RID OF ANY EXTRA COLS.

  shape$piece = as.integer(as.factor(shape$piece)) #RENUMBERS THESE TO BE INTEGERS STARTING AT 1 IF THEY AREN'T ALREADY.

  return(shape)
}


#' Get registered geom_point_plus() shapes
#'
#' Internal helper that returns the current ggplotplus point-shape registry,
#' including both built-in package shapes and any shapes added during the
#' current R session with \code{add_shape_plus()}.
#'
#' @return A named list of point-shape data frames.
#'
#' @keywords internal
.pointplus_shapes = function() {

  .initialize_pointplus_shape_registry()

  shape_names = .pointplus_shape_registry$.order

  shape_names = shape_names[
    vapply(shape_names, function(x) {
      exists(x, envir = .pointplus_shape_registry, inherits = FALSE)
    }, logical(1))
  ]

  stats::setNames(
    lapply(shape_names, function(x) {
      get(x, envir = .pointplus_shape_registry, inherits = FALSE)
    }),
    shape_names
  )
}


#' Safely check whether an S7 property is TRUE
#'
#' Internal helper that safely checks whether an object:
#' \enumerate{
#'   \item exists and is not \code{NULL},
#'   \item is an S7 object,
#'   \item contains a specified property, and
#'   \item has that property set to \code{TRUE}.
#' }
#'
#' This helper is primarily used to avoid errors when optional
#' ggplotplus S7 components may not (yet) exist on a plot object.
#'
#' @param object An object that may or may not be an S7 object.
#' @param prop A single character string giving the property name
#'   to check.
#'
#' @return A logical scalar. Returns \code{TRUE} only if the object
#'   is a valid S7 object, the property exists, and the property
#'   value is exactly \code{TRUE}. Otherwise returns \code{FALSE}.
#'
#' @keywords internal
.s7_prop_is_true = function(object, prop) {

  if(is.null(object)) {
    return(FALSE)
  }

  if(!S7::S7_inherits(object, S7::S7_object)) {
    return(FALSE)
  }

  if(!S7::prop_exists(object, prop)) {
    return(FALSE)
  }

  isTRUE(S7::prop(object, prop))
}

#' Check whether a plot maps one or more aesthetics
#'
#' Internal helper that checks whether any layer in a ggplot object maps one
#' or more requested aesthetics, either directly in the layer mapping or through
#' inherited plot-level mappings.
#'
#' This is used before plot building to determine whether an aesthetic such as
#' \code{alpha} or \code{size} is being mapped explicitly, in which case
#' ggplotplus should avoid overriding that aesthetic in legend keys.
#'
#' @param plot A ggplot object.
#' @param aes_name A character vector of aesthetic names to check.
#'
#' @return A logical scalar. Returns \code{TRUE} if any requested aesthetic is
#'   mapped in any layer, including through inherited global mappings.
#'
#' @keywords internal
.plot_has_mapped_aes = function(plot, aes_name) {

  plot_mapping = plot@mapping

  any(vapply(plot@layers, function(layer) {

    layer_maps_aes = !is.null(layer$mapping[[aes_name]])

    inherits_plot_mapping =
      isTRUE(layer$inherit.aes) &&
      !is.null(plot_mapping[[aes_name]])

    layer_maps_aes || inherits_plot_mapping

  }, logical(1)))
}

#' Check whether a scale suppresses a guide
#'
#' Internal helper that checks whether a plot contains a scale for a given
#' aesthetic with \code{guide = "none"}.
#'
#' This is used to preserve user intent when ggplotplus applies automatic legend
#' key overrides. If a user has explicitly suppressed a guide through a scale,
#' ggplotplus should not reintroduce that guide.
#'
#' @param plot A ggplot object.
#' @param aes_name A single character string giving the aesthetic name to check.
#'
#' @return A logical scalar. Returns \code{TRUE} if a matching scale has
#'   \code{guide = "none"}; otherwise returns \code{FALSE}.
#'
#' @keywords internal
.has_guide_none_for_aes = function(plot, aes_name) {

  scales = plot@scales$scales

  any(vapply(scales, function(scale) {

    aes_name %in% scale$aesthetics &&
      identical(scale$guide, "none")

  }, logical(1)))
}

#' Check whether plot-level guides suppress an aesthetic
#'
#' Internal helper that checks whether a plot-level guide specification suppresses
#' the guide for a given aesthetic, such as through
#' \code{guides(fill = "none")}.
#'
#' This complements the scale-level guide checker above, because guide
#' suppression can be declared either on a scale or through \code{guides()}.
#'
#' @param plot A ggplot object.
#' @param aes_name A single character string giving the aesthetic name to check.
#'
#' @return A logical scalar. Returns \code{TRUE} if the plot-level guide for the
#'   aesthetic is \code{"none"} or a \code{GuideNone} object.
#'
#' @keywords internal
.has_plot_guide_none_for_aes = function(plot, aes_name) {

  guide = plot@guides$guides[[aes_name]]

  identical(guide, "none") ||
    inherits(guide, "GuideNone")
}

#' Check whether a guide is suppressed for an aesthetic
#'
#' Internal helper that checks both scale-level and plot-level guide declarations
#' to determine whether the guide for a given aesthetic has been explicitly
#' suppressed.
#'
#' This is used before adding ggplotplus legend-key overrides so that suppressed
#' legends are not accidentally restored.
#'
#' @param plot A ggplot object.
#' @param aes_name A single character string giving the aesthetic name to check.
#'
#' @return A logical scalar. Returns \code{TRUE} if the guide is suppressed
#'   either through a scale or through \code{guides()}.
#'
#' @keywords internal
.guide_is_none_for_aes = function(plot, aes_name) {
  .has_guide_none_for_aes(plot, aes_name) ||
    .has_plot_guide_none_for_aes(plot, aes_name)
}


#' Check whether an aesthetic is mapped to a continuous (numeric) variable
#'
#' Internal helper that determines whether a given aesthetic is mapped to a
#' continuous variable. Used to guard legend override operations that are
#' appropriate only for discrete legends (not colorbars), because applying a
#' \code{guide_legend()} to a continuous scale would suppress the colorbar.
#'
#' The helper first checks for an explicit continuous scale on the plot. If
#' none is found, it inspects the mapped expression and looks up the variable
#' in the plot and layer data to determine whether it's numeric.
#'
#' @param plot A ggplot object.
#' @param aes_name A single character string giving the aesthetic to check.
#'
#' @return A logical scalar. Returns \code{TRUE} if the aesthetic appears to be
#'   mapped to a continuous variable; otherwise returns \code{FALSE}.
#'
#' @keywords internal
.aes_mapped_var_is_continuous = function(plot, aes_name) {

  #IF THE SCALE FOR THIS AES IS NOT NULL (SO, THERE IS ONE, YOU SEE IF IT INHERITS GGPLOT2'S CVONTINUOUS SCALES.)
  scale = plot@scales$get_scales(aes_name)
  if(!is.null(scale)) {
    return(inherits(scale, "ScaleContinuous"))
  }

  #OTHERWISE, CHECK THE MAPPING TO SEE IF THIS AES HAS BEEN MAPPED.
  mapping = plot@mapping[[aes_name]]

  #ALSO GO THRU THE LOCAL LAYERS, IN CASE IT WAS MAPPED LOCALLY RATHER THAN GLOBALLY.
  data_frames = list()
  if(is.data.frame(plot@data)) {
    data_frames = c(data_frames, list(plot@data))
  }

  for(layer in plot@layers) {
    if(is.null(mapping) && !is.null(layer$mapping[[aes_name]])) {
      mapping = layer$mapping[[aes_name]]
    }
    if(is.data.frame(layer$data)) {
      data_frames = c(data_frames, list(layer$data))
    }
  }

  if(is.null(mapping)) {
    return(FALSE)
  }

  var_name = tryCatch(rlang::as_name(mapping), error = function(e) { NULL })
  if(is.null(var_name)) {
    return(FALSE)
  }

  #FINALLY, SEE IF THE VARIABLE MAPPED TO THIS AES IS NUMERIC IN ANY LAYER.
  for(df in data_frames) {
    if(var_name %in% names(df)) {
      return(is.numeric(df[[var_name]]))
    }
  }

  FALSE #FAIL.
}


#' Merge ggplotplus legend-key overrides with an existing guide
#'
#' Internal helper that merges ggplotplus legend-key defaults with any
#' user-specified \code{override.aes} values for a guide.
#'
#' ggplotplus uses this helper to improve legend readability by overriding
#' non-semantic, hard-to-read, constant \code{alpha} and \code{size} values in
#' legend keys. Existing user overrides are preserved and take precedence over
#' ggplotplus defaults for positive UX.
#'
#' @param plot A ggplot object.
#' @param aes_name A single character string giving the aesthetic whose guide
#'   should be modified.
#' @param override_list A named list of ggplotplus legend-key overrides, such as
#'   \code{list(alpha = 1, size = 5)}.
#'
#' @return A guide object. If no guide exists for \code{aes_name}, returns a new
#'   \code{ggplot2::guide_legend()} using \code{override_list}. If a guide
#'   exists, returns a copy with merged \code{override.aes} values, where
#'   existing user values win over ggplotplus defaults.
#'
#' @keywords internal
.merge_legend_override = function(plot, aes_name, override_list) {

  guide = plot@guides$guides[[aes_name]]

  if(is.null(guide) || inherits(guide, "GuideNone") || identical(guide, "none")) {
    return(ggplot2::guide_legend(override.aes = override_list))
  }

  old_override = guide$params$override.aes

  if(is.null(old_override)) {
    old_override = list()
  }

  guide$params$override.aes = utils::modifyList(
    override_list,
    old_override
  )

  guide
}


#' Collect column names from plot- and layer-level data
#'
#' Internal helper that gathers column names from all data frames associated
#' with a ggplot object, including both plot-level data and any layer-specific
#' data.
#'
#' ggplotplus uses this helper when evaluating whether scale titles appear to
#' still use raw column names, which may indicate that user-facing labels have
#' not yet been customized with \code{ggplot2::labs()}.
#'
#' @param plot A ggplot object.
#'
#' @return A character vector of unique column names found across the plot-level
#'   and layer-level data frames associated with the plot.
#'
#' @keywords internal
.plot_data_names = function(plot) {

  data_names = character(0)

  if(is.data.frame(plot@data)) {
    data_names = c(data_names, names(plot@data))
  }

  for(layer in plot@layers) {
    if(is.data.frame(layer$data)) {
      data_names = c(data_names, names(layer$data))
    }
  }

  unique(data_names)
}

#' Check whether ggplotplus coaching messages are enabled
#'
#' Internal helper that checks the session-level ggplotplus coaching option.
#'
#' ggplotplus uses this helper to determine whether advisory or educational
#' messages ("coaching") should be displayed during plot construction. Coaching
#' messages can be globally enabled or disabled for the current R session using:
#'
#' \preformatted{
#' options(ggplotplus.enable_coaching = TRUE)
#' options(ggplotplus.enable_coaching = FALSE)
#' }
#'
#' User-facing ggplotplus functions additionally expose local
#' \code{enable_coaching} arguments; both the local setting and the global
#' option must evaluate to \code{TRUE} for coaching messages to appear.
#'
#' @return A logical scalar. Returns \code{TRUE} if coaching messages are
#'   globally enabled for the current R session; otherwise returns
#'   \code{FALSE}.
#'
#' @keywords internal
.ggplotplus_coaching_enabled = function() {
  isTRUE(getOption("ggplotplus.enable_coaching", TRUE))
}


#' Retrieve the pre-build label for an aesthetic
#'
#' Internal helper that attempts to determine the eventual user-facing label
#' associated with a mapped aesthetic before the plot has been formally built.
#'
#' The helper first checks for explicit labels supplied through
#' \code{ggplot2::labs()} or related mechanisms. If no explicit label is found,
#' it falls back to the mapped expression itself using
#' \code{rlang::as_label()}.
#'
#' Both plot-level and layer-level mappings are searched.
#'
#' ggplotplus uses this helper during pre-build guide processing to identify
#' aesthetics that are likely to collapse into a shared legend, such as when
#' both \code{fill} and \code{shape} are mapped to the same variable.
#'
#' @param plot A ggplot object.
#' @param aes_name A single character string giving the aesthetic whose
#'   pre-build label should be retrieved.
#'
#' @return A character string representing the best available pre-build label
#'   for the requested aesthetic.
#'
#' @keywords internal
.get_prebuild_aes_label = function(plot, aes_name) {

  lab = plot@labels[[aes_name]]

  if(!is.null(lab)) {
    return(as.character(lab))
  }

  if(!is.null(plot@mapping[[aes_name]])) {
    return(rlang::as_label(plot@mapping[[aes_name]]))
  }

  for(layer in plot@layers) {

    if(isTRUE(layer$inherit.aes) &&
       !is.null(plot@mapping[[aes_name]])) {
      return(rlang::as_label(plot@mapping[[aes_name]]))
    }

    if(!is.null(layer$mapping[[aes_name]])) {
      return(rlang::as_label(layer$mapping[[aes_name]]))
    }
  }

  aes_name
}


#' Choose direct-label anchor points for grouped point data
#'
#' directlabel_points() is an internal helper used by
#' [direct_labels_plus()] when geometry = "point". It chooses one observed
#' point per group, or per group-by-facet combination, to use as the anchor
#' location for a direct label. How the label is then drawn with respect to that
#' chosen point is determined by [ggrepel::geom_label_repel()].
#'
#' The helper works by calculating a target location for each group and then
#' selecting the observed point closest to that target. For "top" and
#' "bottom" placements, the target is centered at the group's median x-value
#' and then shifted toward the group's maximum or minimum y-value. For "left"
#' and "right" placements, the same logic is applied with x and y reversed.
#'
#' @param data A data frame containing the variables to be plotted and labelled.
#' @param x,y Unquoted column names giving the x- and y-variables.
#' @param group Unquoted column name giving the grouping variable from which to
#' construct labels.
#' @param placement Character string giving the preferred label placement
#' relative to each group's cloud of points. One of "top", "right", "bottom",
#' or "left".
#' @param adj_fact A single numeric value giving the proportional adjustment
#' applied to the group-specific target location. For point data, this changes
#' the target used to choose the labelled point, but does not move the final
#' label anchor away from the selected observed point.
#' @param facet_vars Optional character vector of facet variable names. When
#' supplied, label anchor points are calculated separately within each
#' group-by-facet combination.
#'
#' @return A tibble with one row per group or group-by-facet combination. The
#' returned data include standardized x and y columns, .label_plus, and
#' any grouping or faceting variables needed by ggplot2 to assign labels to
#' panels correctly.
#'
#' @details
#' This helper assumes that label locations should be tied to observed points.
#' It therefore never invents a new x/y coordinate for point labels. Empty or
#' all-missing groups should be screened before this helper is called.
#'
#' @keywords internal
.directlabel_points = function(data,
                              x,
                              y,
                              group,
                              placement,
                              adj_fact,
                              facet_vars) {

  group_name = rlang::as_name(rlang::enquo(group))

  label_data = data %>%
    dplyr::mutate(
      x = {{x}},
      y = {{y}},
      .label_plus = as.character({{group}}))

  group_vars = unique(c(group_name, facet_vars)) #CALC LABEL LOCATIONS PER GROUP PER FACET PANEL.

  #THE MATH IS SLIGHTLY DIFFERENT IF WE'RE LABELING AT THE TOP/BOTTOM VS. LEFT/RIGHT.
  if(placement %in% c("top", "bottom")) {

    label_data %>%
      dplyr::group_by(dplyr::across(dplyr::all_of(group_vars))) %>%
      dplyr::mutate(
        y_range = diff(range(y, na.rm = T)), #GET THE X AND Y RANGES
        x_range = diff(range(x, na.rm = T)),
        x_range = dplyr::if_else(x_range == 0, 1, x_range), #GUARDS AGAINST RANGELESS DIMS...?
        y_range = dplyr::if_else(y_range == 0, 1, y_range),
        target_x = stats::median(x, na.rm = TRUE), #GET THE MEDIAN X VALUE
        target_y = if(placement == "top") { max(y, na.rm = T) + (adj_fact * y_range) #CHANGE THE TARGET IN THE DIRECTION INDICATED AND TO THE AMOUNT INDICATED BY ADJ_FACT...CHANGES JUST THE TARGET SPOT, NOT THE FINAL CHOICE.
        } else if(placement == "bottom") { min(y, na.rm = T) - (adj_fact * y_range) },
        dist_to_target = #CALCULATE ALL DISTANCES BTW. GEOM + TARGET X/Y COORDS.
          ((x - target_x) / x_range)^2 +
          ((y - target_y) / y_range)^2
      ) %>%
      dplyr::slice_min(dist_to_target, n = 1, with_ties = FALSE) %>% #SLICE TO THE CLOSEST POINT/LINE LOCATION TO THE TARGET.
      dplyr::select(x, y, .label_plus, dplyr::any_of(group_vars)) %>% #CLIP TO JUST NECESSARY DATA.
      dplyr::ungroup()

  } else {

    label_data %>%
      dplyr::group_by(dplyr::across(dplyr::all_of(group_vars))) %>%
      dplyr::mutate(
        x_range = diff(range(x, na.rm = T)),
        y_range = diff(range(y, na.rm = T)),
        x_range = dplyr::if_else(x_range == 0, 1, x_range), #GUARDS AGAINST RANGELESS DIMS.
        y_range = dplyr::if_else(y_range == 0, 1, y_range),
        target_y = stats::median(y, na.rm = TRUE),
        target_x = if(placement == "right") { max(x, na.rm = T) + (adj_fact * x_range)
        } else if(placement == "left") { min(x, na.rm = T) - (adj_fact * x_range) },
        dist_to_target =
          ((x - target_x) / x_range)^2 +
          ((y - target_y) / y_range)^2
      ) %>%
      dplyr::slice_min(dist_to_target, n = 1, with_ties = FALSE) %>%
      dplyr::select(x, y, .label_plus, dplyr::any_of(group_vars)) %>%
      dplyr::ungroup()
  }
}


#' Choose direct-label anchor points for grouped line data
#'
#' directlabel_lines() is an internal helper used by
#' [direct_labels_plus()] when geometry = "line". It chooses one endpoint or
#' extreme point per group, or per group-by-facet combination, to use as the
#' anchor location for a direct label. How the label is then drawn with respect
#' to that chosen point is determined by [ggrepel::geom_label_repel()].
#'
#' For "right" placement, the helper selects the row with the largest x-value
#' in each group. For "left", it selects the smallest x-value. For "top",
#' it selects the largest y-value. For "bottom", it selects the smallest
#' y-value. After this anchor row is selected, adj_fact can shift the final
#' label anchor outward or inward along the relevant axis.
#'
#' @param data A data frame containing the variables to be labelled.
#' @param x,y Unquoted column names giving the x- and y-variables.
#' @param group Unquoted column name giving the grouping variable with which to
#' label lines.
#' @param placement Character string giving the preferred label placement
#' relative to the target point for each group's line. One of "top", "right",
#' "bottom", or "left".
#' @param adj_fact A single numeric value giving the proportional adjustment
#' applied to the selected label anchor. Values are interpreted relative to
#' the group-specific x- or y-range.
#' @param facet_vars Optional character vector of facet variable names of max
#' length 2 (or else NULL). When supplied, anchor points are calculated
#' separately within each group-by-facet combination.
#'
#' @return A tibble with one row per group or group-by-facet combination. The
#' returned data include standardized x and y columns, .label_plus, and
#' any grouping or faceting variables needed by ggplot2 to assign labels to
#' panels correctly.
#'
#' @details
#' This helper is intended for ordinary Cartesian line-like geometries where
#' endpoint or extreme-value labeling is meaningful. It does not account for
#' data generated by ggplot2 statistics, transformed scales, or coordinate
#' transformations. For fitted lines or smooths, callers should pass the
#' pre-computed fitted data.
#'
#' @keywords internal
.directlabel_lines = function(data,
                             x,
                             y,
                             group,
                             placement,
                             adj_fact,
                             facet_vars) {

  group_name = rlang::as_name(rlang::enquo(group))

  label_data = data %>%
    dplyr::mutate(
      x = {{x}},
      y = {{y}},
      .label_plus = as.character({{group}})
    )

  group_vars = unique(c(group_name, facet_vars))

  label_data %>%
    dplyr::group_by(dplyr::across(dplyr::all_of(group_vars))) %>%
    dplyr::mutate(
      x_range = diff(range(x, na.rm = TRUE)),
      y_range = diff(range(y, na.rm = TRUE)),
      x_range = dplyr::if_else(x_range == 0, 1, x_range),
      y_range = dplyr::if_else(y_range == 0, 1, y_range)
    ) %>%
    {
      if(placement == "right") { #THE CALCS FOR TARGET LOCATIONS WITH LINES ARE SIMPLER THAN FOR POINTS.
        dplyr::slice_max(., x, n = 1, with_ties = FALSE)
      } else if(placement == "left") {
        dplyr::slice_min(., x, n = 1, with_ties = FALSE)
      } else if(placement == "top") {
        dplyr::slice_max(., y, n = 1, with_ties = FALSE)
      } else {
        dplyr::slice_min(., y, n = 1, with_ties = FALSE)
      }
    } %>%
    dplyr::mutate(
      x = if(placement == "right") {
        x + (adj_fact * x_range)
      } else if(placement == "left") {
        x - (adj_fact * x_range)
      } else {
        x #PRESERVES THIS COL IN ALL OTHER CASES.
      },
      y = if(placement == "top") {
        y + (adj_fact * y_range)
      } else if(placement == "bottom") {
        y - (adj_fact * y_range)
      } else {
        y
      }
    ) %>%
    dplyr::select(.label_plus, x, y, dplyr::any_of(group_vars)) %>%
    dplyr::ungroup()

}


#' Apply replacement labels for direct labels
#'
#' .apply_key_labels_plus() is an internal helper used by
#' [direct_labels_plus()] to replace default group labels with user-supplied
#' labels.
#'
#' @param label_data A character vector of labels, i.e., the .label_plus
#' column produced by directlabel_points() or directlabel_lines().
#' @param key_labels Optional replacement labels. May be one of:
#' \itemize{
#' \item NULL, in which case label_data is returned unchanged;
#' \item a function applied to label_data, such as a labelling function;
#' \item a named vector of the form c("old_label" = "New label"); or
#' \item an unnamed vector with one replacement label per unique group.
#' }
#'
#' @return A character vector of labels with the same length as label_data.
#'
#' @details
#' Named key_labels are treated as an explicit lookup table. Every unique
#' value in label_data must appear among the names of key_labels; otherwise,
#' the helper fails with an informative error.
#'
#' Unnamed key_labels are assigned to unique labels in alphanumeric order.
#' This behavior may be surprising, so the helper emits a message recommending
#' named labels when users want more control.
#'
#' @keywords internal
.apply_key_labels_plus = function(label_data, key_labels) {

  #IF NO CUSTOM LABELS, BAIL.
  if(is.null(key_labels)) {
    return(label_data)
  }

  #IF USER SUPPLIED A LABELLING FUNCTION, TRY TO APPLY IT.
  if(is.function(key_labels)) {

    label_data = key_labels(label_data)
    return(label_data)

  }

  #OTHERWISE, IF THEY GAVE A NAMED VECTOR OF LABELS.
  if(!is.null(names(key_labels))) {

    if(any(names(key_labels) == "")) {
      stop("All elements of named `key_labels` must have names.", call. = FALSE)
    }


    old_labels = sort(unique(as.character(label_data))) #SINCE WE'RE LINING UP ON NAMES ANYWAY, IT DOESN'T MATTER THAT WE SORT HERE.
    missing_labels = setdiff(old_labels, names(key_labels))

    if(length(missing_labels) > 0) {
      stop(
        "`key_labels` is missing replacement(s) for: ",
        paste(missing_labels, collapse = ", "),
        call. = FALSE
      )
    }

    #THEN, SWAP OLD LABELS FOR NEW USING THE NAMED VECTOR AS A LOOKUP.
    new_labels = unname(key_labels[as.character(label_data)])

    label_data = ifelse(
      is.na(new_labels),
      as.character(label_data),
      new_labels
    )

    return(label_data)

  }

  #IF IT'S NOT A NAMED VECTOR, ASSIGN THE LABELS

  if(length(key_labels) != #<-BY NOT UNIQUEING HERE, WE PUNISH GIVING THE SAME LABEL TWICE UNINTENTIONALLY.
     length(unique(label_data))) {
    stop("`key_labels` must have one label per group. Use a named vector of the form c(\"old_name\" = \"new_name\") to ensure labels are properly assigned to groups.", call. = FALSE)
  }

  old_labels = sort(unique(label_data))
  new_labels = key_labels
  names(new_labels) = old_labels #TO SEARCH BY OLD AND REPLACE WITH NEW, YOU'LL USE THE OLD NAMES AS INDEXES.

  message("Assigning labels to groups in alphanumeric order...Use a named vector to assign them manually.")

  label_data = unname(new_labels[as.character(label_data)])

  return(label_data)

}

# Shared minimal test data ------------------------------------------------

point_data = data.frame(
  x     = c(1, 2, 3, 4, 5, 6),
  y     = c(1, 3, 2, 6, 5, 7),
  group = c("A", "A", "A", "B", "B", "B")
)

line_data = data.frame(
  x     = c(1, 2, 3, 1, 2, 3),
  y     = c(1, 3, 2, 4, 6, 5),
  group = c("A", "A", "A", "B", "B", "B")
)

facet_data = data.frame(
  x      = c(1, 2, 3, 4, 1, 2, 3, 4),
  y      = c(1, 2, 3, 4, 5, 6, 7, 8),
  group  = c("A", "A", "B", "B", "A", "A", "B", "B"),
  panel  = c("p1", "p1", "p1", "p1", "p2", "p2", "p2", "p2")
)


# .directlabel_points() ---------------------------------------------------

test_that(".directlabel_points returns one row per group", {
  result = ggplotplus:::.directlabel_points(
    data      = point_data,
    x         = x,
    y         = y,
    group     = group,
    placement = "top",
    adj_fact  = 0,
    facet_vars = NULL
  )

  expect_equal(nrow(result), 2L)
  expect_setequal(result$.label_plus, c("A", "B"))
})

test_that(".directlabel_points returns required columns", {
  result = ggplotplus:::.directlabel_points(
    data      = point_data,
    x         = x,
    y         = y,
    group     = group,
    placement = "top",
    adj_fact  = 0,
    facet_vars = NULL
  )

  expect_true(all(c("x", "y", ".label_plus", "group") %in% names(result)))
})

test_that(".directlabel_points works for all four placements", {
  for(placement in c("top", "bottom", "left", "right")) {
    result = ggplotplus:::.directlabel_points(
      data      = point_data,
      x         = x,
      y         = y,
      group     = group,
      placement = placement,
      adj_fact  = 0,
      facet_vars = NULL
    )
    expect_equal(nrow(result), 2L, info = paste("placement =", placement))
  }
})

test_that(".directlabel_points top placement picks from near the max y", {
  result = ggplotplus:::.directlabel_points(
    data      = point_data,
    x         = x,
    y         = y,
    group     = group,
    placement = "top",
    adj_fact  = 0,
    facet_vars = NULL
  )

  # For group A (y = 1, 3, 2), closest to max y = 3 should be row with y = 3
  expect_equal(result$y[result$.label_plus == "A"], 3)
  # For group B (y = 6, 5, 7), closest to max y = 7 should be row with y = 7
  expect_equal(result$y[result$.label_plus == "B"], 7)
})

test_that(".directlabel_points bottom placement picks from near the min y", {
  result = ggplotplus:::.directlabel_points(
    data      = point_data,
    x         = x,
    y         = y,
    group     = group,
    placement = "bottom",
    adj_fact  = 0,
    facet_vars = NULL
  )

  expect_equal(result$y[result$.label_plus == "A"], 1)
  expect_equal(result$y[result$.label_plus == "B"], 5)
})

test_that(".directlabel_points right placement picks from near the max x", {
  result = ggplotplus:::.directlabel_points(
    data      = point_data,
    x         = x,
    y         = y,
    group     = group,
    placement = "right",
    adj_fact  = 0,
    facet_vars = NULL
  )

  expect_equal(result$x[result$.label_plus == "A"], 3)
  expect_equal(result$x[result$.label_plus == "B"], 6)
})

test_that(".directlabel_points with facet_vars returns one row per group-facet combo", {
  result = ggplotplus:::.directlabel_points(
    data       = facet_data,
    x          = x,
    y          = y,
    group      = group,
    placement  = "top",
    adj_fact   = 0,
    facet_vars = "panel"
  )

  expect_equal(nrow(result), 4L) # 2 groups * 2 panels
})

test_that(".directlabel_points doesn't error when a group has a single point (zero range)", {
  single_point_data = data.frame(
    x     = c(1, 2, 3, 5),
    y     = c(1, 2, 3, 5),
    group = c("A", "A", "A", "B") # B has one point
  )

  expect_no_error(
    ggplotplus:::.directlabel_points(
      data      = single_point_data,
      x         = x,
      y         = y,
      group     = group,
      placement = "top",
      adj_fact  = 0,
      facet_vars = NULL
    )
  )
})


# .directlabel_lines() ----------------------------------------------------

test_that(".directlabel_lines returns one row per group", {
  result = ggplotplus:::.directlabel_lines(
    data      = line_data,
    x         = x,
    y         = y,
    group     = group,
    placement = "right",
    adj_fact  = 0,
    facet_vars = NULL
  )

  expect_equal(nrow(result), 2L)
  expect_setequal(result$.label_plus, c("A", "B"))
})

test_that(".directlabel_lines right placement picks the max-x row per group", {
  result = ggplotplus:::.directlabel_lines(
    data      = line_data,
    x         = x,
    y         = y,
    group     = group,
    placement = "right",
    adj_fact  = 0,
    facet_vars = NULL
  )

  expect_equal(result$x[result$.label_plus == "A"], 3)
  expect_equal(result$x[result$.label_plus == "B"], 3)
})

test_that(".directlabel_lines left placement picks the min-x row per group", {
  result = ggplotplus:::.directlabel_lines(
    data      = line_data,
    x         = x,
    y         = y,
    group     = group,
    placement = "left",
    adj_fact  = 0,
    facet_vars = NULL
  )

  expect_equal(result$x[result$.label_plus == "A"], 1)
  expect_equal(result$x[result$.label_plus == "B"], 1)
})

test_that(".directlabel_lines top placement picks the max-y row per group", {
  result = ggplotplus:::.directlabel_lines(
    data      = line_data,
    x         = x,
    y         = y,
    group     = group,
    placement = "top",
    adj_fact  = 0,
    facet_vars = NULL
  )

  # Group A: y = 1, 3, 2 -> max is 3
  expect_equal(result$y[result$.label_plus == "A"], 3)
  # Group B: y = 4, 6, 5 -> max is 6
  expect_equal(result$y[result$.label_plus == "B"], 6)
})

test_that(".directlabel_lines bottom placement picks the min-y row per group", {
  result = ggplotplus:::.directlabel_lines(
    data      = line_data,
    x         = x,
    y         = y,
    group     = group,
    placement = "bottom",
    adj_fact  = 0,
    facet_vars = NULL
  )

  expect_equal(result$y[result$.label_plus == "A"], 1)
  expect_equal(result$y[result$.label_plus == "B"], 4)
})

test_that(".directlabel_lines adj_fact shifts anchor outward for right placement", {
  result_no_adj = ggplotplus:::.directlabel_lines(
    data      = line_data,
    x         = x,
    y         = y,
    group     = group,
    placement = "right",
    adj_fact  = 0,
    facet_vars = NULL
  )

  result_adj = ggplotplus:::.directlabel_lines(
    data      = line_data,
    x         = x,
    y         = y,
    group     = group,
    placement = "right",
    adj_fact  = 0.1,
    facet_vars = NULL
  )

  expect_true(all(result_adj$x > result_no_adj$x))
})

test_that(".directlabel_lines with facet_vars returns one row per group-facet combo", {
  result = ggplotplus:::.directlabel_lines(
    data       = facet_data,
    x          = x,
    y          = y,
    group      = group,
    placement  = "right",
    adj_fact   = 0,
    facet_vars = "panel"
  )

  expect_equal(nrow(result), 4L)
})


# .apply_key_labels_plus() ------------------------------------------------

test_that(".apply_key_labels_plus returns data unchanged when key_labels is NULL", {
  labels = c("A", "B", "A", "B")
  expect_equal(ggplotplus:::.apply_key_labels_plus(labels, NULL), labels)
})

test_that(".apply_key_labels_plus applies a function to labels", {
  labels = c("a", "b", "a")
  result = ggplotplus:::.apply_key_labels_plus(labels, toupper)
  expect_equal(result, c("A", "B", "A"))
})

test_that(".apply_key_labels_plus swaps labels using a named vector", {
  labels = c("A", "B", "A")
  result = ggplotplus:::.apply_key_labels_plus(labels, c("A" = "Group A", "B" = "Group B"))
  expect_equal(result, c("Group A", "Group B", "Group A"))
})

test_that(".apply_key_labels_plus errors when a named vector is missing a group", {
  labels = c("A", "B", "C")
  expect_error(
    ggplotplus:::.apply_key_labels_plus(labels, c("A" = "Group A", "B" = "Group B")),
    regexp = "missing replacement"
  )
})

test_that(".apply_key_labels_plus errors when a named vector has any unnamed elements", {
  labels = c("A", "B")
  expect_error(
    ggplotplus:::.apply_key_labels_plus(labels, c("A" = "Group A", "Group B")),
    regexp = "must have names"
  )
})

test_that(".apply_key_labels_plus assigns unnamed vector in alphanumeric order with a message", {
  labels = c("B", "A", "B", "A")
  expect_message(
    result <- ggplotplus:::.apply_key_labels_plus(labels, c("Alpha", "Beta")),
    regexp = "alphanumeric"
  )
  # A -> Alpha, B -> Beta
  expect_equal(result, c("Beta", "Alpha", "Beta", "Alpha"))
})

test_that(".apply_key_labels_plus errors when unnamed vector length doesn't match group count", {
  labels = c("A", "B", "C")
  expect_error(
    ggplotplus:::.apply_key_labels_plus(labels, c("X", "Y")),
    regexp = "one label per group"
  )
})


# direct_labels_plus() input validation -----------------------------------

test_that("direct_labels_plus errors when data is not a data frame", {
  expect_error(
    direct_labels_plus(data = "not a data frame", x = x, y = y, group = group),
    regexp = "data frame"
  )
})

test_that("direct_labels_plus errors on invalid placement", {
  expect_error(
    direct_labels_plus(data = point_data, x = x, y = y, group = group, placement = "diagonal"),
    regexp = "arg"
  )
})

test_that("direct_labels_plus errors on invalid geometry", {
  expect_error(
    direct_labels_plus(data = point_data, x = x, y = y, group = group, geometry = "ribbon"),
    regexp = "arg"
  )
})

test_that("direct_labels_plus errors when group is an expression rather than a bare name", {
  expect_error(
    direct_labels_plus(data = point_data, x = x, y = y, group = toupper(group)),
    regexp = "bare column name"
  )
})

test_that("direct_labels_plus errors when adj_fact is not a single numeric", {
  expect_error(
    direct_labels_plus(data = point_data, x = x, y = y, group = group, adj_fact = "a lot"),
    regexp = "numeric"
  )
  expect_error(
    direct_labels_plus(data = point_data, x = x, y = y, group = group, adj_fact = c(0.1, 0.2)),
    regexp = "numeric"
  )
})

test_that("direct_labels_plus errors when facet_vars contains more than two names", {
  expect_error(
    direct_labels_plus(data = point_data, x = x, y = y, group = group, facet_vars = c("a", "b", "c")),
    regexp = "<= 2"
  )
})

test_that("direct_labels_plus errors when facet_vars names are not in data", {
  expect_error(
    direct_labels_plus(data = point_data, x = x, y = y, group = group, facet_vars = "nonexistent"),
    regexp = "not found"
  )
})


# direct_labels_plus() output ---------------------------------------------

test_that("direct_labels_plus returns a ggplot2 layer", {
  layer = direct_labels_plus(
    data      = point_data,
    x         = x,
    y         = y,
    group     = group,
    placement = "top",
    geometry  = "point"
  )

  expect_true(inherits(layer, "Layer"))
})

test_that("direct_labels_plus builds successfully for point geometry", {
  p = ggplot2::ggplot(point_data, ggplot2::aes(x, y, color = group)) +
    ggplot2::geom_point() +
    direct_labels_plus(
      data      = point_data,
      x         = x,
      y         = y,
      group     = group,
      placement = "top",
      geometry  = "point"
    )

  expect_s3_class(ggplot2::ggplot_build(p), "ggplot_built")
})

test_that("direct_labels_plus builds successfully for line geometry", {
  p = ggplot2::ggplot(line_data, ggplot2::aes(x, y, color = group)) +
    ggplot2::geom_line(ggplot2::aes(group = group)) +
    direct_labels_plus(
      data      = line_data,
      x         = x,
      y         = y,
      group     = group,
      placement = "right",
      geometry  = "line"
    )

  expect_s3_class(ggplot2::ggplot_build(p), "ggplot_built")
})

test_that("direct_labels_plus with key_labels named vector builds without error", {
  p = ggplot2::ggplot(point_data, ggplot2::aes(x, y, color = group)) +
    ggplot2::geom_point() +
    direct_labels_plus(
      data       = point_data,
      x          = x,
      y          = y,
      group      = group,
      placement  = "top",
      geometry   = "point",
      key_labels = c("A" = "Group A", "B" = "Group B")
    )

  expect_s3_class(ggplot2::ggplot_build(p), "ggplot_built")
})

test_that("direct_labels_plus with facet_vars builds without error", {
  p = ggplot2::ggplot(facet_data, ggplot2::aes(x, y, color = group)) +
    ggplot2::geom_point() +
    ggplot2::facet_wrap(~panel) +
    direct_labels_plus(
      data       = facet_data,
      x          = x,
      y          = y,
      group      = group,
      placement  = "top",
      geometry   = "point",
      facet_vars = "panel"
    )

  expect_s3_class(ggplot2::ggplot_build(p), "ggplot_built")
})

test_that("direct_labels_plus passes dot args through to geom_label_repel", {
  # size is one of the overridable ggrepel defaults; passing a custom value
  # should not produce an error or warning
  expect_no_error(
    direct_labels_plus(
      data      = point_data,
      x         = x,
      y         = y,
      group     = group,
      placement = "top",
      geometry  = "point",
      size      = 3
    )
  )
})

test_that("scale_focus_plus returns a ggplot2 scale for colour and fill", {

  group_var = c("a", "b", "c", "d")

  expect_s3_class(
    scale_focus_plus(aes = "colour",
                     group_var = group_var,
                     focal_groups = "a"),
    "Scale"
  )

  expect_s3_class(
    scale_focus_plus(aes = "color",
                     group_var = group_var,
                     focal_groups = "a"),
    "Scale"
  )

  expect_s3_class(
    scale_focus_plus(aes = "fill",
                     group_var = group_var,
                     focal_groups = "a"),
    "Scale"
  )
})


test_that("scale_focus_plus applies one shared non-focal color by default", {

  dat = data.frame(
    x = 1:4,
    y = 1:4,
    group = c("a", "b", "c", "d")
  )

  p = ggplot2::ggplot(dat, ggplot2::aes(x = x,
                                        y = y,
                                        colour = group)) +
    ggplot2::geom_point(size = 4) +
    scale_focus_plus(aes = "colour",
                     group_var = dat$group,
                     focal_groups = "a")

  built = ggplot2::ggplot_build(p)
  plot_cols = built$data[[1]]$colour
  names(plot_cols) = dat$group

  expect_length(unique(plot_cols[names(plot_cols) != "a"]), 1)
  expect_false(plot_cols["a"] %in% plot_cols[names(plot_cols) != "a"])
})


test_that("scale_focus_plus differentiates non-focal colors when requested", {

  dat = data.frame(
    x = 1:4,
    y = 1:4,
    group = c("a", "b", "c", "d")
  )

  p = ggplot2::ggplot(dat, ggplot2::aes(x = x,
                                        y = y,
                                        colour = group)) +
    ggplot2::geom_point(size = 4) +
    scale_focus_plus(aes = "colour",
                     group_var = dat$group,
                     focal_groups = "a",
                     diff_nonfocal = TRUE)

  built = ggplot2::ggplot_build(p)
  plot_cols = built$data[[1]]$colour
  names(plot_cols) = dat$group

  expect_length(unique(plot_cols[names(plot_cols) != "a"]), 3)
})


test_that("scale_focus_plus differentiates focal colors by default", {

  dat = data.frame(
    x = 1:5,
    y = 1:5,
    group = c("a", "b", "c", "d", "e")
  )

  p = ggplot2::ggplot(dat, ggplot2::aes(x = x,
                                        y = y,
                                        colour = group)) +
    ggplot2::geom_point(size = 4) +
    scale_focus_plus(aes = "colour",
                     group_var = dat$group,
                     focal_groups = c("a", "b"))

  built = ggplot2::ggplot_build(p)
  plot_cols = built$data[[1]]$colour
  names(plot_cols) = dat$group

  expect_false(identical(plot_cols["a"], plot_cols["b"]))
  expect_length(unique(plot_cols[c("c", "d", "e")]), 1)
})


test_that("scale_focus_plus can use one shared focal color", {

  dat = data.frame(
    x = 1:5,
    y = 1:5,
    group = c("a", "b", "c", "d", "e")
  )

  p = ggplot2::ggplot(dat, ggplot2::aes(x = x,
                                        y = y,
                                        colour = group)) +
    ggplot2::geom_point(size = 4) +
    scale_focus_plus(aes = "colour",
                     group_var = dat$group,
                     focal_groups = c("a", "b"),
                     diff_focal = FALSE)

  built = ggplot2::ggplot_build(p)
  plot_cols = built$data[[1]]$colour
  names(plot_cols) = dat$group

  expect_identical(unname(plot_cols["a"]), unname(plot_cols["b"]))
  expect_false(plot_cols["a"] %in% plot_cols[c("c", "d", "e")])
})


test_that("scale_focus_plus respects custom differentiated focal colors", {

  dat = data.frame(
    x = 1:5,
    y = 1:5,
    group = c("a", "b", "c", "d", "e")
  )

  p = ggplot2::ggplot(dat, ggplot2::aes(x = x,
                                        y = y,
                                        colour = group)) +
    ggplot2::geom_point(size = 4) +
    scale_focus_plus(aes = "colour",
                     group_var = dat$group,
                     focal_groups = c("a", "b"),
                     custom_focal = c(a = "red", b = "blue"))

  built = ggplot2::ggplot_build(p)
  plot_cols = built$data[[1]]$colour
  names(plot_cols) = dat$group

  expect_identical(unname(plot_cols["a"]), "red")
  expect_identical(unname(plot_cols["b"]), "blue")
})


test_that("scale_focus_plus respects one shared custom focal color", {

  dat = data.frame(
    x = 1:5,
    y = 1:5,
    group = c("a", "b", "c", "d", "e")
  )

  p = ggplot2::ggplot(dat, ggplot2::aes(x = x,
                                        y = y,
                                        colour = group)) +
    ggplot2::geom_point(size = 4) +
    scale_focus_plus(aes = "colour",
                     group_var = dat$group,
                     focal_groups = c("a", "b"),
                     diff_focal = FALSE,
                     custom_focal = "purple")

  built = ggplot2::ggplot_build(p)
  plot_cols = built$data[[1]]$colour
  names(plot_cols) = dat$group

  expect_identical(unname(plot_cols["a"]), "purple")
  expect_identical(unname(plot_cols["b"]), "purple")
})


test_that("scale_focus_plus respects custom differentiated non-focal colors", {

  dat = data.frame(
    x = 1:4,
    y = 1:4,
    group = c("a", "b", "c", "d")
  )

  p = ggplot2::ggplot(dat, ggplot2::aes(x = x,
                                        y = y,
                                        colour = group)) +
    ggplot2::geom_point(size = 4) +
    scale_focus_plus(aes = "colour",
                     group_var = dat$group,
                     focal_groups = "a",
                     diff_nonfocal = TRUE,
                     custom_nonfocal = c(b = "gray20",
                                         c = "gray50",
                                         d = "gray80"))

  built = ggplot2::ggplot_build(p)
  plot_cols = built$data[[1]]$colour
  names(plot_cols) = dat$group

  expect_identical(unname(plot_cols["b"]), "gray20")
  expect_identical(unname(plot_cols["c"]), "gray50")
  expect_identical(unname(plot_cols["d"]), "gray80")
})


test_that("scale_focus_plus works for fill scales", {

  dat = data.frame(
    group = c("a", "b", "c", "d"),
    value = c(5, 8, 3, 6)
  )

  p = ggplot2::ggplot(dat, ggplot2::aes(x = group,
                                        y = value,
                                        fill = group)) +
    ggplot2::geom_col() +
    scale_focus_plus(aes = "fill",
                     group_var = dat$group,
                     focal_groups = "b",
                     custom_focal = "orange",
                     diff_focal = FALSE)

  built = ggplot2::ggplot_build(p)
  plot_fills = built$data[[1]]$fill
  names(plot_fills) = dat$group

  expect_identical(unname(plot_fills["b"]), "orange")
  expect_length(unique(plot_fills[names(plot_fills) != "b"]), 1)
})


test_that("scale_focus_plus forwards arguments to scale manual", {

  dat = data.frame(
    x = 1:4,
    y = 1:4,
    group = c("a", "b", "c", "d")
  )

  p = ggplot2::ggplot(dat, ggplot2::aes(x = x,
                                        y = y,
                                        colour = group)) +
    ggplot2::geom_point(size = 4) +
    scale_focus_plus(aes = "colour",
                     group_var = dat$group,
                     focal_groups = "a",
                     name = "My grouping variable",
                     breaks = c("a", "b"))

  built = ggplot2::ggplot_build(p)
  trained_scale = built$plot$scales$get_scales("colour")

  expect_equal(trained_scale$name, "My grouping variable")
  expect_equal(
    as.character(trained_scale$get_breaks()), #<--OTHERWISE, CHECKS THE POSITIONS ATTR.--THE LABELS ARE THE NAMES HERE.
    c("a", "b")
  )
})


test_that("scale_focus_plus warns and ignores user-provided values", {

  dat = data.frame(
    x = 1:4,
    y = 1:4,
    group = c("a", "b", "c", "d")
  )

  expect_warning(
    { #THE FIRST ARG NEEDS TO BE "OBJECT" SO WE CAN'T DO ASSIGNMENT "INSIDE" THAT ARG UNLESS WE BRACE TO SCOPE IT.
      scale_obj = scale_focus_plus(aes = "colour",
                                   group_var = dat$group,
                                   focal_groups = "a",
                                   values = c(a = "black",
                                              b = "black",
                                              c = "black",
                                              d = "black"))
    },
    regexp = "Don't provide a `values` argument"
  )

  p = ggplot2::ggplot(dat, ggplot2::aes(x = x,
                                        y = y,
                                        colour = group)) +
    ggplot2::geom_point(size = 4) +
    scale_obj

  built = ggplot2::ggplot_build(p)
  plot_cols = built$data[[1]]$colour

  expect_false(all(plot_cols == "black"))
})


test_that("scale_focus_plus rejects invalid inputs", {

  group_var = c("a", "b", "c")

  expect_error(
    scale_focus_plus(aes = "alpha",
                     group_var = group_var,
                     focal_groups = "a"),
    regexp = "'arg' should be one of"
  )

  expect_error(
    scale_focus_plus(aes = "colour",
                     group_var = c("a", "a", "a"),
                     focal_groups = "a"),
    regexp = "Invalid `group_var`"
  )

  expect_error(
    scale_focus_plus(aes = "colour",
                     group_var = group_var,
                     focal_groups = 1),
    regexp = "Invalid `focal_groups`"
  )

  expect_error(
    scale_focus_plus(aes = "colour",
                     group_var = group_var,
                     focal_groups = "z"),
    regexp = "must be present"
  )

  expect_error(
    scale_focus_plus(aes = "colour",
                     group_var = group_var,
                     focal_groups = c("a", "b", "c")),
    regexp = "not enough unique levels"
  )

  expect_error(
    scale_focus_plus(aes = "colour",
                     group_var = group_var,
                     focal_groups = "a",
                     gray_start = -0.1),
    regexp = "between"
  )

  expect_error(
    scale_focus_plus(aes = "colour",
                     group_var = group_var,
                     focal_groups = "a",
                     custom_focal = c(a = "red", b = "blue")),
    regexp = "Invalid `custom_focal`"
  )

  expect_error(
    scale_focus_plus(aes = "colour",
                     group_var = group_var,
                     focal_groups = "a",
                     diff_nonfocal = TRUE,
                     custom_nonfocal = c(b = "gray20")),
    regexp = "Invalid `custom_nonfocal`"
  )
})


test_that("scale_focus_plus rejects invalid diff_focal and diff_nonfocal inputs", {

  group_var = c("a", "b", "c")

  expect_error(
    scale_focus_plus(aes = "colour",
                     group_var = group_var,
                     focal_groups = "a",
                     diff_focal = NA),
    regexp = "length-1 logical"
  )

  expect_error(
    scale_focus_plus(aes = "colour",
                     group_var = group_var,
                     focal_groups = "a",
                     diff_nonfocal = NA),
    regexp = "length-1 logical"
  )

  expect_error(
    scale_focus_plus(aes = "colour",
                     group_var = group_var,
                     focal_groups = "a",
                     diff_focal = c(TRUE, FALSE)),
    regexp = "length-1 logical"
  )

  expect_error(
    scale_focus_plus(aes = "colour",
                     group_var = group_var,
                     focal_groups = "a",
                     diff_nonfocal = c(TRUE, FALSE)),
    regexp = "length-1 logical"
  )

  expect_error(
    scale_focus_plus(aes = "colour",
                     group_var = group_var,
                     focal_groups = "a",
                     diff_focal = 1),
    regexp = "length-1 logical"
  )

  expect_error(
    scale_focus_plus(aes = "colour",
                     group_var = group_var,
                     focal_groups = "a",
                     diff_nonfocal = "yes"),
    regexp = "length-1 logical"
  )

  expect_error(
    scale_focus_plus(aes = "colour",
                     group_var = group_var,
                     focal_groups = "a",
                     diff_focal = NULL),
    regexp = "length-1 logical"
  )

  expect_error(
    scale_focus_plus(aes = "colour",
                     group_var = group_var,
                     focal_groups = "a",
                     diff_nonfocal = NULL),
    regexp = "length-1 logical"
  )
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.get_prebuild_aes_label}
\alias{.get_prebuild_aes_label}
\title{Retrieve the pre-build label for an aesthetic}
\usage{
.get_prebuild_aes_label(plot, aes_name)
}
\arguments{
\item{plot}{A ggplot object.}

\item{aes_name}{A single character string giving the aesthetic whose
pre-build label should be retrieved.}
}
\value{
A character string representing the best available pre-build label
for the requested aesthetic.
}
\description{
Internal helper that attempts to determine the eventual user-facing label
associated with a mapped aesthetic before the plot has been formally built.
}
\details{
The helper first checks for explicit labels supplied through
\code{ggplot2::labs()} or related mechanisms. If no explicit label is found,
it falls back to the mapped expression itself using
\code{rlang::as_label()}.

Both plot-level and layer-level mappings are searched.

ggplotplus uses this helper during pre-build guide processing to identify
aesthetics that are likely to collapse into a shared legend, such as when
both \code{fill} and \code{shape} are mapped to the same variable.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.is_log10_transformed_scale}
\alias{.is_log10_transformed_scale}
\title{Check whether an axis uses a log10 scale transformation}
\usage{
.is_log10_transformed_scale(built, axis)
}
\arguments{
\item{built}{A built ggplot object, typically produced by
ggplot2::ggplot_build().}

\item{axis}{A character string giving the axis to inspect. Expected values are
"x" or "y".}
}
\value{
A single logical value.
}
\description{
Internal helper used by bar/column coaching checks. Ordinary bars encode
values from a baseline of 0, but ggplot2 places the bar baseline at 1 on
log-scaled axes. This helper detects the log10 case so the coaching check can
test for visibility of the appropriate baseline.
}
\details{
The transformation is checked from the first panel because transformations are
scale-level properties rather than panel-specific properties. This helper
relies on ggplot2's built-plot internals and should therefore be treated as
best-effort/version-sensitive.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\docType{data}
\name{ggplotplus_shapes_list}
\alias{ggplotplus_shapes_list}
\title{Custom shape palette for \code{geom_point_plus()}}
\format{
A named list of data frames. Each data frame has columns:
\describe{
\item{x}{X coordinates for the shape outline.}
\item{y}{Y coordinates for the shape outline.}
\item{piece}{Integer identifier for separate polygon pieces or holes.}
}
}
\usage{
ggplotplus_shapes_list
}
\description{
A named list of custom point shapes used by \code{geom_point_plus()}. Each element
contains \code{x} and \code{y} coordinates plus a \code{piece} identifier used when drawing
filled shapes and holes.
}
\examples{
names(ggplotplus_shapes_list)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UserFacingCoreFunctions.R
\name{theme_plus}
\alias{theme_plus}
\title{A Universal Design-Oriented Base Ggplot2 Theme With Scalable and Overridable Defaults}
\usage{
theme_plus(
  ...,
  legend_pos = "top",
  base_font_size = 16,
  base_linewidth = 1.2,
  base_rectlinewidth = 1.2,
  line_color = "black",
  text_color = "black",
  background_color = "#FFFEFD",
  palette_discrete = "D",
  palette_continuous = "E",
  begin_discrete = 0,
  end_discrete = 0.72,
  begin_continuous = 0,
  end_continuous = 1,
  export_width = 7.25,
  export_height = 5.95,
  override_legend_alphasize = TRUE,
  enable_coaching = TRUE
)
}
\arguments{
\item{...}{Optional additional theme settings passed to \code{\link[ggplot2:theme]{ggplot2::theme()}}. These are applied \emph{after} the base theme, so the theme's defaults only "win" when no matching settings are provided by the user
(same as in ggplot2).}

\item{legend_pos}{Where to put the legend. \code{"top"} (default) creates a
horizontal stripe at the top for the legend (box) when one is present;
\code{"right"} uses a vertical legend at the right (ggplot2’s usual position)
but with design modifications. \code{"bottom"} is also an option (largely same
as \code{"top"}).}

\item{base_font_size}{Base text size (in points) for most text elements. These
will scale via \code{rel()}. Default is \code{16}.}

\item{base_linewidth}{Baseline thickness for most \strong{line} theme elements
(e.g., axis lines and tick marks). Defaults to \code{1.2}. Specific elements
may use \code{rel()} multipliers on top of this.}

\item{base_rectlinewidth}{Baseline line thickness for most \strong{rect} theme elements
(e.g., legend frames). Defaults to \code{1.2}.}

\item{line_color}{Default color for most line elements (axis lines, frames, etc.).
Defaults to \code{"black"}.}

\item{text_color}{Default color for most text elements. Defaults to \code{"black"}.}

\item{background_color}{Background fill applied to the panel, plot, legend,
and strip backgrounds. Defaults to a slightly warm white, \code{"#FFFEFD"}, to reduce eyestrain.}

\item{palette_discrete, palette_continuous}{Default viridis-family color palette codes ("A" through "H") to use for discrete and continuous scales, respectively.}

\item{begin_discrete, end_discrete, begin_continuous, end_continuous}{Numeric values ranging between 0 and 1 for where to begin drawing colors from a viridis palette for a discrete and continuous color scale, respectively.}

\item{export_width, export_height}{Length-1 numeric values indicating your intended export (most likely via ggplot2::ggsave()) width and height, respectively. This rescales font and line sizes internally to stay relatively appropriately for your intended export size.}

\item{override_legend_alphasize}{Logical indicating whether, for any fill, color, and/or shape legends, to override the size and/or alpha values to the ggplotplus default values (1 and 5, respectively). Defaults to TRUE.}

\item{enable_coaching}{Logical indicating whether ggplotplus should perform certain automated checks for possible non-Universal graph design and provide coaching messages when these are detected. Defaults to TRUE.}
}
\value{
A ggplot2 theme object to add with \code{+}.
}
\description{
\code{theme_plus()} returns a ggplot2 theme designed to make
publication-quality, accessible graphs easier to produce. It keeps all of
ggplot2’s normal behaviors (last theme wins; user overrides take precedence),
but bakes in opinionated defaults with Universal Design in mind. A few knobs
let you scale typography/lines, flip the legend layout, and switch the
background color if desired.
}
\details{
Internally, text sizes are expressed with \code{rel()}, so they scale with
\code{base_font_size}. Line/rect line thicknesses start from \code{base_linewidth} and
\code{base_rectlinewidth} and scale similarly with \code{rel()}.

The function builds
a  base theme, then \emph{adds} any user overrides via
\code{theme(...)}, so the user's preferences always take precedence.
}
\examples{
# Basic use
library(ggplot2)
ggplot(iris, aes(Sepal.Length, Petal.Length, colour = Species)) +
  geom_point() +
  theme_plus()

# Prefer the right-side legend and pure white background
ggplot(mtcars, aes(wt, mpg)) +
  geom_point() +
  theme_plus(legend_pos = "right", background_color = "white")

# Scale text up and make lines a bit lighter
ggplot(iris, aes(Sepal.Length, Petal.Length)) +
  geom_point() +
  theme_plus(base_font_size = 18, base_linewidth = 1.0)

# You can still override any element normally via theme()
ggplot(iris, aes(Sepal.Length, Petal.Length)) +
  geom_point() +
  theme_plus() +
  theme(axis.line = element_line(linewidth = 0.8))

# But you could just as easily do so via theme_plus()
ggplot(iris, aes(Sepal.Length, Petal.Length)) +
  geom_point() +
  theme_plus(axis.line = element_line(linewidth = 0.8))

}
\seealso{
\code{\link[ggplot2:theme]{ggplot2::theme()}}, \code{\link[ggplot2:ggtheme]{ggplot2::theme_gray()}}, \code{\link[ggplot2:get_theme]{ggplot2::theme_get()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.s7_prop_is_true}
\alias{.s7_prop_is_true}
\title{Safely check whether an S7 property is TRUE}
\usage{
.s7_prop_is_true(object, prop)
}
\arguments{
\item{object}{An object that may or may not be an S7 object.}

\item{prop}{A single character string giving the property name
to check.}
}
\value{
A logical scalar. Returns \code{TRUE} only if the object
is a valid S7 object, the property exists, and the property
value is exactly \code{TRUE}. Otherwise returns \code{FALSE}.
}
\description{
Internal helper that safely checks whether an object:
\enumerate{
\item exists and is not \code{NULL},
\item is an S7 object,
\item contains a specified property, and
\item has that property set to \code{TRUE}.
}
}
\details{
This helper is primarily used to avoid errors when optional
ggplotplus S7 components may not (yet) exist on a plot object.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.directlabel_lines}
\alias{.directlabel_lines}
\title{Choose direct-label anchor points for grouped line data}
\usage{
.directlabel_lines(data, x, y, group, placement, adj_fact, facet_vars)
}
\arguments{
\item{data}{A data frame containing the variables to be labelled.}

\item{x, y}{Unquoted column names giving the x- and y-variables.}

\item{group}{Unquoted column name giving the grouping variable with which to
label lines.}

\item{placement}{Character string giving the preferred label placement
relative to the target point for each group's line. One of "top", "right",
"bottom", or "left".}

\item{adj_fact}{A single numeric value giving the proportional adjustment
applied to the selected label anchor. Values are interpreted relative to
the group-specific x- or y-range.}

\item{facet_vars}{Optional character vector of facet variable names of max
length 2 (or else NULL). When supplied, anchor points are calculated
separately within each group-by-facet combination.}
}
\value{
A tibble with one row per group or group-by-facet combination. The
returned data include standardized x and y columns, .label_plus, and
any grouping or faceting variables needed by ggplot2 to assign labels to
panels correctly.
}
\description{
directlabel_lines() is an internal helper used by
\code{\link[=direct_labels_plus]{direct_labels_plus()}} when geometry = "line". It chooses one endpoint or
extreme point per group, or per group-by-facet combination, to use as the
anchor location for a direct label. How the label is then drawn with respect
to that chosen point is determined by \code{\link[ggrepel:geom_text_repel]{ggrepel::geom_label_repel()}}.
}
\details{
For "right" placement, the helper selects the row with the largest x-value
in each group. For "left", it selects the smallest x-value. For "top",
it selects the largest y-value. For "bottom", it selects the smallest
y-value. After this anchor row is selected, adj_fact can shift the final
label anchor outward or inward along the relevant axis.

This helper is intended for ordinary Cartesian line-like geometries where
endpoint or extreme-value labeling is meaningful. It does not account for
data generated by ggplot2 statistics, transformed scales, or coordinate
transformations. For fitted lines or smooths, callers should pass the
pre-computed fitted data.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UserFacingCoreFunctions.R
\name{geom_point_plus_shapes}
\alias{geom_point_plus_shapes}
\title{Demo plot showing geom_point_plus() shapes}
\format{
A ggplot object.
}
\usage{
geom_point_plus_shapes()
}
\value{
Returns a ggplot2 graph showing the ggplotplus shapes palette.
}
\description{
A prebuilt ggplot object displaying the nine custom point shapes designed specifically for use in \code{geom_point_plus()}. As of Version 0.5.2, other shapes are also available, but those are not shown via this function. See the documentation for \code{geom_point_plus()} for details.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UserFacingCoreFunctions.R
\name{ggplotplus_to_cowplot}
\alias{ggplotplus_to_cowplot}
\title{Convert a ggplotplus plot into a grob}
\usage{
ggplotplus_to_cowplot(plot)
}
\arguments{
\item{plot}{A ggplot or ggplotplus plot object.}
}
\value{
A grob object suitable for use with grid-based plotting utilities
such as \code{cowplot::plot_grid()}.
}
\description{
Convenience helper for converting ggplotplus plot objects into grobs using
\code{ggplot2::ggplotGrob()}.
}
\details{
This is primarily intended for compatibility with packages such as
\pkg{cowplot} that may not yet fully recognize custom S7 plot subclasses
during internal dispatch.
}
\examples{
p = ggplot2::ggplot(iris, ggplot2::aes(Sepal.Length, Petal.Length, colour = Species)) +
  geom_point_plus()

if(requireNamespace("cowplot", quietly = TRUE)) {

cowplot::plot_grid(
  ggplotplus_to_cowplot(p),
  ggplotplus_to_cowplot(p)
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.pointplus_shapes}
\alias{.pointplus_shapes}
\title{Get registered geom_point_plus() shapes}
\usage{
.pointplus_shapes()
}
\value{
A named list of point-shape data frames.
}
\description{
Internal helper that returns the current ggplotplus point-shape registry,
including both built-in package shapes and any shapes added during the
current R session with \code{add_shape_plus()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.merge_legend_override}
\alias{.merge_legend_override}
\title{Merge ggplotplus legend-key overrides with an existing guide}
\usage{
.merge_legend_override(plot, aes_name, override_list)
}
\arguments{
\item{plot}{A ggplot object.}

\item{aes_name}{A single character string giving the aesthetic whose guide
should be modified.}

\item{override_list}{A named list of ggplotplus legend-key overrides, such as
\code{list(alpha = 1, size = 5)}.}
}
\value{
A guide object. If no guide exists for \code{aes_name}, returns a new
\code{ggplot2::guide_legend()} using \code{override_list}. If a guide
exists, returns a copy with merged \code{override.aes} values, where
existing user values win over ggplotplus defaults.
}
\description{
Internal helper that merges ggplotplus legend-key defaults with any
user-specified \code{override.aes} values for a guide.
}
\details{
ggplotplus uses this helper to improve legend readability by overriding
non-semantic, hard-to-read, constant \code{alpha} and \code{size} values in
legend keys. Existing user overrides are preserved and take precedence over
ggplotplus defaults for positive UX.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UserFacingCoreFunctions.R
\name{direct_labels_plus}
\alias{direct_labels_plus}
\title{Add direct labels to grouped point or line plots}
\usage{
direct_labels_plus(
  data,
  x,
  y,
  group,
  placement = "top",
  geometry = "point",
  adj_fact = 0,
  key_labels = NULL,
  facet_vars = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame containing the variables to be both plotted and
labelled. Most often, this will be the same data frame as supplied to
\code{\link[ggplot2:ggplot]{ggplot2::ggplot()}} but needn't be.}

\item{x, y}{Unquoted column names giving the x- and y-coordinates of the data
to be plotted and thus also labelled.}

\item{group}{Unquoted column name giving the grouping variable with which to
label the underlying data. Must be a single unquoted column name and not an
expression.}

\item{placement}{Where labels should be placed relative to each group.
One of "top", "right", "bottom", or "left". For geometry = "point", this
controls the target location used to choose a representative point from each
group. For geometry = "line", this chooses the endpoint or extreme point to
label. Experimenting with different placements to find the one that works
best for a particular graph is advised!}

\item{geometry}{The kind of geometry being labelled. Currently supports
"point" and "line".}

\item{adj_fact}{A single numeric value controlling how far label targets are
adjusted toward or away from the selected edge of each group. Values are
interpreted as a proportion of the group-specific x- or y-range. Positive
values move targets outward (towards the plot edge); negative values move
them inward. For geometry = "point", this changes the target spot used to
choose a particular point from each group to label, but does not move the
final label anchor away from the selected point. In practice, this is likely
not particularly useful for point geometries most of the time as a result.}

\item{key_labels}{Optional replacement contents for the labels. May be one
of:
\itemize{
\item NULL, in which case group values are used as labels (the default
behavior);
\item A labelling function;
\item a named character vector of the form
c("old_group_name" = "New label"); or
\item an unnamed character vector with one label per group. Unnamed
labels will be assigned to groups in alphanumeric order.
}}

\item{facet_vars}{Optional character vector of max length 2 (or else NULL)
giving one or two faceting variables you're using to facet your plot. When
supplied, label locations are calculated separately within each
group-by-facet combination. This is useful when adding direct labels to
faceted plots.}

\item{...}{Additional arguments passed along to
\code{\link[ggrepel:geom_text_repel]{ggrepel::geom_label_repel()}}.}
}
\value{
A ggplot2 layer produced by \code{\link[ggrepel:geom_text_repel]{ggrepel::geom_label_repel()}}.
}
\description{
\code{\link[=direct_labels_plus]{direct_labels_plus()}} adds procedurally placed text labels to grouped
point or line plots as an alternative to using a legend, which might be
space-inefficient, off to one side away from where readers will see it,
or force readers to jump their focus long distances to align groups with
labels. The function is intended as a friendlier alternative to
manually placing group labels via \code{\link[ggplot2:annotate]{ggplot2::annotate()}}. Labels are drawn
using \code{\link[ggrepel:geom_text_repel]{ggrepel::geom_label_repel()}}, so they'll repel one another as well
as the plotted data they're labeling (within reason).
}
\details{
This function is experimental. It currently works best for ordinary
scatterplots and grouped line/path plots where each group has a visually
meaningful position in two-dimensional (x/y) space. It will not
work for every ggplot2 geometry, statistic, coordinate system, or faceting
arrangement.

For point geometries, direct_labels_plus() calculates one label location
per group by finding the observed point closest to a group-specific target
positioned towards one of the "edges" of a group's cluster of points.
For "top" and "bottom" placement, the target is horizontally centered on
the group's median x-value and vertically placed near the group's maximum or
minimum y-value. For "left" and "right" placement, the same logic is
applied with x and y reversed.

For line geometries, labels are placed at the group-specific endpoint or
extreme value implied by placement: the largest x-value for "right", the
smallest x-value for "left", the largest y-value for "top", and the
smallest y-value for "bottom". The final label anchor may then be adjusted
by adj_fact.

To help labels repel from plotted points or lines, the function silently adds
empty-label rows at the original data coordinates. ggrepel does not draw
these empty labels, but still uses their positions when placing visible
labels. This helps to ensure, in general, that labels neither cover each other
nor the underlying data they're labelling.

Currently, the function sets defaults for the following parameters of
\code{\link[ggrepel:geom_text_repel]{ggrepel::geom_label_repel()}}: size (5), box.padding (0.5), max.overlaps
(Inf), segment.size (1), and min.segment.length (0). However, these are
overridable, if the user provides named arguments that at least partially
match.

Known limitations:
\itemize{
\item Label locations are calculated in the raw data space supplied to the
function. Transformed scales, reversed axes, and non-Cartesian coordinate
systems may give unexpected results. Pre-transform data rather than entering
raw data and transforming later.
\item coord_flip(), coord_polar(), map projections, and other coordinate
transformations are not currently supported.
\item The function places labels according to the data values supplied in
\code{data}, not those subsequently generated by ggplot2 stat functions. For
example, to label fitted smooth lines like those from
\code{\link[ggplot2:geom_smooth]{ggplot2::geom_smooth()}}, feed this function fitted values outputted from
such a function instead of the raw data.
\item Very dense plots or plots with many groups will likely still have
overlapping or poorly placed labels. ggrepel helps, but it cannot make a
crowded plot uncrowded. Alas.
\item Polygon, ribbon, area, segment, curve, and spatial geometries are not
currently supported but might be in the future.
}
}
\examples{
ggplot2::ggplot(iris, ggplot2::aes(Sepal.Length, Sepal.Width, color = Species)) +
ggplot2::geom_point() +
direct_labels_plus(
data = iris,
x = Sepal.Length,
y = Sepal.Width,
group = Species,
placement = "right",
geometry = "point"
) +
ggplot2::guides(color = "none")

line_data = ChickWeight |>
dplyr::group_by(Diet, Time) |>
dplyr::summarize(weight = mean(weight), .groups = "drop")

ggplot2::ggplot(line_data, ggplot2::aes(Time, weight, color = Diet)) +
ggplot2::geom_line(ggplot2::aes(group = Diet)) +
direct_labels_plus(
data = line_data,
x = Time,
y = weight,
group = Diet,
placement = "right",
geometry = "line",
adj_fact = 0.05
) +
ggplot2::guides(color = "none")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UserFacingCoreFunctions.R
\name{add_shape_plus}
\alias{add_shape_plus}
\title{Add a custom shape for geom_point_plus()}
\usage{
add_shape_plus(name = NULL, shape = NULL, overwrite = FALSE, ...)
}
\arguments{
\item{name}{A single character string giving the name of the new shape.}

\item{shape}{A data frame with columns \code{x}, \code{y}, and \code{piece}.}

\item{overwrite}{Logical. If \code{FALSE}, the default, an error is returned
when \code{name} already exists in the shape registry. If \code{TRUE}, the
existing shape is replaced with the newly provided shape. Useful if, e.g.,
you'd like to scale an existing shape up or down in size.}

\item{...}{Additional arguments. Partial argument matching is supported for
friendly UX.}
}
\value{
Invisibly returns the registered shape name.
}
\description{
Registers a custom point shape for use with \code{geom_point_plus()}.
Custom shapes are defined by a data frame of connected polygon coordinates
and, once validated, are stored in a session-level shape registry.
}
\details{
A shape must be supplied as a data frame with columns \code{x}, \code{y},
and \code{piece}. The \code{x} and \code{y} columns define the vertices of
the shape, centered around \code{0, 0}. Coordinates should generally be
scaled to about \code{+/-0.4} to match the scaling of built-in point shapes.
The \code{piece} column identifies separate polygon pieces within the same
shape to generate "holes," as appropriate.

For inspiration and to model inputs, see \code{ggplotplus_shapes_list} for the
structure of the built-in point shapes.
}
\examples{
test_star = data.frame(
  x = c(0.000,  0.118,  0.380,  0.190,  0.235,
        0.000, -0.235, -0.190, -0.380, -0.118),
  y = c(0.400,  0.124,  0.124, -0.047, -0.324,
       -0.153, -0.324, -0.047,  0.124,  0.124),
  piece = 1
)

add_shape_plus("test_star", test_star)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.plot_data_names}
\alias{.plot_data_names}
\title{Collect column names from plot- and layer-level data}
\usage{
.plot_data_names(plot)
}
\arguments{
\item{plot}{A ggplot object.}
}
\value{
A character vector of unique column names found across the plot-level
and layer-level data frames associated with the plot.
}
\description{
Internal helper that gathers column names from all data frames associated
with a ggplot object, including both plot-level data and any layer-specific
data.
}
\details{
ggplotplus uses this helper when evaluating whether scale titles appear to
still use raw column names, which may indicate that user-facing labels have
not yet been customized with \code{ggplot2::labs()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UserFacingCoreFunctions.R
\name{scale_continuous_plus}
\alias{scale_continuous_plus}
\title{Continuous Scales With Endpoint-Aware Breaks}
\usage{
scale_continuous_plus(
  scale = NA,
  ...,
  thin.labels = FALSE,
  pad.labels = "start",
  target.breaks = 5,
  buffer_frac = 0.05,
  split_name = FALSE
)
}
\arguments{
\item{scale}{Character string specifying which scale to modify. Options are
\code{"x"}, \code{"y"}, \code{"colour"}/\code{"color"}, and \code{"fill"}.}

\item{...}{Additional arguments passed to the corresponding ggplot2
continuous scale function. Arguments such as \code{name}, \code{labels}, \code{guide},
\code{position}, and \code{expand} may be supplied. User-supplied \code{breaks}, \code{limits},
\code{n.breaks}, \code{trans}, and \code{transform} are ignored with a warning because
this function controls those components directly.}

\item{thin.labels}{Logical. If \code{TRUE}, every other break label is blanked
to reduce crowding. Defaults to \code{FALSE}.}

\item{pad.labels}{Character string, either \code{"start"} or \code{"end"}. Used when a
user-supplied label vector is shorter than the internally computed break
vector by one label (which must be assessed via trial and error currently). \code{"start"} pads a blank label at the beginning;
\code{"end"} pads one at the end.}

\item{target.breaks}{Integer target number of major breaks. This is a target and
not a guarantee because breaks are chosen using a "pretty" break algorithm.
Default is \code{5}.}

\item{buffer_frac}{Numeric fraction of the data span used to decide whether a
break is close enough to each endpoint. Default is \code{0.05}.}

\item{split_name}{Logical. If \code{TRUE}, spaces in a named \code{name} argument are
replaced with line breaks. This can help long axis or legend titles fit
better. Default is \code{FALSE}.}
}
\value{
A ggplot2 continuous scale object.
}
\description{
\code{scale_continuous_plus()} is an opinionated wrapper around ggplot2's
continuous x, y, colour, and fill scales. It chooses "pretty" breaks while
gently expanding the scale limits so breaks generally will appear near both
ends of the data range.
}
\details{
This is useful because ggplot2's default continuous scales frequently will leave the ends of an
axis or colorbar visually unlabeled, making it look as if an endpoint break is
missing.

\code{scale_continuous_plus()} routes to one of
\code{\link[ggplot2:scale_continuous]{ggplot2::scale_x_continuous()}}, \code{\link[ggplot2:scale_continuous]{ggplot2::scale_y_continuous()}},
\code{\link[ggplot2:scale_colour_continuous]{ggplot2::scale_colour_continuous()}}, or
\code{\link[ggplot2:scale_colour_continuous]{ggplot2::scale_fill_continuous()}} based on \code{scale}.

Unlike the ggplot2 defaults, this function intentionally controls \code{breaks}
and \code{limits}. If either is supplied through \code{...}, it's ignored with a
warning. Transformed scales are also not currently supported; pre-transform
the data or use ggplot2's scale functions directly when a transformed scale is
needed.

User-supplied label vectors are supported, but endpoint-aware breaks can
sometimes create hidden outer breaks. When possible, this function pads label
vectors with blank labels to align them with the computed break vector in length. If
alignment is ambiguous by one label, use \code{pad.labels} to choose which side of the input labels vector to
pad.
}
\examples{
library(ggplot2)

ggplot(iris, aes(Sepal.Width, Sepal.Length)) +
  geom_point() +
  scale_continuous_plus(scale = "x") +
  scale_continuous_plus(scale = "y")

ggplot(iris, aes(Sepal.Width, Sepal.Length, fill = Petal.Length)) +
  geom_point(shape = 21) +
  scale_continuous_plus(scale = "x") +
  scale_continuous_plus(scale = "y") +
  scale_continuous_plus(scale = "fill")

ggplot(iris, aes(Sepal.Width, Sepal.Length)) +
  geom_point() +
  scale_continuous_plus(
    scale = "y",
    name = "Sepal length",
    labels = LETTERS[1:5],
    pad.labels = "start"
  )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.has_plot_guide_none_for_aes}
\alias{.has_plot_guide_none_for_aes}
\title{Check whether plot-level guides suppress an aesthetic}
\usage{
.has_plot_guide_none_for_aes(plot, aes_name)
}
\arguments{
\item{plot}{A ggplot object.}

\item{aes_name}{A single character string giving the aesthetic name to check.}
}
\value{
A logical scalar. Returns \code{TRUE} if the plot-level guide for the
aesthetic is \code{"none"} or a \code{GuideNone} object.
}
\description{
Internal helper that checks whether a plot-level guide specification suppresses
the guide for a given aesthetic, such as through
\code{guides(fill = "none")}.
}
\details{
This complements the scale-level guide checker above, because guide
suppression can be declared either on a scale or through \code{guides()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.get_visible_panel_ranges}
\alias{.get_visible_panel_ranges}
\title{Get visible panel ranges for an axis}
\usage{
.get_visible_panel_ranges(built, axis)
}
\arguments{
\item{built}{A built ggplot object, typically produced by
ggplot2::ggplot_build().}

\item{axis}{A character string giving the axis to inspect. Expected values are
"x" or "y".}
}
\value{
A list of numeric vectors, one per panel. Each vector gives the
visible range for the requested axis.
}
\description{
Internal helper used by coaching checks to retrieve the visible coordinate
range for each panel along a requested axis. This differs from the trained
scale range: for example, coord_cartesian() may crop the visible panel range
without dropping data from the scale.
}
\details{
This helper relies on ggplot2's built-plot internals and should therefore be
treated as best-effort/version-sensitive.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UserFacingCoreFunctions.R
\name{yaxis_title_plus}
\alias{yaxis_title_plus}
\title{Relocate a Y Axis Title to Above the Y Axis on a Ggplot and Turn it Horizontal.}
\usage{
yaxis_title_plus(
  location = "top",
  nudgeTopLegendDown = FALSE,
  nudgeHowMuch = 20,
  override_legend_alphasize = TRUE,
  enable_coaching = TRUE
)
}
\arguments{
\item{location}{A length-1 character string matching either "top" or "bottom" for the placement of the new y axis title. Defaults to \code{"top"}. \code{"bottom"} should generally only be used when the x axis labels have been moved to the top of the graph (uncommon).}

\item{nudgeTopLegendDown}{A length-1 logical indicating whether a top legend (box) (if any) should be moved down to align with the relocated y axis title (where they could clip into each other). Defaults to FALSE.}

\item{nudgeHowMuch}{A length-1 positive integer indicating how much to nudge the top legend (box) (if any) down, if \code{nudgeTopLegendDown} == \code{TRUE}. Defaults to \code{20} points as a general guess and may need adjusting.}

\item{override_legend_alphasize}{Logical indicating whether, for any fill, color, and/or shape legends, to override the size and/or alpha values to the ggplotplus default values (1 and 5, respectively). Defaults to TRUE.}

\item{enable_coaching}{Logical indicating whether ggplotplus should perform certain automated checks for possible non-Universal graph design and provide coaching messages when these are detected. Defaults to TRUE.}
}
\value{
An ggplot class object for adding to a plot with \code{+}.
}
\description{
This function relocates the y axis title of a ggplot to the top, above the y axis line and left-justified to the left edge of the y axis labels, sort of like a plot subtitle. It also orients the text horizontally for space-efficiency and easy reading. This is otherwise difficult to do using \code{ggplot2}'s default styling tools.
}
\examples{
#WE DO NOT RECOMMEND USING yaxis_title_plus() WITHOUT theme_plus()
ggplot2::ggplot(iris, ggplot2::aes(x=Sepal.Length, y=Petal.Length)) +
ggplot2::geom_point() +
theme_plus() +
yaxis_title_plus()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.ggplotplus_coaching_enabled}
\alias{.ggplotplus_coaching_enabled}
\title{Check whether ggplotplus coaching messages are enabled}
\usage{
.ggplotplus_coaching_enabled()
}
\value{
A logical scalar. Returns \code{TRUE} if coaching messages are
globally enabled for the current R session; otherwise returns
\code{FALSE}.
}
\description{
Internal helper that checks the session-level ggplotplus coaching option.
}
\details{
ggplotplus uses this helper to determine whether advisory or educational
messages ("coaching") should be displayed during plot construction. Coaching
messages can be globally enabled or disabled for the current R session using:

\preformatted{
options(ggplotplus.enable_coaching = TRUE)
options(ggplotplus.enable_coaching = FALSE)
}

User-facing ggplotplus functions additionally expose local
\code{enable_coaching} arguments; both the local setting and the global
option must evaluate to \code{TRUE} for coaching messages to appear.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_point_plus.R
\docType{data}
\name{GeomPointPlus}
\alias{GeomPointPlus}
\title{An alternative version of \code{ggplot2}'s geomPoint proto that incorporates new, distinctive shapes.}
\format{
An object of class \code{PointPlus} (inherits from \code{GeomPoint}, \code{Geom}, \code{ggproto}, \code{gg}) of length 3.
}
\usage{
GeomPointPlus
}
\value{
A \code{ggplot2} ggproto subclass object.
}
\description{
This ggplot proto object is called internally by \code{geom_point2()} and inherits most, but not all, of its methods and properties from \code{ggplot2}'s standard \code{geomPoint} proto. However, it has different default aesthetics, a different shapes palette, and can draw these new shapes in a legend. This subclass is not meant to be encountered by the user and is instead fodder for \code{geom_point_plus()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.aes_mapped_var_is_continuous}
\alias{.aes_mapped_var_is_continuous}
\title{Check whether an aesthetic is mapped to a continuous (numeric) variable}
\usage{
.aes_mapped_var_is_continuous(plot, aes_name)
}
\arguments{
\item{plot}{A ggplot object.}

\item{aes_name}{A single character string giving the aesthetic to check.}
}
\value{
A logical scalar. Returns \code{TRUE} if the aesthetic appears to be
mapped to a continuous variable; otherwise returns \code{FALSE}.
}
\description{
Internal helper that determines whether a given aesthetic is mapped to a
continuous variable. Used to guard legend override operations that are
appropriate only for discrete legends (not colorbars), because applying a
\code{guide_legend()} to a continuous scale would suppress the colorbar.
}
\details{
The helper first checks for an explicit continuous scale on the plot. If
none is found, it inspects the mapped expression and looks up the variable
in the plot and layer data to determine whether it's numeric.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.is_between}
\alias{.is_between}
\title{Check whether a value falls inside a range}
\usage{
.is_between(range, val, inclusive = TRUE)
}
\arguments{
\item{range}{A numeric vector of length 2 giving the range to check.}

\item{val}{A numeric value to test against range.}

\item{inclusive}{Logical. If TRUE, values equal to either range endpoint are
treated as inside the range. If FALSE, values must fall strictly between
the endpoints.}
}
\value{
A single logical value.
}
\description{
Internal helper used by coaching checks to determine whether a target value
falls within a numeric range. The input range is sorted before comparison, so
the function is insensitive to whether the range is supplied as low-high or
high-low.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_point_plus.R
\name{geom_point2}
\alias{geom_point2}
\title{A backend version of \code{geom_point()} that can access distinctive shapes.}
\usage{
geom_point2(
  mapping = NULL,
  data = NULL,
  stat = "identity",
  position = "identity",
  shapes = NULL,
  ...,
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by aes(), as in \code{ggplot2::geom_point()}.}

\item{data}{The data to be displayed in this layer, as in \code{ggplot2::geom_point()}.}

\item{stat}{The statistical transformation to use on the data for this layer, as in \code{ggplot2::geom_point()}.}

\item{position}{A position adjustment to use on the data for this layer, as in \code{ggplot2::geom_point()}.}

\item{shapes}{A named list of custom shapes to be drawn in place of \code{ggplot2}'s standard palette of shapes.}

\item{...}{Other arguments passed on to layer()'s params argument, as in \code{ggplot2::geom_point()}.}

\item{na.rm}{Logical controlling whether missing values should be removed from the data with a warning or silently, as in \code{ggplot2::geom_point()}.}

\item{show.legend}{Logical controlling whether this layer should be included in the legend(s), as in \code{ggplot2::geom_point()}.}

\item{inherit.aes}{Logical controlling whether global aesthetics specified in \code{ggplot2::ggplot()} should be inherited locally by this layer or not, as in \code{ggplot2::geom_point()}.}
}
\value{
A ggplot2 layer object.
}
\description{
This function behaves much like \code{ggplot2}'s \code{geom_point()} function except that it allows access to a palette of nine new shapes that vary in their openness, spikiness, and intersectionality, making them more easily distinguished. This function is not meant to be called directly--instead, \code{geom_point_plus()} calls and modifies this function and is the intended function for users.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggplot.plus-package.R
\docType{package}
\name{ggplotplus-package}
\alias{ggplotplus}
\alias{ggplotplus-package}
\title{ggplotplus: Universal Design-Oriented Enhancements for 'ggplot2'}
\description{
A collection of enhancements to 'ggplot2', with a focus on creating Universally Designed, accessible graphs easily and quickly.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/MAISRC/ggplotplus}
  \item Report bugs at \url{https://github.com/MAISRC/ggplotplus/issues}
}

}
\author{
\strong{Maintainer}: Alex Bajcz \email{bajcz003@umn.edu} (\href{https://orcid.org/0000-0002-5909-4676}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UserFacingCoreFunctions.R
\name{gridlines_plus}
\alias{gridlines_plus}
\title{Subtle, Minimal Gridlines For When and Where They Help}
\usage{
gridlines_plus(
  color = "gray90",
  linewidth = 1.2,
  linetype = "solid",
  notx = FALSE,
  noty = FALSE,
  override_legend_alphasize = TRUE,
  enable_coaching = TRUE
)
}
\arguments{
\item{color}{Gridline color. Single character string. Default: \code{"gray90"}.}

\item{linewidth}{Gridline width (theme line units). Single numeric. Default: \code{1.2}.}

\item{linetype}{Gridline type. Single string (e.g., \code{"solid"}, \code{"dashed"}).}

\item{notx, noty}{Logicals indicating whether gridlines should not be drawn in a specific direction (i.e., the user wants those to be blank even when the axis is continuous). Default to FALSE (gridlines are added).}

\item{override_legend_alphasize}{Logical indicating whether, for any fill, color, and/or shape legends, to override the size and/or alpha values to the ggplotplus default values (1 and 5, respectively). Defaults to TRUE.}

\item{enable_coaching}{Logical indicating whether ggplotplus should perform certain automated checks for possible non-Universal graph design and provide coaching messages when these are detected. Defaults to TRUE.}
}
\value{
An ggplot class object for adding to a plot with \code{+}.
}
\description{
Adds light and easily ignored major gridlines along only axes mapped to
continuous variables. Minor gridlines are
blanked. This enables the benefits of gridlines (in instances where there are some) but minimizes visual clutter and cognitive load.
}
\details{
\code{gridlines_plus()} ignores any theme instructions to blank major panel scales in either the x or y direction via, e.g., \code{theme_plus(panel.grid.major.y = ggplot2::element_blank())}. Instead, users can set, e.g., noty == TRUE to prevent gridlines from being drawn in a specific direction, even if that scale is continuous. Similarly, gridline color, linewidth, and linetypes should be set directly in \code{gridlines_plus()} instead of via \code{theme()}. Trying to do the latter will fail without error or warning!

Under the hood, \code{gridlines_plus()} checks layer and/or global mappings to
see if \code{x} and/or \code{y} are continuous. It does this by inspecting the trained panel scales.
It then turns on \strong{major} gridlines for continuous directions and explicitly
blanks gridlines for other axes (as well as \strong{all minor} gridlines).
}
\examples{
library(ggplot2)

ggplot2::ggplot(iris, ggplot2::aes(Sepal.Length, Petal.Length)) +
  ggplot2::geom_point() +
  theme_plus() + #WE DON'T RECOMMEND USING gridlines_plus() WITHOUT ALSO USING theme_plus()
  gridlines_plus()

# Only y is continuous here (x is discrete) → y-only major gridlines
ggplot2::ggplot(mtcars, ggplot2::aes(factor(cyl), mpg)) +
  ggplot2::geom_boxplot() +
  gridlines_plus(color = "grey85", linewidth = 1, linetype = "dashed")

# Works with derived continuous axes (histogram)
ggplot2::ggplot(mtcars, aes(mpg)) +
  ggplot2::geom_histogram() +
  gridlines_plus()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.validate_pointplus_shape}
\alias{.validate_pointplus_shape}
\title{Validate a geom_point_plus() shape definition}
\usage{
.validate_pointplus_shape(shape, name = NULL)
}
\arguments{
\item{shape}{A proposed shape definition. Must be a data frame with columns
\code{x}, \code{y}, and \code{piece}.}

\item{name}{Shape name, used by callers.}
}
\value{
A cleaned shape data frame containing only \code{x}, \code{y}, and
\code{piece}. The \code{piece} column is converted to sequential integers if
it wasn't already formatted like that.
}
\description{
Internal helper used by \code{add_shape_plus()} to check that a custom
point shape has the structure needed by the ggplotplus shape-drawing
machinery.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UserFacingCoreFunctions.R
\name{geom_point_plus}
\alias{geom_point_plus}
\title{Create and add a scatterplot layer to your \code{ggplot2} graph with new, distinctive shapes.}
\usage{
geom_point_plus(
  mapping = NULL,
  data = NULL,
  stat = "identity",
  position = "identity",
  avail_shapes = NULL,
  n_shapes = length(avail_shapes),
  chosen_shapes = NULL,
  legend_title = NULL,
  legend_labels = NULL,
  include_shape_legend = TRUE,
  ...,
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE,
  show_shape_scale = TRUE
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by aes(), as in \code{ggplot2::geom_point()}.}

\item{data}{The data to be displayed in this layer, as in \code{ggplot2::geom_point()}.}

\item{stat}{The statistical transformation to use on the data for this layer, as in \code{ggplot2::geom_point()}.}

\item{position}{A position adjustment to use on the data for this layer, as in \code{ggplot2::geom_point()}.}

\item{avail_shapes}{A named list of custom shapes to be drawn in place of \code{ggplot2}'s standard palette of shapes. Defaults to \code{NULL} and is replaced internally with the palette of shapes designed specifically for use in \code{geom_point_plus()}. This should probably not be changed unless users have created new shapes they would like to use instead.}

\item{n_shapes}{A length-1 integer corresponding to the number of distinct shapes the function is allowed to pull from the shapes palette specified to \code{avail_shapes}. Defaults to the length of \code{avail_shapes} and should probably not be changed.}

\item{chosen_shapes}{A character string referring by name to elements in the current shapes registry that the function should use to allocate shapes to values, e.g. \verb{c("flower", "octagon", "squircle)}. These are provided internally to a \code{scale_shape_manual()} call and are meant to circumvent the need for such a call to specify a specific subset of shapes to be used from the new shapes palette. Defaults to \code{NULL}, i.e., shapes are pulled from \code{shapes.list} in order. Numerical values will use \code{ggplot2}'s default shapes instead.}

\item{legend_title}{A length-1 character string corresponding to the name to be used for the shape legend title (if any). This is passed internally to \code{scale_shape_manual()} and is meant to help circumvent the need for the user to specify any such call directly.}

\item{legend_labels}{A character vector corresponding to the names to be used for the shape legend labels (if any). This is passed internally to \code{scale_shape_manual()} and is meant to help circumvent the need for the user to specify any such call directly.}

\item{include_shape_legend}{Logical indicating whether a shape legend will be shown (one is always shown unless this is set to FALSE, even when shape is being mapped to a constant and thus a legend may not be appropriate).}

\item{...}{Other arguments passed on to this layer()'s params argument, as in \code{ggplot2::geom_point()}.}

\item{na.rm}{Logical value controlling whether missing values should be removed from the data with a warning or silently, as in \code{ggplot2::geom_point()}.}

\item{show.legend}{Logical value controlling whether this layer should be included in the legend(s), as in \code{ggplot2::geom_point()}.}

\item{inherit.aes}{Logical controlling whether global aesthetics specified in \code{ggplot2::ggplot()} should be inherited locally by this layer or not, as in \code{ggplot2::geom_point()}.}

\item{show_shape_scale}{Logical controlling whether a call to \code{ggplot2::shape_scale_manual()} should be included as part of the function's operations. Generally, this should be set to \code{TRUE} unless shape is being mapped to a constant, in which case leaving this \code{TRUE} would trigger a legend that is redundant.}
}
\value{
A ggplot2 layer object.
}
\description{
This function behaves similarly to \code{ggplot2::geom_point()} except that it takes several new inputs: \code{shapes}, \code{n_shapes}, \code{shape_values}, \code{legend_title}, \code{key_size}, and \code{show_shape_scale}. These are explained below.
}
\details{
Collectively, these inputs allow \code{geom_point_plus()} to access and draw several new and distinctive shapes that are designed to be more readily distinguishable from one another when shape communicates difference.

To see the special shapes available via this function run \code{geom_point_plus_shapes()}.

Note: As of Version 0.5.2, shapes 21-25 in R's default shapes palette are now also available via \code{geom_point_plus_shapes()}; these are called "circle", "square", "diamond", "triangle_up", and "triangle_down", respectively, though they can also be referred to by number.
}
\examples{
ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg, fill = drat)) +
geom_point_plus(ggplot2::aes(shape = factor(gear)), size = 5)
ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg, fill = factor(cyl))) +
geom_point_plus(ggplot2::aes(shape = factor(carb)),
shape_values = c("squircle", "lotus", "sunburst", "octagon", "cross", "oval"),
size = 5, stroke = 0.4)
ggplot2::ggplot(iris, ggplot2::aes(Petal.Width, Petal.Length, fill = Species)) +
geom_point_plus(ggplot2::aes(shape = Species), size = 5, alpha = 0.7)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/onloads.R
\name{.onLoad}
\alias{.onLoad}
\title{geom_point_plus() shape registry}
\usage{
.onLoad(libname, pkgname)
}
\description{
Package-private environment used to store built-in and user-registered
point shapes for \code{geom_point_plus()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.standardize_pointplus_shape_names}
\alias{.standardize_pointplus_shape_names}
\title{Standardize ggplotplus shape names}
\usage{
.standardize_pointplus_shape_names(shape)
}
\arguments{
\item{shape}{A vector of shape identifiers. May be numeric,
character, or factor.}
}
\value{
A character vector of standardized shape names.
}
\description{
Internal helper to normalize shape identifiers used by
\code{geom_point_plus()}. Converts numeric shape codes
corresponding to base R's fillable point shapes (21--25)
into their ggplotplus string equivalents.
}
\details{
This ensures consistent downstream handling regardless of
whether users supply shapes as numbers (e.g., \code{21}) or
names (e.g., \code{"circle"}).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.has_any_barCol_geom}
\alias{.has_any_barCol_geom}
\title{Check whether a built plot contains a bar-like geom}
\usage{
.has_any_barCol_geom(built)
}
\arguments{
\item{built}{A built ggplot object, typically produced by
ggplot2::ggplot_build().}
}
\value{
A single logical value.
}
\description{
Internal helper used by coaching checks to determine whether any layer in a
built plot uses GeomBar. This catches both geom_bar() and geom_col(),
since geom_col() uses the same underlying geom with a different stat.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UserFacingCoreFunctions.R
\name{scale_focus_plus}
\alias{scale_focus_plus}
\title{Turn on a "focus mode" to highlight specific groups via color or fill}
\usage{
scale_focus_plus(
  aes,
  group_var,
  focal_groups,
  diff_nonfocal = FALSE,
  diff_focal = TRUE,
  gray_start = 0.35,
  gray_end = 0.65,
  focal_start = 0.75,
  focal_end = 0.25,
  custom_focal = NULL,
  custom_nonfocal = NULL,
  ...
)
}
\arguments{
\item{aes}{A character string indicating which aesthetic should receive the
focus scale. Must be one of \code{"color"}, \code{"colour"}, or \code{"fill"}.}

\item{group_var}{A character vector or factor containing the grouping variable
mapped to \code{aes}. This should generally be the same vector mapped to \code{colour},
\code{color}, or \code{fill} in the plot.}

\item{focal_groups}{A character vector giving the group value(s) in the
\code{group_var} to be emphasized. All values must be present in \code{group_var}.}

\item{diff_nonfocal}{Logical. If \code{TRUE}, non-focal groups are assigned
different (gray/custom) color values. If \code{FALSE}, all non-focal groups
receive the same (gray/custom) value. Defaults to \code{FALSE}.}

\item{diff_focal}{Logical. If \code{TRUE}, focal groups are assigned different
(viridis/custom) colors. If \code{FALSE}, all focal groups receive the same
(viridis/custom) color. Defaults to \code{TRUE}.}

\item{gray_start, gray_end}{Numeric values between 0 and 1 controlling the
range of gray values used for non-focal groups when custom_nonfocal is not
NULL. Lower values are darker and higher values are lighter. \code{gray_start}
must be less than or equal to \code{gray_end}.}

\item{focal_start, focal_end}{Numeric values between 0 and 1 controlling the
portions of the viridis palette used for focal groups when custom_focal is
not NULL. By default, \code{scale_focus_plus()} draws from the darker and lighter
ends of the palette while avoiding the middle, as humans are generally more
drawn to very bright and very dark colors. \code{focal_end} must be less than
or equal to \code{focal_start}.}

\item{custom_focal}{Optional custom color(s) for focal group(s). When
\code{diff_focal = TRUE}, this must be a named character vector whose names
match the unique values in \code{focal_groups}. When \code{diff_focal = FALSE},
this must be a character vector of length 1.}

\item{custom_nonfocal}{Optional custom color(s) for non-focal group(s). When
\code{diff_nonfocal = TRUE}, this must be a named character vector whose names
match the non-focal values in \code{group_var}. When \code{diff_nonfocal = FALSE},
this must be a character vector of length 1.}

\item{...}{Additional arguments passed to
\code{\link[ggplot2:scale_manual]{ggplot2::scale_colour_manual()}} or \code{\link[ggplot2:scale_manual]{ggplot2::scale_fill_manual()}}. Do not
supply \code{values}; \code{scale_focus_plus()} constructs the \code{values} argument
internally. All other arguments should be accessible, including \code{labels}
and \code{name}. Use these to relabel and retitle the created scale.}
}
\value{
A ggplot2 scale object.
}
\description{
\code{scale_focus_plus()} is a wrapper for \code{ggplot2}'s \code{scale_fill/colour_manual}
functions that helps a user quickly creates a manual color or fill scale
that highlights one or more focal groups while de-emphasizing all other
groups (via desaturation, by default). This is useful when a graph would
ideally call attention to specific groups without removing the broader
context and transparency provided by the remaining groups. This reduces
cognitive load by allowing non-essential information to "fall into the
background" to be readily ignored, increasing the reading rate, and also
helps to signpost for the reader where the "message" is within the plot.
}
\details{
By default, focal groups are assigned visually prominent colors from the
Universal-Design-oriented viridis color palette via \code{viridisLite::viridis()},
while non-focal groups are assigned a shared gray. Focal groups are
differentiated from one another by default; non-focal groups are not. These
defaults are intended to support a common graph-design pattern:
show all groups, but make the intended comparison obvious.

Note that the default values for \code{gray_start}, \code{gray_end}, \code{focal_start}, and
\code{focal_end} intentionally map to different regions of the luminance (light
to dark) scale so as to render the colors still distinguishable in grayscale
by virtue of variance in luminance. This variance should be maintained for
accessibility even if different inputs are provided.

\code{scale_focus_plus()} is a convenience wrapper around
\code{\link[ggplot2:scale_manual]{ggplot2::scale_colour_manual()}} and \code{\link[ggplot2:scale_manual]{ggplot2::scale_fill_manual()}}. It does
not alter the data or add any geoms. Instead, it constructs a named vector of
colors from \code{group_var} and \code{focal_groups}, then passes that vector to the
appropriate ggplot2 manual color/fill scale.

This function is most useful when the focal/non-focal distinction is the
primary visual message, and all non-focal groups may be secondary. When
individual non-focal groups must remain distinguishable, set
\code{diff_nonfocal = TRUE} or provide \code{custom_nonfocal}.

Although \code{scale_focus_plus()} is designed for discrete data, it can also be
used when the data requiring (de-)emphasis is continuous. In those cases,
first create a discrete grouping variable in the data, then map that variable
to \code{colour} or \code{fill}. For example, a continuous measurement could be
converted to groups such as \code{"High cover"} and \code{"Other"} before calling
\code{scale_focus_plus()}. See the examples section for an example.
}
\examples{
lake_dat = data.frame(
  year = rep(2012:2020, times = 5),
  lake = rep(paste("Lake", LETTERS[1:5]), each = 9),
  cpue = c(
    10, 12, 13, 14, 16, 18, 19, 21, 23,
    18, 17, 16, 16, 15, 14, 13, 13, 12,
    8, 9, 11, 13, 16, 20, 24, 27, 30,
    22, 21, 21, 20, 19, 18, 18, 17, 17,
    12, 13, 13, 15, 15, 16, 17, 18, 20
  )
)

ggplot2::ggplot(
  lake_dat,
  ggplot2::aes(x = year,
               y = cpue,
               colour = lake,
               group = lake)
) +
  ggplot2::geom_line(linewidth = 1) +
  ggplot2::geom_point(size = 2) +
  scale_focus_plus(aes = "colour",
             group_var = lake_dat$lake,
             focal_groups = c("Lake A", "Lake C"))

# One can use a shared focal color and a custom non-focal gray.
ggplot2::ggplot(
  lake_dat,
  ggplot2::aes(x = year,
               y = cpue,
               colour = lake,
               group = lake)
) +
  ggplot2::geom_line(linewidth = 1) +
  scale_focus_plus(aes = "colour",
             group_var = lake_dat$lake,
             focal_groups = c("Lake A", "Lake C"),
             diff_focal = FALSE,
             custom_focal = "#440154",
             custom_nonfocal = "gray70")

cover_dat = data.frame(
  taxon = c("Native plants",
            "Starry stonewort",
            "Eurasian watermilfoil",
            "Curly-leaf pondweed"),
  mean_cover = c(62, 28, 18, 11)
)

ggplot2::ggplot(
  cover_dat,
  ggplot2::aes(x = taxon,
               y = mean_cover,
               fill = taxon)
) +
  ggplot2::geom_col() +
  scale_focus_plus(aes = "fill",
             group_var = cover_dat$taxon,
             focal_groups = "Starry stonewort") +
  ggplot2::labs(x = NULL,
                y = "Mean percent cover")

# Focal groups can also be created from continuous variables. Here's how:
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UserFacingCoreFunctions.R
\name{ggplotplus_to_patchwork}
\alias{ggplotplus_to_patchwork}
\title{Convert a ggplotplus plot into a patchwork-compatible element}
\usage{
ggplotplus_to_patchwork(plot)
}
\arguments{
\item{plot}{A ggplot or ggplotplus plot object.}
}
\value{
A patchwork-compatible wrapped plot element.
}
\description{
Convenience helper for converting ggplotplus plot objects into
patchwork-compatible wrapped elements.
}
\details{
This helper is primarily intended for compatibility with \pkg{patchwork},
whose plot-composition operators may not yet fully recognize custom S7
plot subclasses under ggplot2 4.x.

Internally, the plot is first converted into a grob using
\code{ggplotplus_to_cowplot()}, then wrapped using
\code{patchwork::wrap_elements()}.

This helper requires the \pkg{patchwork} package, but ggplotplus does not
import patchwork. Users only need patchwork installed if they want to compose
ggplotplus plots with patchwork.
}
\examples{
if(requireNamespace("patchwork", quietly = TRUE)) {

p1 = ggplot2::ggplot(iris, ggplot2::aes(Sepal.Length, Petal.Length, colour = Species)) +
  geom_point_plus()

p2 = ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg, shape = factor(cyl))) +
  geom_point_plus()

(ggplotplus_to_patchwork(p1) |
  ggplotplus_to_patchwork(p2)) +
  patchwork::plot_annotation(
    title = "ggplotplus + patchwork"
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.apply_key_labels_plus}
\alias{.apply_key_labels_plus}
\title{Apply replacement labels for direct labels}
\usage{
.apply_key_labels_plus(label_data, key_labels)
}
\arguments{
\item{label_data}{A character vector of labels, i.e., the .label_plus
column produced by directlabel_points() or directlabel_lines().}

\item{key_labels}{Optional replacement labels. May be one of:
\itemize{
\item NULL, in which case label_data is returned unchanged;
\item a function applied to label_data, such as a labelling function;
\item a named vector of the form c("old_label" = "New label"); or
\item an unnamed vector with one replacement label per unique group.
}}
}
\value{
A character vector of labels with the same length as label_data.
}
\description{
.apply_key_labels_plus() is an internal helper used by
\code{\link[=direct_labels_plus]{direct_labels_plus()}} to replace default group labels with user-supplied
labels.
}
\details{
Named key_labels are treated as an explicit lookup table. Every unique
value in label_data must appear among the names of key_labels; otherwise,
the helper fails with an informative error.

Unnamed key_labels are assigned to unique labels in alphanumeric order.
This behavior may be surprising, so the helper emits a message recommending
named labels when users want more control.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.plot_has_mapped_aes}
\alias{.plot_has_mapped_aes}
\title{Check whether a plot maps one or more aesthetics}
\usage{
.plot_has_mapped_aes(plot, aes_name)
}
\arguments{
\item{plot}{A ggplot object.}

\item{aes_name}{A character vector of aesthetic names to check.}
}
\value{
A logical scalar. Returns \code{TRUE} if any requested aesthetic is
mapped in any layer, including through inherited global mappings.
}
\description{
Internal helper that checks whether any layer in a ggplot object maps one
or more requested aesthetics, either directly in the layer mapping or through
inherited plot-level mappings.
}
\details{
This is used before plot building to determine whether an aesthetic such as
\code{alpha} or \code{size} is being mapped explicitly, in which case
ggplotplus should avoid overriding that aesthetic in legend keys.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.guide_is_none_for_aes}
\alias{.guide_is_none_for_aes}
\title{Check whether a guide is suppressed for an aesthetic}
\usage{
.guide_is_none_for_aes(plot, aes_name)
}
\arguments{
\item{plot}{A ggplot object.}

\item{aes_name}{A single character string giving the aesthetic name to check.}
}
\value{
A logical scalar. Returns \code{TRUE} if the guide is suppressed
either through a scale or through \code{guides()}.
}
\description{
Internal helper that checks both scale-level and plot-level guide declarations
to determine whether the guide for a given aesthetic has been explicitly
suppressed.
}
\details{
This is used before adding ggplotplus legend-key overrides so that suppressed
legends are not accidentally restored.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.has_guide_none_for_aes}
\alias{.has_guide_none_for_aes}
\title{Check whether a scale suppresses a guide}
\usage{
.has_guide_none_for_aes(plot, aes_name)
}
\arguments{
\item{plot}{A ggplot object.}

\item{aes_name}{A single character string giving the aesthetic name to check.}
}
\value{
A logical scalar. Returns \code{TRUE} if a matching scale has
\code{guide = "none"}; otherwise returns \code{FALSE}.
}
\description{
Internal helper that checks whether a plot contains a scale for a given
aesthetic with \code{guide = "none"}.
}
\details{
This is used to preserve user intent when ggplotplus applies automatic legend
key overrides. If a user has explicitly suppressed a guide through a scale,
ggplotplus should not reintroduce that guide.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/middleware.R
\name{.directlabel_points}
\alias{.directlabel_points}
\title{Choose direct-label anchor points for grouped point data}
\usage{
.directlabel_points(data, x, y, group, placement, adj_fact, facet_vars)
}
\arguments{
\item{data}{A data frame containing the variables to be plotted and labelled.}

\item{x, y}{Unquoted column names giving the x- and y-variables.}

\item{group}{Unquoted column name giving the grouping variable from which to
construct labels.}

\item{placement}{Character string giving the preferred label placement
relative to each group's cloud of points. One of "top", "right", "bottom",
or "left".}

\item{adj_fact}{A single numeric value giving the proportional adjustment
applied to the group-specific target location. For point data, this changes
the target used to choose the labelled point, but does not move the final
label anchor away from the selected observed point.}

\item{facet_vars}{Optional character vector of facet variable names. When
supplied, label anchor points are calculated separately within each
group-by-facet combination.}
}
\value{
A tibble with one row per group or group-by-facet combination. The
returned data include standardized x and y columns, .label_plus, and
any grouping or faceting variables needed by ggplot2 to assign labels to
panels correctly.
}
\description{
directlabel_points() is an internal helper used by
\code{\link[=direct_labels_plus]{direct_labels_plus()}} when geometry = "point". It chooses one observed
point per group, or per group-by-facet combination, to use as the anchor
location for a direct label. How the label is then drawn with respect to that
chosen point is determined by \code{\link[ggrepel:geom_text_repel]{ggrepel::geom_label_repel()}}.
}
\details{
The helper works by calculating a target location for each group and then
selecting the observed point closest to that target. For "top" and
"bottom" placements, the target is centered at the group's median x-value
and then shifted toward the group's maximum or minimum y-value. For "left"
and "right" placements, the same logic is applied with x and y reversed.

This helper assumes that label locations should be tied to observed points.
It therefore never invents a new x/y coordinate for point labels. Empty or
all-missing groups should be screened before this helper is called.
}
\keyword{internal}
